/*
 * (c) Copyright 2005 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.common.creator;

import java.lang.reflect.Field;
import java.text.DateFormat;
import java.util.StringTokenizer;

import org.w3c.dom.Element;

import cookxml.core.DecodeEngine;
import cookxml.core.interfaces.Creator;

/**
 * @cxdoc
 * This is a fairly complicated way of creating a DateFormat object.  You can also
 * use {@tag simpledateformat}.
 *
 * @cxexample
 * <codeblock type="xml">
 * 	<dateformat type="date"/>
 * 	<dateformat type="time"/>
 * 	<dateformat type="datetime"/>
 * 	<dateformat type="date" datestyle="MONTH_FIELD | YEAR_FIELD | DATE_FIELD"/>
 * 	<dateformat type="time" timestyle="HOUR_OF_DAY0_FIELD | MINUTE_FIELD"/>
 * </codeblock>
 *
 * @cxattr	type
 * 			Specifies format for "date", "time" or "datetime" style.
 * @cxattr	datestyle
 * 			The date style.
 * @cxattr	timestyle
 * 			The time style.
 *
 * @see DateFormat
 * @author Heng Yuan
 * @version $Id: DateFormatCreator.java 242 2007-06-07 04:05:08Z coconut $
 * @since CookXml 1.0
 */
public class DateFormatCreator implements Creator
{
	public static String TYPE_ATTR = "type";

	public static String DATE_TYPE = "date";
	public static String TIME_TYPE = "time";
	public static String DATETIME_TYPE = "datetime";

	public static String DATE_STYLE_ATTR = "datestyle";
	public static String TIME_STYLE_ATTR = "timestyle";

	public Object create (String parentNS, String parentTag, Element elm, Object parentObj, DecodeEngine decodeEngine)
	{
		decodeEngine.setDoAdd (false);

		String type = elm.getAttribute (TYPE_ATTR).toLowerCase ();

		if (DATE_TYPE.equals (type))
		{
			return DateFormat.getDateInstance (getStyle (elm, DATE_STYLE_ATTR, DateFormat.DEFAULT));
		}
		if (TIME_TYPE.equals (type))
		{
			return DateFormat.getTimeInstance (getStyle (elm, TIME_STYLE_ATTR, DateFormat.DEFAULT));
		}
		// default if (DATETIME_TYPE.equals (type))
		{
			return DateFormat.getDateTimeInstance (getStyle (elm, DATE_STYLE_ATTR, DateFormat.DEFAULT),
												   getStyle (elm, TIME_STYLE_ATTR, DateFormat.DEFAULT));
		}
	}

	public Object editFinished (String parentNS, String parentTag, Element elm, Object parentObj, Object obj, DecodeEngine decodeEngine)
	{
		if (obj == null || !(obj instanceof DateFormat))
			return obj;
		decodeEngine.addChild (parentNS, parentTag, elm, parentObj, obj);
		return obj;
	}

	public static int getStyle (Element elm, String styleAttr, int defaultVal)
	{
		int returnVal = defaultVal;
		try
		{
			String value = elm.getAttribute (styleAttr);
			if (value == null || value.length () == 0)
				return returnVal;
			StringTokenizer tokenizer = new StringTokenizer (value, "| ");
			do
			{
				String tok = tokenizer.nextToken ();
				Field field = DateFormat.class.getField (tok);
				returnVal |= field.getInt (null);
			}
			while (tokenizer.hasMoreElements ());
		}
		catch (Exception ex)
		{
			throw new IllegalArgumentException (styleAttr);
		}
		return returnVal;
	}
}
