/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.common.creator;

import org.w3c.dom.Element;

import cookxml.core.DecodeEngine;
import cookxml.core.interfaces.Creator;
import cookxml.core.util.TextUtils;

/**
 * This creator either uses the src attribute to load text from a resource, or retrieve
 * the first CDATA (text between start and ending tags of the element).  It is helpful
 * to enter texts that are difficult to enter using StringHelper.
 *
 * @cxdoc
 * This tag is used to enter fairly complicated text that cannot be easily
 * entered using &lt;string> tag.
 * <p>
 * If "<code>src</code>" attribute is specified, then the string is loaded from
 * the resource file specified.
 * <p>
 * Otherwise, the text between the opening and closing tags of &lt;text> are
 * used as the string.
 *
 * @cxexample
 * <codeblock type="xml">
 * 	<!-- loads text from a resource file -->
 * 	<text src="test.txt"/>
 * 	<!-- get the text in between the openning and the closing tag -->
 * 	<text>This is a test.</text>
 * 	<text><![CDATA[This is a test.]]></text>
 * </codeblock>
 *
 * @cxattr	src
 * 			Specifies the text resource that can be loaded by ClassLoader.
 *
 * @see cookxml.common.helper.StringHelper
 * @see cookxml.common.creator.HtmlCreator
 * @author Heng Yuan
 * @version $Id: TextCreator.java 245 2007-06-07 04:49:06Z coconut $
 * @since CookXml 1.0
 */
public class TextCreator implements Creator
{
	public static String SRC_ATTR = "src";

	public Object create (String parentNS, String parentTag, Element elm, Object parentObj, DecodeEngine decodeEngine) throws Exception
	{
		// first check if there is a src attribute
		String srcAttr = elm.getAttribute (SRC_ATTR);
		if (srcAttr != null && srcAttr.length () > 0)
		{
			return TextUtils.readText (srcAttr, decodeEngine.getCookXml ().getClassLoader ());
		}

		return TextUtils.getText (elm);
	}

	public Object editFinished (String parentNS, String parentTag, Element elm, Object parentObj, Object obj, DecodeEngine decodeEngine)
	{
		return obj;
	}
}
