/*
 * (c) Copyright 2006 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.common.setter;

import java.beans.PropertyChangeListener;
import java.lang.reflect.Method;
import java.util.LinkedList;

import cookxml.core.DecodeEngine;
import cookxml.core.doclet.DocletActionReporter;
import cookxml.core.exception.SetterException;
import cookxml.core.interfaces.Setter;
import cookxml.core.interfaces.TagLibrary;

/**
 * This setter is used to hook up PropertyChangeListener.
 *
 * @cxdoc
 * The syntax is "propertyListenerVariable", or "propertyListenerVariable,propertyName".
 *
 * @author Heng Yuan
 * @version $Id: PropertyChangeListenerSetter.java 240 2007-06-06 18:49:38Z coconut $
 * @since CookXml 2.5
 */
public class PropertyChangeListenerSetter implements Setter, DocletActionReporter
{
	public static char SEPARATOR = ',';

	private final static Class[] ONE_PARAM = new Class[]{ PropertyChangeListener.class };
	private final static Class[] TWO_PARAM = new Class[]{ String.class, PropertyChangeListener.class };

	public void setAttribute (String ns, String tag, String attrNS, String attr, Object obj, Object value, DecodeEngine decodeEngine) throws SetterException
	{
		if (value == null)
			return;

		String varName;
		String propName;
		PropertyChangeListener listener;

		if (value instanceof String)
		{
			String str = ((String)value);
			int index = str.indexOf (SEPARATOR);

			if (index < 0)
			{
				varName = str;
				propName = null;
			}
			else
			{
				varName = str.substring (0, index);
				propName = str.substring (index + 1);
				if (propName.length () == 0)
					propName = null;
			}

			// now parse the varName
			listener = (PropertyChangeListener)decodeEngine.getVariable (varName);
		}
		else if (value instanceof PropertyChangeListener)
		{
			listener = (PropertyChangeListener)value;
			propName = null;
		}
		else
		{
			throw new SetterException (decodeEngine, null, this, ns, tag, attrNS, attr, obj, value);
		}

		try
		{

			if (propName == null)
			{
				Method method = obj.getClass ().getMethod ("addPropertyChangeListener", ONE_PARAM);
				method.invoke (obj, new Object[]{ listener });
			}
			else
			{
				Method method = obj.getClass ().getMethod ("addPropertyChangeListener", TWO_PARAM);
				method.invoke (obj, new Object[]{ propName, listener });
			}
		}
		catch (Exception ex)
		{
			throw new SetterException (decodeEngine, ex, this, ns, tag, attrNS, attr, obj, value);
		}
	}

	public Object[] getActions (TagLibrary tagLibrary, String tagNS, String tag, String attrNS, String attr, Class cl)
	{
		if (cl == null)
			return new Object[0];
		LinkedList list = new LinkedList ();
		Method method;
		try
		{
			method = cl.getMethod ("addPropertyChangeListener", ONE_PARAM);
			if (method != null)
				list.add (method);
		}
		catch (Exception ex)
		{
		}
		try
		{
			method = cl.getMethod ("addPropertyChangeListener", TWO_PARAM);
			if (method != null)
				list.add (method);
		}
		catch (Exception ex)
		{
		}
		return list.toArray ();
	}
}
