/*
 * (c) Copyright 2006 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookformlayout;

import javax.swing.*;

import com.jgoodies.forms.debug.FormDebugPanel;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.Sizes;

import cookxml.cookformlayout.adder.CellConstraintsAdder;
import cookxml.cookformlayout.adder.ColumnGroupAdder;
import cookxml.cookformlayout.adder.IgnoreCellConstraintsAdder;
import cookxml.cookformlayout.adder.RowGroupAdder;
import cookxml.cookformlayout.converter.AlignmentConverter;
import cookxml.cookformlayout.creator.FormBuilderCreator;
import cookxml.cookformlayout.creator.FormBuilderHelperCreator;
import cookxml.cookformlayout.creator.FormBuilderPropertyCreator;
import cookxml.cookformlayout.creator.SeparatorHelperCreator;
import cookxml.cookformlayout.helper.*;
import cookxml.cookformlayout.setter.ColumnSpecSetter;
import cookxml.cookformlayout.setter.OriginSetter;
import cookxml.cookformlayout.setter.RowSpecSetter;
import cookxml.core.adder.DefaultAdder;
import cookxml.core.converter.CallCtorConverter;
import cookxml.core.creator.DefaultCreator;
import cookxml.core.creator.HelperCreator;
import cookxml.core.exception.CookXmlException;
import cookxml.core.interfaces.Adder;
import cookxml.core.interfaces.Creator;
import cookxml.core.setter.ConstantSetter;
import cookxml.core.taglibrary.InheritableTagLibrary;

/**
 * @author Heng Yuan
 * @version $Id: CookFormLayoutLib.java 215 2007-06-06 03:59:41Z coconut $
 * @since CookFormLayout 1.0
 */
public class CookFormLayoutLib
{
	public static String FORMLAYOUT = "formlayout";
	public static String PANEL_BUILDER_TAG = "panelbuilder";
	public static String DEFAULT_FORM_BUILDER_TAG = "formbuilder";
	public static String BUTTON_BAR_BUILDER_TAG = "buttonbarbuilder";
	public static String BUTTON_STACK_BUILDER_TAG = "buttonstackbuilder";

	/**
	 * The name of the tag for the CellConstraints object.  Rename it
	 * if you want to have a shorter name.  Like "cell".
	 */
	public static String CELL_CONSTRAINTS_TAG = "cellconstraints";

	/**
	 * Adds JGoodies FormLayout to a Swing tag library.
	 *
	 * @param	tagLibrary
	 *			the Swing tag library to be setup
	 * @throws	cookxml.core.exception.CookXmlException
	 * 			in case of error.
	 * @since	CookFormLayout 1.0
	 */
	public static void setupTags (InheritableTagLibrary tagLibrary) throws CookXmlException
	{
		// debug panel
		tagLibrary.setCreator ("formdebugpanel", DefaultCreator.getCreator (FormDebugPanel.class));
		tagLibrary.inheritTag ("panel", "formdebugpanel");

		// setup FormLayout
		tagLibrary.setCreator (FORMLAYOUT, DefaultCreator.getCreator (FormLayout.class));
		tagLibrary.setSetter (FORMLAYOUT, "rowspec", new RowSpecSetter ());
		tagLibrary.setSetter (FORMLAYOUT, "columnspec", new ColumnSpecSetter ());

		// since Swing tag library should already have setLayout adder setup, we don't need to do it
		// again.

		Adder ignoreCellConstraintsAdder = new IgnoreCellConstraintsAdder ();

		// row and column groups
		tagLibrary.setCreator ("form-rowgroup", DefaultCreator.getCreator (RowGroupHelper.class, true));
		tagLibrary.addAdder (FORMLAYOUT, new RowGroupAdder ());
		tagLibrary.setCreator ("form-columngroup", DefaultCreator.getCreator (ColumnGroupHelper.class, true));
		tagLibrary.addAdder (FORMLAYOUT, new ColumnGroupAdder ());
		tagLibrary.addAdder (FORMLAYOUT, ignoreCellConstraintsAdder);

		// now CellConstraints
		// CellConstraints may appear under the <formlayout> tag or an AbstractFormBuilder tag.
		tagLibrary.setCreator (CELL_CONSTRAINTS_TAG, HelperCreator.getCreator (CellConstraintsHelper.class));
		tagLibrary.setConverter (CellConstraints.class, CallCtorConverter.getConverter (CellConstraints.class));
		tagLibrary.setConverter (CellConstraints.Alignment.class, AlignmentConverter.getInstance ());
		tagLibrary.setAdder (CELL_CONSTRAINTS_TAG, new CellConstraintsAdder ());

		// Form Builders

		Creator formBuilderCreator = new FormBuilderCreator ();

		// PanelBuilder
		tagLibrary.setCreator (PANEL_BUILDER_TAG, formBuilderCreator);
		tagLibrary.setAdder (PANEL_BUILDER_TAG, DefaultAdder.getInstance ());
		tagLibrary.addAdder (PANEL_BUILDER_TAG, ignoreCellConstraintsAdder);

		tagLibrary.setCreator ("form-properties", new FormBuilderPropertyCreator ());
		tagLibrary.setSetter ("form-properties", "origin", new OriginSetter ());

		// Form Builder Function Helpers
		tagLibrary.setCreator ("form-append", FormBuilderHelperCreator.getCreator (AbstractFormBuilderHelper.class, AppendHelper.class));
		tagLibrary.setCreator ("form-label", FormBuilderHelperCreator.getCreator (PanelBuilderHelper.class, LabelHelper.class));
		tagLibrary.setCreator ("form-separator", new SeparatorHelperCreator ());
		tagLibrary.setSetter ("form-separator", "alignment", new ConstantSetter (SwingConstants.class));
		tagLibrary.setCreator ("form-title", FormBuilderHelperCreator.getCreator (PanelBuilderHelper.class, TitleHelper.class));
		tagLibrary.setCreator ("form-nextline", FormBuilderHelperCreator.getCreator (AbstractFormBuilderHelper.class, NextLineHelper.class));
		tagLibrary.setCreator ("form-nextcolumn", FormBuilderHelperCreator.getCreator (AbstractFormBuilderHelper.class, NextColumnHelper.class));

		// we totally ignore the I15dPanelBuilder since CookSwing has its own resourcebundle
		// handling facility.

		// DefaultFormBuilder
		tagLibrary.setCreator (DEFAULT_FORM_BUILDER_TAG, formBuilderCreator);
		tagLibrary.inheritTag (PANEL_BUILDER_TAG, DEFAULT_FORM_BUILDER_TAG);

		ConstantSetter constantSizeSetter = new ConstantSetter (Sizes.class);
		tagLibrary.setSetter ("form-properties", "linegapsize", constantSizeSetter);
		tagLibrary.setSetter ("form-properties", "paragraphgapsize", constantSizeSetter);

		// JGoodies Borders
		tagLibrary.setCreator ("formborder", HelperCreator.getCreator (FormBorderHelper.class));
		tagLibrary.setSetter ("formborder", "name", new ConstantSetter (Borders.class));

		// although ButtonBarBuilder inherits PanelBuilder, it seems PanelBuilder functions
		// should not be used.  So I will not make these two tags inheriting the panelbuilder tag.
		tagLibrary.setCreator (BUTTON_BAR_BUILDER_TAG, formBuilderCreator);
		tagLibrary.setCreator (BUTTON_STACK_BUILDER_TAG, formBuilderCreator);

		tagLibrary.setCreator ("form-add", FormBuilderHelperCreator.getCreator (AbstractFormBuilderHelper.class, AddHelper.class));
		tagLibrary.setCreator ("form-gap", FormBuilderHelperCreator.getCreator (AbstractFormBuilderHelper.class, GapHelper.class));
		tagLibrary.setCreator ("form-glue", FormBuilderHelperCreator.getCreator (AbstractFormBuilderHelper.class, GlueHelper.class));
		tagLibrary.setCreator ("form-strut", FormBuilderHelperCreator.getCreator (AbstractFormBuilderHelper.class, StrutHelper.class));
		tagLibrary.setSetter ("form-strut", "size", constantSizeSetter);
	}

	private CookFormLayoutLib ()
	{
	}
}
