/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookjface;

import java.util.ResourceBundle;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.eclipse.jface.action.StatusLineManager;
import org.eclipse.jface.preference.PreferenceContentProvider;
import org.eclipse.jface.preference.PreferenceLabelProvider;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.custom.TableTree;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Tree;

import cookxml.common.CommonLib;
import cookxml.common.setter.HookVariableSetter;
import cookxml.cookjface.converter.ImageDescriptorConverter;
import cookxml.cookjface.creator.*;
import cookxml.cookjface.setter.SetControlSetter;
import cookxml.cookjface.ui.ArrayTableLabelProvider;
import cookxml.cookswt.CookSwt;
import cookxml.cookswt.creator.WidgetCreator;
import cookxml.core.CookXml;
import cookxml.core.adder.CallFunctionAdder;
import cookxml.core.adder.CallSetterAdder;
import cookxml.core.adder.DoNothingAdder;
import cookxml.core.creator.DefaultCreator;
import cookxml.core.exception.CookXmlException;
import cookxml.core.interfaces.Adder;
import cookxml.core.interfaces.Creator;
import cookxml.core.interfaces.VarLookup;
import cookxml.core.setter.DoNothingSetter;
import cookxml.core.taglibrary.InheritableTagLibrary;

/**
 * @author Heng Yuan
 */
public class CookJFace extends CookXml
{
	//
	// internal tags
	//
	public final static String LISTENERS = CommonLib.LISTENERS;
	public final static String VIEWER = "viewer";
	public final static String CONTENT_VIEWER = "contentviewer";
	public final static String ABSTRACT_LIST_VIEWER = "abstractlistviewer";

	//
	// tags that are being inherited
	//
	public final static String TABLE_VIEWER = "tableviewer";

	/**
	 * the JFace tag library
	 */
	private static InheritableTagLibrary s_jfaceTagLib = CookSwt.getSwtTagLibrary ();
	/**
	 * this the document builder used to create the CookJFace object by default
	 */
	private static DocumentBuilder s_jfaceDocBuilder;
	/**
	 * this is the system wide resource bundle used to handle things such as locale based
	 * text string translation.
	 */
	private static ResourceBundle s_systemResourceBundle;

	/**
	 * a simple way of adding a Viewer tag.
	 *
	 * @param viewerCreator
	 * @param	tagLibrary the tag library to be used.
	 * @param	viewerClass the Viewer class
	 * @param	parentTag the parent tag
	 */
	public static void addViewerTag (InheritableTagLibrary tagLibrary, Class viewerClass, Creator viewerCreator, Class controlClass, String parentTag) throws CookXmlException
	{
		// setup the Viewer tag first
		String name = viewerClass.getName ();
		String tagName = name.substring (name.lastIndexOf (".") + 1).toLowerCase ();
		if (viewerCreator == null)
			viewerCreator = WidgetCreator.getCreator (viewerClass);
		tagLibrary.setCreator (tagName, viewerCreator);
		if (parentTag != null)
			tagLibrary.inheritTag (parentTag, tagName);
		// then we need to update the control tag
		name = controlClass.getName ();
		tagName = name.substring (name.lastIndexOf (".") + 1).toLowerCase ();
		tagLibrary.setCreator (tagName, ViewControlCreator.getCreator (controlClass));
	}

	static
	{
		try
		{
			// setup the system wide jface document builder.
			// change it if necessary to consider things such as validated schema,
			// entityresolver and such.  By default, none of these are necessary.
			setJFaceDocumentBuilder (DocumentBuilderFactory.newInstance ().newDocumentBuilder ());

			//setupJFaceTags (s_jfaceTagLib);
			setupJFaceTags (CookSwt.getSwtTagLibrary ());
		}
		catch (Exception ex)
		{
			ex.printStackTrace ();
		}
	}

	public static void setupJFaceTags (InheritableTagLibrary tagLibrary) throws CookXmlException
	{
		Adder setControlAdder = new CallSetterAdder ("control", null, Viewer.class);

		tagLibrary.setConverter (ImageDescriptor.class, ImageDescriptorConverter.getInstance ());

		tagLibrary.setCreator ("applicationwindow", ApplicationWindowCreator.getInstance ());
		tagLibrary.setCreator ("menumanager", MenuManagerCreator.getInstance ());
		tagLibrary.setSetter ("menumanager", "text", DoNothingSetter.getInstance ());
		tagLibrary.setCreator ("separator", SeparatorCreator.getInstance ());
		tagLibrary.setSetter ("separator", "group", DoNothingSetter.getInstance ());
		tagLibrary.setCreator ("coolbarmanager", CoolBarManagerCreator.getInstance ());
		tagLibrary.setSetter ("coolbarmanager", "style", DoNothingSetter.getInstance ());
		tagLibrary.setCreator ("toolbarmanager", ToolBarManagerCreator.getInstance ());
		tagLibrary.setSetter ("toolbarmanager", "style", DoNothingSetter.getInstance ());
		tagLibrary.setCreator ("statuslinemanager", DefaultCreator.getCreator (StatusLineManager.class));

		tagLibrary.setCreator ("xmlcontentprovider", XmlContentProviderCreator.getInstance ());
		tagLibrary.setSetter ("xmlcontentprovider", "varobj", new HookVariableSetter ("setVarObj", Object.class));

		Adder contentProviderAdder = new CallFunctionAdder ("setContentProvider", null, IContentProvider.class);
		Adder labelProviderAdder = new CallFunctionAdder ("setLabelProvider", null, IBaseLabelProvider.class);
		Adder sorterAdder = new CallFunctionAdder ("setSorter", null, ViewerSorter.class);
		Adder filterAdder = new CallFunctionAdder ("setFilter", null, ViewerFilter.class);

		tagLibrary.addAdder (CONTENT_VIEWER, contentProviderAdder);
		tagLibrary.addAdder (CONTENT_VIEWER, labelProviderAdder);
		tagLibrary.addAdder (CONTENT_VIEWER, sorterAdder);
		tagLibrary.addAdder (CONTENT_VIEWER, filterAdder);
		tagLibrary.setSetter (CONTENT_VIEWER, "style", DoNothingSetter.getInstance ());

		tagLibrary.inheritTag (CONTENT_VIEWER, ABSTRACT_LIST_VIEWER);
		//taglibrary.addAdder (ABSTRACT_LIST_VIEWER, InheritAdder.getInstance ());
		//taglibrary.addAdder (ABSTRACT_LIST_VIEWER, DefaultAdder.getInstance ());

		// ComboViewer
		addViewerTag (tagLibrary, ComboViewer.class, null, Combo.class, ABSTRACT_LIST_VIEWER);
		// ListViewer
		addViewerTag (tagLibrary, ListViewer.class, null, List.class, ABSTRACT_LIST_VIEWER);
		// TableTreeViewer
		addViewerTag (tagLibrary, TableTreeViewer.class, null, TableTree.class, CONTENT_VIEWER);
		// TreeViewer
		addViewerTag (tagLibrary, TreeViewer.class, null, Tree.class, CONTENT_VIEWER);
		// ChecboxTreeViewer
		addViewerTag (tagLibrary, CheckboxTreeViewer.class, null, Tree.class, CONTENT_VIEWER);

		// TableViewer
		addViewerTag (tagLibrary, TableViewer.class, null, Table.class, CONTENT_VIEWER);
		tagLibrary.inheritTag (CONTENT_VIEWER, TABLE_VIEWER);
		//taglibrary.addAdder (TABLE_VIEWER, InheritAdder.getInstance ());
		//taglibrary.addAdder (TABLE_VIEWER, DefaultAdder.getInstance ());
		// CheckboxTableViewer
		addViewerTag (tagLibrary, CheckboxTableViewer.class, CheckboxTableViewerCreator.getInstance (), Table.class, TABLE_VIEWER);

		tagLibrary.addAdder ("ctabitem", setControlAdder);
		tagLibrary.setSetter ("ctabitem", "control", SetControlSetter.getInstance ());
		tagLibrary.addAdder ("tabitem", setControlAdder);
		tagLibrary.setSetter ("tabitem", "control", SetControlSetter.getInstance ());

		// content providers
		tagLibrary.setCreator ("arraycontentprovider", DefaultCreator.getCreator (ArrayContentProvider.class));
		tagLibrary.setCreator ("preferencecontentprovider", DefaultCreator.getCreator (PreferenceContentProvider.class));

		// label providers
		tagLibrary.setCreator ("labelprovider", DefaultCreator.getCreator (LabelProvider.class));
		tagLibrary.setCreator ("arraytablelabelprovider", DefaultCreator.getCreator (ArrayTableLabelProvider.class));
		//taglibrary.setCreator ("decoratinglabelprovider", DefaultCreator.getCreator (DecoratingLabelProvider.class));	// we need to deal with it later
		tagLibrary.setCreator ("preferencelabelprovider", DefaultCreator.getCreator (PreferenceLabelProvider.class));

        // cell editors
		tagLibrary.setCreator ("textcelleditor", TextCellEditorCreator.getInstance ());
		tagLibrary.setCreator ("comboboxcelleditor", ComboBoxCellEditorCreator.getInstance ());

        // cell modifier
		tagLibrary.setCreator ("arraycellmodifier", ArrayCellModifierCreator.getInstance ());

		// ViewerSorter
		tagLibrary.setCreator ("viewersorter", DefaultCreator.getCreator (ViewerSorter.class));

		// Additional layouts
		tagLibrary.setCreator ("tablelayout", DefaultCreator.getCreator (TableLayout.class));
		tagLibrary.addAdder ("tablelayout", new CallFunctionAdder ("addColumnData", null, ColumnLayoutData.class));
		tagLibrary.setCreator ("columnpixeldata", ColumnPixelDataCreator.getInstance ());
		tagLibrary.setAdder ("columnpixeldata", DoNothingAdder.getInstance ());
		tagLibrary.setCreator ("columnweightdata", ColumnWeightDataCreator.getInstance ());
		tagLibrary.setAdder ("columnweightdata", DoNothingAdder.getInstance ());
	}

	/**
	 * If you don't like the current jfaceTabLibrary, it is possible to replace it
	 * with your own.  Or better yet, overload CookXml yourself.
	 *
	 * @param	tagLibrary the tag library to use instead.
	 */
	public static void setJFaceTagLibrary (InheritableTagLibrary tagLibrary)
	{
		s_jfaceTagLib = tagLibrary;
	}

	/**
	 * Get the system wide jface tag library.
	 *
	 * @return	the system wide jface tag library.
	 */
	public static InheritableTagLibrary getJFaceTagLibrary ()
	{
		return s_jfaceTagLib;
	}

	/**
	 * Get the default DocumentBuilder for creating a CookJFace object.
	 *
	 * @return the default DocumentBuilder for creating a CookJFace object.
	 */
	public static DocumentBuilder getJFaceDocumentBuilder ()
	{
		return s_jfaceDocBuilder;
	}

	/**
	 * Set the default DocumentBuilder used for creating a CookJFace object.
	 *
	 * @param	docBuilder the DocumentBuilder object to be used.
	 */
	public static void setJFaceDocumentBuilder (DocumentBuilder docBuilder)
	{
		s_jfaceDocBuilder = docBuilder;
	}

	/**
	 * set the system wide resource bundle for string translation.
	 *
	 * @param bundle the resource bundle to be used.
	 */
	public static void setSystemResourceBundle (ResourceBundle bundle)
	{
		s_systemResourceBundle = bundle;
	}

	/**
	 * create a CookJFace which does not really use variables.
	 */
	public CookJFace ()
	{
		this (null);
	}

	/**
	 * create a CookJFace with an object in which all variable names are looked up.
	 *
	 * @param	varObj the object in which all variable names are looked up.
	 */
	public CookJFace (Object varObj)
	{
		super (getJFaceDocumentBuilder (), s_jfaceTagLib, varObj);
		setResourceBundle (s_systemResourceBundle);
	}

	/**
	 * create a CookJFace with a variable lookup handler.
     *
	 * @param	varLookup	the handler for handling variable lookup
	 */
	public CookJFace (VarLookup varLookup)
	{
		super (getJFaceDocumentBuilder (), s_jfaceTagLib, varLookup);
		setResourceBundle (s_systemResourceBundle);
	}
}
