/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookswing;

import java.awt.*;
import java.util.ResourceBundle;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import cookxml.core.CookXml;
import cookxml.core.interfaces.VarLookup;
import cookxml.core.taglibrary.InheritableTagLibrary;

/**
 * This class basically customizes CookXml for Swing.
 *
 * @author Heng Yuan
 * @version $Id: CookSwing.java 215 2007-06-06 03:59:41Z coconut $
 * @since CookSwing 1.0
 */
public class CookSwing extends CookXml
{
	/** the default swing tag library. */
	private static InheritableTagLibrary s_swingTagLib;
	/** this the document builder used to create the CookSwing object by default. */
	private static DocumentBuilder s_swingDocBuilder;
	/**
	 * this is the system wide resource bundle used to handle things such as locale based
	 * text string translation.
	 */
	private static ResourceBundle s_systemResourceBundle;

	static
	{
		try
		{
			// setup the system wide swing document builder.
			// change it if necessary to consider things such as validated schema,
			// entityresolver and such.  By default, none of these are necessary.

			//
			// Starting from CookSwing 1.5, optional XML namespace is supported.
			//
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance ();
			factory.setNamespaceAware (true);
			setSwingDocumentBuilder (factory.newDocumentBuilder ());
		}
		catch (Exception ex)
		{
			ex.printStackTrace ();
		}
	}

	/**
	 * If you don't like the current swingTabLibrary, it is possible to replace it
	 * with your own.  Or better yet, overload CookXml yourself.
	 *
	 * @param	tagLibrary
	 * 			the tag library to be set
	 */
	public static void setSwingTagLibrary (InheritableTagLibrary tagLibrary)
	{
		s_swingTagLib = tagLibrary;
	}

	/**
	 * Get the system wide swing tag library.  If the user did not setup a version of swing tag library.
	 * A default library is created.
	 *
	 * @return	the system wide swing tag library.
	 */
	public static InheritableTagLibrary getSwingTagLibrary ()
	{
		if (s_swingTagLib == null)
			s_swingTagLib = CookSwingLibSingleton.getTagLibrary ();
		return s_swingTagLib;
	}

	/**
	 * Get the default DocumentBuilder for creating a CookSwing object.
	 * @return the default DocumentBuilder for creating a CookSwing object.
	 */
	public static DocumentBuilder getSwingDocumentBuilder ()
	{
		return s_swingDocBuilder;
	}

	/**
	 * Set the default DocumentBuilder used for creating a CookSwing object.
	 * @param	docBuilder
	 *			the document builder for XML documents.
	 */
	public static void setSwingDocumentBuilder (DocumentBuilder docBuilder)
	{
		s_swingDocBuilder = docBuilder;
	}

	/**
	 * set the system wide resource bundle for string translation.
	 * @param   bundle
	 *          the resource bundle to be used.
	 */
	public static void setSystemResourceBundle (ResourceBundle bundle)
	{
		s_systemResourceBundle = bundle;
	}

	/**
	 * create a cookswing which does not really use variables.
	 */
	public CookSwing ()
	{
		this ((VarLookup)null);
	}

	/**
	 * create a cookswing with a variable lookup handler.
	 * @param	varLookup	the handler for handling variable lookup
	 */
	public CookSwing (VarLookup varLookup)
	{
		super (getSwingDocumentBuilder (), getSwingTagLibrary (), varLookup);
		setResourceBundle (s_systemResourceBundle);
	}

	/**
	 * create a cookswing with an object in which all variable names are looked up.
	 * @param	varObj	the object in which all variable names are looked up.
	 */
	public CookSwing (Object varObj)
	{
		super (getSwingDocumentBuilder (), getSwingTagLibrary (), varObj);
		setResourceBundle (s_systemResourceBundle);
	}

	/**
	 * create a cookswing with a variable lookup handler.
	 * @param	varLookup	the handler for handling variable lookup
	 * @param   classLoader the ClassLoader used to locate resources.
	 */
	public CookSwing (VarLookup varLookup, ClassLoader classLoader)
	{
		this (varLookup);
		setClassLoader (classLoader);
	}

	/**
	 * create a cookswing with an object in which all variable names are looked up.
	 * @param	varObj	the object in which all variable names are looked up.
	 * @param   classLoader the ClassLoader used to locate resources.
	 */
	public CookSwing (Object varObj, ClassLoader classLoader)
	{
		this (varObj);
		setClassLoader (classLoader);
	}

	/**
	 * just a short cut to call xmlDecode to avoid type casting.
	 * @param	input
	 * 			document input source
	 * @return	the container object
	 */
	public Container render (Object input)
	{
		try
		{
			return (Container)xmlDecode (input);
		}
		catch (Exception ex)
		{
			ex.printStackTrace ();
			return null;
		}
	}

	/**
	 * another short cut for getting the root object of the xml document and do type casting at the
	 * same time.
	 * @return	the root object which is a container.
	 */
	public Container getRootContainer ()
	{
		try
		{
			return (Container)getRootObject ();
		}
		catch (Exception ex)
		{
			return null;
		}
	}
}
