/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookswing.converter;

import java.awt.*;
import java.lang.reflect.Field;
import javax.swing.*;

import cookxml.cookswing.CookSwingLib;
import cookxml.core.DecodeEngine;
import cookxml.core.interfaces.Converter;

/**
 * Convert a hex representation in "#rrggbb", "rrggbb", or names predefined in Color.
 * Or lookup UI Manager key valued color for strings with <code>ui:</code> prefix.
 *
 * @cxdoc
 * There are three valid types of values:
 * <ul>
 *     <li>"<code>#rrggbb</code>" or "<code>rrggbb</code>" for RGB color.</li>
 *     <li>Name of predefined colors in java.awt.Color.</li>
 *     <li><code>ui:<i>key</i></code> for colors that can be retrieved from UIManager
 *			with the given key.</li>
 * </ul>
 *
 * @author Heng Yuan
 * @version $Id: ColorConverter.java 233 2007-06-06 08:08:49Z coconut $
 * @since CookSwing 1.0
 */
public class ColorConverter implements Converter
{
	public Object convert (String value, DecodeEngine decodeEngine)
	{
		// first check if it is a pre-defined color value
		try
		{
			Field field = Color.class.getField (value.toLowerCase ());
			if (field != null)
				return field.get (null);
		}
		catch (Exception ex)
		{
		}

		// check if it is a UIManager color name.  According to dmoebius's suggestion
		if (value.startsWith (CookSwingLib.UI_PREFIX))
		{
			try
			{
				Color color = UIManager.getColor (value.substring (CookSwingLib.UI_PREFIX.length ()));
				if (color != null)
					return color;
			}
			catch (Exception ex)
			{
			}
		}

		if (value.startsWith ("#"))
			value = value.substring (1);
		// we assume that it is going to be a 6 digit hexdecimal notation
		return new Color (str2hex (value));
	}

	public static int str2hex (String str)
	{
		int size = str.length ();
		int hex = 0;
		for (int i = 0; i < size; ++i)
		{
			char c = str.charAt (i);
			if (c >= '0' && c <= '9')
				hex = hex * 16 + (c - '0');
			else if (c >= 'a' && c <= 'f')
				hex = hex * 16 + 10 + (c - 'a');
			else if (c >= 'A' && c <= 'F')
				hex = hex * 16 + 10 + (c - 'A');
			else
				break;
		}
		return hex;
	}
}
