/*
 * (c) Copyright 2005 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookswing.creator;

import javax.swing.*;

import org.w3c.dom.Element;

import cookxml.core.DecodeEngine;
import cookxml.core.interfaces.Creator;

/**
 * This creator is added to consider the cases where JSpinner is used.  In those cases,
 * JFormattedText field is obtained from the spinner editor.
 *
 * @cxdoc
 * If the parent of this tag is a JSpinner component or the editor for the JSpinner
 * component, then this tag retrieves the JFormattedTextField from the JSpinner
 * rather than creating a new instance, else a new instance of JFormattedTextField
 * is created.
 * <p>
 * Note that, in you want to set the format for a spinner text display, please see
 * the documentation for the spinner tag.
 *
 * @cxexample
 * A formatted text field with number input:
 * <codeblock>
 * 	<formattedtextfield columns="10">
 * 		<numberformatter/>
 * 		<double setas="value" value="-125.125"/>
 * 	</formattedtextfield>
 * </codeblock>
 * <p>
 * A formatted text field with mask input:
 * <codeblock>
 * 	<formattedtextfield columns="20">
 *  	<maskformatter mask="1-***-***-****" validcharacters="0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"/>
 * 		<!-- I am kidding here.  Don't call this number! -->
 * 		<<string setas="value" text="1-900-CAL-HENG"/>
 * 	/formattedtextfield>
 * </pre>
 *
 * @see javax.swing.JFormattedTextField
 * @author Heng Yuan
 * @version $Id: FormattedTextFieldCreator.java 251 2007-06-08 00:19:35Z coconut $
 * @since CookSwing 1.2
 */
public class FormattedTextFieldCreator implements Creator
{
	public Object create (String parentNS, String parentTag, Element elm, Object parentObj, DecodeEngine decodeEngine)
	{
		decodeEngine.setDoAdd (false);
		if (parentObj != null)
		{
			if (parentObj instanceof JSpinner)
			{
				JComponent editor = ((JSpinner)parentObj).getEditor ();
				if (editor instanceof JSpinner.DefaultEditor)
					return ((JSpinner.DefaultEditor)editor).getTextField ();
			}
			else if (parentObj instanceof JSpinner.DefaultEditor)
				return ((JSpinner.DefaultEditor)parentObj).getTextField ();
		}
		decodeEngine.setDoAdd (true);
		return new JFormattedTextField ();
	}

	public Object editFinished (String parentNS, String parentTag, Element elm, Object parentObj, Object obj, DecodeEngine decodeEngine)
	{
		return obj;
	}
}
