/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookswing.helper;

import java.awt.*;
import javax.swing.*;

import cookxml.core.interfaces.NoAdd;

/**
 * Helper class for constructing a spring constraint.  A spring constraint can be very tricky
 * to set up.
 * <p>
 * Given a dimension (x or y), there three types of constraint.  the top/left constraint,
 * the width/height constraints, and the bottom/right constraints.  Only two constraints can be
 * active for a given component in one dimension, since the third can be inferred from the other
 * two.  Thus, if you want to set the constraint for both sides of a component, width and height
 * constraints, which are present by default, must be cleared.
 *
 * @author Heng Yuan
 * @version $Id: SpringConstraintHelper.java 215 2007-06-06 03:59:41Z coconut $
 * @since CookSwing 1.0
 */
public class SpringConstraintHelper implements NoAdd
{
	/** The x padding. */
	public Integer xpad;
	/** The y padding. */
	public Integer ypad;
	/** The width spring. */
	public Spring width;
	/** The height spring. */
	public Spring height;

    /** this flag is used to clear the width spring of the component. */
	public boolean clearWidth;
	/** this flag is used to clear the height spring of the component. */
	public boolean clearHeight;

	/** get and set the width spring of the component to be its preferred width. */
	public boolean preferredWidth;
	/** get and set the height spring of the component to be its preferred height. */
	public boolean preferredHeight;
	/** get and set the width spring of the component to be its maximum width. */
	public boolean maxWidth;
	/** get and set the height spring of the component to be its maximum height. */
	public boolean maxHeight;
	/** get and set the width spring of the component to be its minimum width. */
	public boolean minWidth;
	/** get and set the height spring of the component to be its minimum height. */
	public boolean minHeight;

    /** the padding distance between the two edges */
	public int pad;
	/** the edge of this component. */
	public String edge1;
	/** the edge of the component to be aligned. */
	public String edge2;

	/** the component to be aligned. */
	public Component comp1;
	/** the component to compare to. */
	public Component comp2;

	/**
	 * The first component is the component to be aligned and the second
	 * component is the component to compare to.
	 *
	 * @param	c
	 * 			a GUI component.
	 */
	public void add (Component c)
	{
		if (comp1 == null)
			comp1 = c;
		else
			comp2 = c;
	}

	public void putConstraint (SpringLayout layout)
	{
		if (comp1 == null)
			return;

		SpringLayout.Constraints cons = layout.getConstraints (comp1);
		if (cons == null)
			return;

		//
		// using xpad and ypad are a simple way of setting starting positions
		//
		boolean absOffset = false;
		if (comp2 == null || layout.getConstraints (comp2) == null)
			absOffset = true;

		if (xpad != null)
		{
			Spring x = Spring.constant (xpad.intValue ());
			if (absOffset)
				cons.setX (x);
			else
				cons.setX (Spring.sum (layout.getConstraint (SpringLayout.EAST, comp2), x));
		}

		if (ypad != null)
		{
			Spring y = Spring.constant (ypad.intValue ());
			if (absOffset)
				cons.setY (y);
			else
				cons.setY (Spring.sum (layout.getConstraint (SpringLayout.SOUTH, comp2), y));
		}

		//
		// width and height can be set as well
		//
		if (width != null)
			cons.setWidth (width);
		if (height != null)
			cons.setHeight (height);

		//
		// unfortunately, starting xy, width and height, and ending xy
		// only two can be active at a time.  Usually width and height
		// for a component is pre-set, thus for cases where one wants
		// resize a component
		//
		if (comp1 != null)
		{
			if (clearWidth)
				cons.setWidth (null);
			if (clearHeight)
				cons.setHeight (null);
			if (preferredWidth)
				cons.setWidth (Spring.constant (cons.getWidth ().getPreferredValue ()));
			if (minWidth)
				cons.setWidth (Spring.constant (cons.getWidth ().getMinimumValue ()));
			if (maxWidth)
				cons.setWidth (Spring.constant (cons.getWidth ().getMaximumValue ()));
			if (preferredWidth)
				cons.setHeight (Spring.constant (cons.getHeight ().getPreferredValue ()));
			if (minWidth)
				cons.setHeight (Spring.constant (cons.getHeight ().getMinimumValue ()));
			if (maxWidth)
				cons.setHeight (Spring.constant (cons.getHeight ().getMaximumValue ()));
		}

		//
		// setting the edge constraint
		//
		if (comp1 != null && comp2 != null)
		{
			edge1 = getConstraint (edge1);
			edge2 = getConstraint (edge2);
			if (edge1 != null && edge2 != null)
				layout.putConstraint (edge1, comp1, pad, edge2, comp2);
		}
	}

	public static String getConstraint (String constraint)
	{
		if (SpringLayout.NORTH.equalsIgnoreCase (constraint))
			return SpringLayout.NORTH;
		if (SpringLayout.WEST.equalsIgnoreCase (constraint))
			return SpringLayout.WEST;
		if (SpringLayout.EAST.equalsIgnoreCase (constraint))
			return SpringLayout.EAST;
		if (SpringLayout.SOUTH.equalsIgnoreCase (constraint))
			return SpringLayout.SOUTH;
		return null;
	}
}
