/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookswing.helper;

import java.util.LinkedList;
import javax.swing.*;

import cookxml.cookswing.util.SyncButtonStateListener;
import cookxml.core.interfaces.Helper;

/**
 * Helper class for constructing a synchronized button list.
 *
 * @cxdoc
 * This tag is used to make sure some GUI components to remain in the same
 * state.
 * <p>
 * For example, we may have a JCheckBoxMenuItem and a JToggleButton
 * that both represent that insertion mode is on.  Thus, one is on the
 * menubar and another is on the toolbar.
 * <p>
 * Usually, one has to do it manually to synchronize the state, but this
 * tag can manage this synchronization, and thus avoids hard coding.
 *
 * <codeblock>
 * 	<syncbuttonstate>
 * 		<!-- assuming that the following two GUI compoments were already setup
 * 			 in the earlier part of the XML.  Here just retrieve them using
 * 			 their id's -->
 * 		<checkboxmenuitem idref="insertModeMenuItem"/>
 * 		<togglebutton idref="insertModeToggleButton"/>
 * 	</syncbuttonstate>
 * </codeblock>
 *
 * @author Heng Yuan
 * @version $Id: SyncButtonStateHelper.java 215 2007-06-06 03:59:41Z coconut $
 * @since CookSwing 1.0
 */
public class SyncButtonStateHelper implements Helper
{
	private final LinkedList m_buttonList = new LinkedList ();

	/**
	 * Add the button to the list of GUI components that require
	 * to have the same on/off same state.
	 *
	 * @param	button
	 * 			the button object to be synchronized.
	 */
	public void add (AbstractButton button)
	{
		if (button != null && !m_buttonList.contains (button))
			m_buttonList.add (button);
	}

	public Object getFinalObject () throws Exception
	{
		SyncButtonStateListener.syncButtons (m_buttonList);
		return null;
	}
}
