/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookswt;

import java.util.ResourceBundle;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import cookxml.core.CookXml;
import cookxml.core.interfaces.VarLookup;
import cookxml.core.taglibrary.InheritableTagLibrary;

/**
 * @author Heng Yuan
 * @since CookSwt 1.0
 */
public class CookSwt extends CookXml
{
	/** the SWT tag library */
	private static InheritableTagLibrary s_swtTagLib;
	/** this the document builder used to create the CookSwt object by default */
	private static DocumentBuilder s_swtDocBuilder;
	/**
	 * this is the system wide resource bundle used to handle things such as locale based
	 * text string translation.
	 */
	private static ResourceBundle s_systemResourceBundle;

	static
	{
		try
		{
			// setup the system wide swt document builder.
			// change it if necessary to consider things such as validated schema,
			// entityresolver and such.  By default, none of these are necessary.
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance ();
			factory.setNamespaceAware (true);
			setSwtDocumentBuilder (factory.newDocumentBuilder ());
		}
		catch (Exception ex)
		{
			ex.printStackTrace ();
		}
	}

	/**
	 * If you don't like the current swtTabLibrary, it is possible to replace it
	 * with your own.  Or better yet, overload CookXml yourself.
	 *
	 * @param	tagLibrary the tag library to use instead.
	 */
	public static void setSwtTagLibrary (InheritableTagLibrary tagLibrary)
	{
		s_swtTagLib = tagLibrary;
	}

	/**
	 * Get the system wide swt tag library.
	 *
	 * @return the system wide swt tag library.
	 */
	public static InheritableTagLibrary getSwtTagLibrary ()
	{
		if (s_swtTagLib == null)
			s_swtTagLib = CookSwtLibSingleton.getTagLibrary ();
		return s_swtTagLib;
	}

	/**
	 * Get the default DocumentBuilder for creating a CookSwt object.
	 *
	 * @return the default DocumentBuilder for creating a CookSwt object.
	 */
	public static DocumentBuilder getSwtDocumentBuilder ()
	{
		return s_swtDocBuilder;
	}

	/**
	 * Set the default DocumentBuilder used for creating a CookSwt object.
	 *
	 * @param	docBuilder the DocumentBuilder object to be used.
	 */
	public static void setSwtDocumentBuilder (DocumentBuilder docBuilder)
	{
		s_swtDocBuilder = docBuilder;
	}

	/**
	 * set the system wide resource bundle for string translation.
	 *
	 * @param bundle the resource bundle to be used.
	 */
	public static void setSystemResourceBundle (ResourceBundle bundle)
	{
		s_systemResourceBundle = bundle;
	}

	/** create a CookSwt which does not really use variables. */
	public CookSwt ()
	{
		this ((VarLookup)null);
	}

	/**
	 * create a CookSwt with with a variable lookup handler.
	 *
	 * @param	varLookup	the handler for handling variable lookup
	 */
	public CookSwt (VarLookup varLookup)
	{
		super (getSwtDocumentBuilder (), getSwtTagLibrary (), varLookup);
		setResourceBundle (s_systemResourceBundle);
	}

	/**
	 * create a CookSwt with an object in which all variable names are looked up.
	 *
	 * @param	varObj the object in which all variable names are looked up.
	 */
	public CookSwt (Object varObj)
	{
		super (getSwtDocumentBuilder (), getSwtTagLibrary (), varObj);
		setResourceBundle (s_systemResourceBundle);
	}

	/**
	 * create a CookSwt with with a variable lookup handler.
	 *
	 * @param	varLookup	the handler for handling variable lookup
	 * @param   classLoader the ClassLoader used to locate resources.
	 */
	public CookSwt (VarLookup varLookup, ClassLoader classLoader)
	{
		this (varLookup);
		setClassLoader (classLoader);
	}

	/**
	 * create a CookSwt with an object in which all variable names are looked up.
	 *
	 * @param	varObj the object in which all variable names are looked up.
	 * @param   classLoader the ClassLoader used to locate resources.
	 */
	public CookSwt (Object varObj, ClassLoader classLoader)
	{
		this (varObj);
		setClassLoader (classLoader);
	}
}
