/*
 * (c) Copyright 2005 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookswt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.*;
import org.eclipse.swt.custom.*;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.opengl.GLData;
import org.eclipse.swt.printing.PrintDialog;
import org.eclipse.swt.printing.PrinterData;
import org.eclipse.swt.widgets.*;

import cookxml.common.CommonLib;
import cookxml.common.setter.HookVariableSetter;
import cookxml.common.setter.ObjectOrHookVariableSetter;
import cookxml.common.util.Utils;
import cookxml.cookswt.adder.*;
import cookxml.cookswt.converter.*;
import cookxml.cookswt.creator.*;
import cookxml.cookswt.helper.*;
import cookxml.cookswt.setter.*;
import cookxml.cookswt.ui.XmlDialog;
import cookxml.core.adder.CallFunctionAdder;
import cookxml.core.adder.CallSetterAdder;
import cookxml.core.adder.DefaultAdder;
import cookxml.core.adder.DoNothingAdder;
import cookxml.core.creator.DefaultCreator;
import cookxml.core.creator.HelperCreator;
import cookxml.core.exception.CookXmlException;
import cookxml.core.interfaces.Adder;
import cookxml.core.setter.ConstantSetter;
import cookxml.core.setter.DoNothingSetter;
import cookxml.core.taglibrary.InheritableTagLibrary;

/**
 * @since CookSwt 1.3
 * @author Heng Yuan
 */
public class CookSwtLib
{
	/** this is the namespace for CookSwt tags */
	public static String NAMESPACE = "http://cookxml.sf.net/cookswt/";

	//
	// internal tags
	//
	public final static String CONTROL = "control";
	public final static String DIALOG = "dialog";
	public final static String WIDGET = "widget";

	//
	// tags that are being inherited
	//
	public final static String COMPOSITE = "composite";
	public final static String CANVAS = "canvas";
	public final static String ITEM = "item";
	public final static String DECORATIONS = "decorations";

	/** the attribute for setting general commands */
	public final static String COMMAND_ATTR = "command";

	/**
	 * a simple way of adding a Widget tag.
	 *
	 * @param	tagLibrary
	 * 			the tag library to be used.
	 * @param	widgetClass
	 * 			the Widget class
	 * @param	parentTag
	 * 			the parent tag
	 */
	public static void addWidgetTag (InheritableTagLibrary tagLibrary, Class widgetClass, String parentTag) throws CookXmlException
	{
		String name = widgetClass.getName ();
		String tagName = name.substring (name.lastIndexOf (".") + 1).toLowerCase ();
		tagLibrary.setCreator (tagName, WidgetCreator.getCreator (widgetClass));
		if (parentTag != null)
			tagLibrary.inheritTag (parentTag, tagName);
	}

	/**
	 * a simple way of adding a Dialog tag.  The parent tag is assumed to be "dialog".
	 *
	 * @param	tagLibrary
	 * 			the tag library to be used.
	 * @param	dialogClass
	 * 			dialog class
	 * @see #DIALOG
	 */
	public static void addDialogTag (InheritableTagLibrary tagLibrary, Class dialogClass) throws CookXmlException
	{
		addDialogTag (tagLibrary, dialogClass, DIALOG);
	}

	/**
	 * a simple way of adding a Dialog tag.
	 *
	 * @param	tagLibrary
	 * 			the tag library to be used.
	 * @param	dialogClass
	 * 			dialog class
	 * @param	parentTag
	 * 			the parent tag
	 */
	public static void addDialogTag (InheritableTagLibrary tagLibrary, Class dialogClass, String parentTag) throws CookXmlException
	{
		addWidgetTag (tagLibrary, dialogClass, parentTag);
	}

	/**
	 * Setup the SWT tag library
	 *
	 * @param tagLibrary the tag library to be setup
	 */
	public static void setupTags (InheritableTagLibrary tagLibrary, InheritableTagLibrary commonTagLibrary) throws CookXmlException
	{
		ConstantSetter swtConstantSetter = new ConstantSetter (SWT.class);

		////////////////////////////////////////////////////////////////////////////
		//
		// Converters and some supporting object tags
		//
		////////////////////////////////////////////////////////////////////////////

		// Font
		tagLibrary.setCreator ("font", HelperCreator.getCreator (FontHelper.class));
		tagLibrary.setConverter (Font.class, FontConverter.getInstance ());
		// Image
		tagLibrary.setCreator ("image", HelperCreator.getCreator (ImageHelper.class));
		tagLibrary.setConverter (Image.class, ImageConverter.getInstance ());
		// ImageData
		tagLibrary.setConverter (ImageData.class, new ImageDataConverter ());
		// Color
		tagLibrary.setCreator ("color", HelperCreator.getCreator (ColorHelper.class));
		tagLibrary.setConverter (Color.class, ColorConverter.getInstance ());
		// Cursor
		tagLibrary.setCreator ("cursor", HelperCreator.getCreator (CursorHelper.class));
		tagLibrary.setConverter (Cursor.class, CursorConverter.getInstance ());
		// Point
		tagLibrary.setCreator ("point", new PointCreator ());
		tagLibrary.setConverter (Point.class, new PointConverter ());
		// Rectangle
		tagLibrary.setCreator ("rectangle", new RectangleCreator ());
		tagLibrary.setConverter (Rectangle.class, new RectangleConverter ());

		////////////////////////////////////////////////////////////////////////////
		//
		// SWT Components
		//
		////////////////////////////////////////////////////////////////////////////

		Adder menuAdder = new CallSetterAdder ("menu", null, Menu.class);
		Adder setControlAdder = CallFunctionAdder.getAdder ("setControl", null, Control.class);

		// basic widget class
		tagLibrary.setAdder (WIDGET, DoNothingAdder.getInstance ());
		tagLibrary.setSetter (WIDGET, "style", DoNothingSetter.getInstance ());
		tagLibrary.setSetter (WIDGET, COMMAND_ATTR, new DataSetter ());
		tagLibrary.inheritTag (CommonLib.NAMESPACE, CommonLib.LISTENERS, WIDGET);

		tagLibrary.setCreator ("display", new DisplayCreator ());
		tagLibrary.setAdder ("display", DoNothingAdder.getInstance ());

		tagLibrary.setCreator ("shell", new ShellCreator ());
		tagLibrary.setSetter ("shell", "imeinputmode", swtConstantSetter);
		tagLibrary.inheritTag (DECORATIONS, "shell");

		// child classes of Widget
		addWidgetTag (tagLibrary, Caret.class, WIDGET);
		tagLibrary.setSetter ("caret", "image", ImageSetter.getInstance ());
		tagLibrary.setSetter ("caret", "font", FontSetter.getInstance ());
		addWidgetTag (tagLibrary, DragSource.class, WIDGET);
		addWidgetTag (tagLibrary, DropTarget.class, WIDGET);
		tagLibrary.setCreator ("menu", new MenuCreator ());
		tagLibrary.inheritTag (WIDGET, "menu");
		tagLibrary.setCreator ("menubar", new MenuBarCreator ());
		tagLibrary.inheritTag (WIDGET, "menubar");
		tagLibrary.setCreator ("tracker", new TrackerCreator ());
		tagLibrary.setSetter ("tracker", "cursor", CursorSetter.getInstance ());
		tagLibrary.inheritTag (WIDGET, "tracker");
		tagLibrary.setCreator ("systemtray", new SystemTrayCreator ());
		tagLibrary.inheritTag (WIDGET, "systemtray");

		// Control
		tagLibrary.inheritTag (WIDGET, CONTROL);
		tagLibrary.addAdder (CONTROL, menuAdder);
		tagLibrary.setSetter (CONTROL, "font", FontSetter.getInstance ());
		tagLibrary.setSetter (CONTROL, "cursor", CursorSetter.getInstance ());
		tagLibrary.setSetter (CONTROL, "foreground", ColorSetter.getInstance ());
		tagLibrary.setSetter (CONTROL, "background", ColorSetter.getInstance ());

		// child classes of Control
		addWidgetTag (tagLibrary, Button.class, CONTROL);
		tagLibrary.setSetter ("button", "image", ImageSetter.getInstance ());
		tagLibrary.setSetter ("button", "alignment", swtConstantSetter);

		addWidgetTag (tagLibrary, Label.class, CONTROL);
		tagLibrary.setSetter ("label", "image", ImageSetter.getInstance ());
		tagLibrary.setSetter ("label", "alignment", swtConstantSetter);

		addWidgetTag (tagLibrary, ProgressBar.class, CONTROL);
		addWidgetTag (tagLibrary, Sash.class, CONTROL);
		addWidgetTag (tagLibrary, Scale.class, CONTROL);
		addWidgetTag (tagLibrary, Slider.class, CONTROL);
		// child classes of Scrollable, but since nothing really special was there, so we
		// use CONTROL as parent tag
		addWidgetTag (tagLibrary, Composite.class, CONTROL);
		tagLibrary.addAdder (COMPOSITE, CallFunctionAdder.getAdder ("setLayout", Composite.class, Layout.class));
		tagLibrary.setSetter (COMPOSITE, "backgroundmode", swtConstantSetter);
		addWidgetTag (tagLibrary, Link.class, CONTROL);
		addWidgetTag (tagLibrary, Spinner.class, COMPOSITE);
		addWidgetTag (tagLibrary, List.class, CONTROL);
		tagLibrary.addAdder ("list", DefaultAdder.getInstance ());		// for add(String)
		// Text - we rename it textarea to avoid conflict ith generic text node
		tagLibrary.setCreator ("textarea", WidgetCreator.getCreator (Text.class));
		tagLibrary.inheritTag (CONTROL, "textarea");
		tagLibrary.setSetter ("textarea", "orientation", swtConstantSetter);
		// child classes of Composite
		addWidgetTag (tagLibrary, Browser.class, COMPOSITE);
		addWidgetTag (tagLibrary, Canvas.class, COMPOSITE);
		addWidgetTag (tagLibrary, CBanner.class, COMPOSITE);
		addWidgetTag (tagLibrary, CCombo.class, COMPOSITE);
		tagLibrary.addAdder ("ccombo", DefaultAdder.getInstance ());		// for add(String)
		addWidgetTag (tagLibrary, Combo.class, COMPOSITE);
		tagLibrary.addAdder ("combo", DefaultAdder.getInstance ());		// for add(String)
		tagLibrary.setSetter ("combo", "orientation", swtConstantSetter);
		tagLibrary.setCreator ("coolbar", new CoolBarCreator ());
		tagLibrary.inheritTag (COMPOSITE, "coolbar");
		addWidgetTag (tagLibrary, CTabFolder.class, COMPOSITE);
		tagLibrary.setSetter ("ctabfolder", "selectionbackground", ColorAndImageSetter.getInstance ());
		tagLibrary.setSetter ("ctabfolder", "selectionforeground", ColorSetter.getInstance ());
		addWidgetTag (tagLibrary, Group.class, COMPOSITE);
		addWidgetTag (tagLibrary, SashForm.class, COMPOSITE);
		tagLibrary.setSetter ("sashform", "orientation", swtConstantSetter);
		addWidgetTag (tagLibrary, ScrolledComposite.class, COMPOSITE);
		addWidgetTag (tagLibrary, TabFolder.class, COMPOSITE);
		addWidgetTag (tagLibrary, Table.class, COMPOSITE);
		tagLibrary.addAdder ("table", TableAdder.getInstance ());
		tagLibrary.setSetter ("table", "sortdirection", swtConstantSetter);
		tagLibrary.setCreator ("toolbar", new ToolBarCreator ());
		tagLibrary.inheritTag (COMPOSITE, "toolbar");
		addWidgetTag (tagLibrary, Tree.class, COMPOSITE);
		tagLibrary.setSetter ("tree", "sortdirection", swtConstantSetter);
		addWidgetTag (tagLibrary, ViewForm.class, COMPOSITE);
		// child classes of Canvas
		//addWidgetTag (taglibrary, AnimatedProgress.class);    // deprecated as of SWT 2.1
		addWidgetTag (tagLibrary, CLabel.class, CANVAS);
		tagLibrary.setSetter ("clabel", "image", ImageSetter.getInstance ());
		tagLibrary.setSetter ("clabel", "background", ColorAndImageSetter.getInstance ());
		tagLibrary.setSetter ("clabel", "alignment", swtConstantSetter);
		addWidgetTag (tagLibrary, Decorations.class, CANVAS);
		tagLibrary.addAdder (DECORATIONS, MenuBarAdder.getInstance ());
		tagLibrary.setSetter (DECORATIONS, "image", ImageSetter.getInstance ());
		addWidgetTag (tagLibrary, StyledText.class, CANVAS);
		tagLibrary.setSetter ("styledtext", "orientation", swtConstantSetter);
		tagLibrary.setSetter ("styledtext", "selectionbackground", ColorSetter.getInstance ());
		tagLibrary.setSetter ("styledtext", "selectionforeground", ColorSetter.getInstance ());
		addWidgetTag (tagLibrary, TableCursor.class, CANVAS);
		// child classes of Item
		tagLibrary.inheritTag (CommonLib.NAMESPACE, CommonLib.LISTENERS, ITEM);
		tagLibrary.setSetter (ITEM, "image", ImageSetter.getInstance ());
		tagLibrary.setSetter (ITEM, "style", DoNothingSetter.getInstance ());
		addWidgetTag (tagLibrary, CoolItem.class, ITEM);
		tagLibrary.addAdder ("coolitem", CoolItemAdder.getInstance ());
		addWidgetTag (tagLibrary, CTabItem.class, ITEM);
		tagLibrary.addAdder ("ctabitem", setControlAdder);
		tagLibrary.setSetter ("ctabitem", "font", FontSetter.getInstance ());
		addWidgetTag (tagLibrary, MenuItem.class, ITEM);
		tagLibrary.addAdder ("menuitem", menuAdder);
		tagLibrary.setSetter ("menuitem", "accelerator", new AcceleratorSetter ());
		tagLibrary.setCreator ("menusplit", new MenuSplitCreator ());
		tagLibrary.inheritTag (ITEM, "menusplit");
		addWidgetTag (tagLibrary, TabItem.class, ITEM);
		tagLibrary.addAdder ("tabitem", setControlAdder);
		addWidgetTag (tagLibrary, TableColumn.class, ITEM);
		tagLibrary.setSetter ("tablecolumn", "alignment", swtConstantSetter);
		addWidgetTag (tagLibrary, TableItem.class, ITEM);
		tagLibrary.setSetter ("tableitem", "font", FontSetter.getInstance ());
		tagLibrary.setSetter ("tableitem", "background", ColorSetter.getInstance ());
		tagLibrary.setSetter ("tableitem", "foreground", ColorSetter.getInstance ());
		addWidgetTag (tagLibrary, ToolItem.class, ITEM);
		tagLibrary.addAdder ("toolitem", DropDownMenuAdder.getInstance ());
		tagLibrary.addAdder ("toolitem", ToolItemAdder.getInstance ());
		tagLibrary.setSetter ("toolitem", "disabledimage", ImageSetter.getInstance ());
		tagLibrary.setSetter ("toolitem", "hotimage", ImageSetter.getInstance ());
		addWidgetTag (tagLibrary, TrayItem.class, ITEM);
		tagLibrary.addAdder ("trayitem", CallFunctionAdder.getAdder ("setToolTip", TrayItem.class, ToolTip.class));
		addWidgetTag (tagLibrary, TreeColumn.class, ITEM);
		tagLibrary.setCreator ("treeitem", new TreeItemCreator ());
		tagLibrary.inheritTag (ITEM, "treeitem");
		tagLibrary.setSetter ("treeitem", "font", FontSetter.getInstance ());
		tagLibrary.setSetter ("treeitem", "background", ColorSetter.getInstance ());
		tagLibrary.setSetter ("treeitem", "foreground", ColorSetter.getInstance ());

		////////////////////////////////////////////////////////////////////////////
		//
		// SWT 3.2
		//
		////////////////////////////////////////////////////////////////////////////

		addWidgetTag (tagLibrary, ExpandBar.class, COMPOSITE);
		addWidgetTag (tagLibrary, ExpandItem.class, ITEM);
		tagLibrary.addAdder ("expanditem", ExpandItemAdder.getInstance ());

		addWidgetTag (tagLibrary, ToolTip.class, WIDGET);

		tagLibrary.setCreator ("gldata", DefaultCreator.getCreator (GLData.class, true));
		tagLibrary.setCreator ("glcanvas", new GLCanvasCreator ());
		tagLibrary.inheritTag (CANVAS, "glcanvas");

		////////////////////////////////////////////////////////////////////////////
		//
		// SWT Dialogs
		//
		////////////////////////////////////////////////////////////////////////////

		tagLibrary.setSetter (DIALOG, "style", DoNothingSetter.getInstance ());
		addDialogTag (tagLibrary, ColorDialog.class);
		addDialogTag (tagLibrary, DirectoryDialog.class);
		addDialogTag (tagLibrary, FileDialog.class);
		addDialogTag (tagLibrary, FontDialog.class);
		addDialogTag (tagLibrary, MessageBox.class);
		addDialogTag (tagLibrary, PrintDialog.class);
		tagLibrary.setSetter ("printdialog", "scope", new ConstantSetter (PrinterData.class));
		addDialogTag (tagLibrary, XmlDialog.class);
		tagLibrary.setSetter ("xmldialog", "varobj", new HookVariableSetter ("setVarObj", Object.class));

		////////////////////////////////////////////////////////////////////////////
		//
		// SWT Layout
		//
		////////////////////////////////////////////////////////////////////////////

		// FillLayout - the easiest case
		tagLibrary.setCreator ("filllayout", DefaultCreator.getCreator (FillLayout.class));
		tagLibrary.setAdder ("filllayout", DoNothingAdder.getInstance ());
		tagLibrary.setSetter ("filllayout", "type", swtConstantSetter);
		tagLibrary.setCreator ("formdata", DefaultCreator.getCreator (FormData.class));
		tagLibrary.setCreator ("formattachment", DefaultCreator.getCreator (FormAttachmentHelper.class));
		// FormLayout looks like to be difficult to be implemented
		tagLibrary.setCreator ("formlayout", DefaultCreator.getCreator (FormLayout.class));
		tagLibrary.setAdder ("formlayout", DoNothingAdder.getInstance ());
		tagLibrary.setCreator ("formdata", HelperCreator.getCreator (FormDataHelper.class));
		tagLibrary.setCreator ("formattachment", DefaultCreator.getCreator (FormAttachmentHelper.class));
		tagLibrary.setSetter ("formattachment", "alignment", swtConstantSetter);
		// GridLayout
		tagLibrary.setCreator ("gridlayout", DefaultCreator.getCreator (GridLayout.class));
		tagLibrary.setAdder ("gridlayout", DoNothingAdder.getInstance ());
		tagLibrary.setCreator ("griddata", new GridDataCreator ());
		tagLibrary.setAdder ("griddata", LayoutDataAdder.getInstance ());
		ConstantSetter gridDataConstantSetter = new ConstantSetter (GridData.class);
		tagLibrary.setSetter ("griddata", "style", DoNothingSetter.getInstance ());
		tagLibrary.setSetter ("griddata", "heighthint", gridDataConstantSetter);
		tagLibrary.setSetter ("griddata", "horizontalalignment", gridDataConstantSetter);
		tagLibrary.setSetter ("griddata", "verticalalignment", gridDataConstantSetter);
		tagLibrary.setSetter ("griddata", "widthhint", gridDataConstantSetter);
		// RowLayout
		tagLibrary.setCreator ("rowlayout", DefaultCreator.getCreator (RowLayout.class));
		tagLibrary.setAdder ("rowlayout", DoNothingAdder.getInstance ());
		tagLibrary.setSetter ("rowlayout", "type", swtConstantSetter);
		tagLibrary.setCreator ("rowdata", DefaultCreator.getCreator (RowData.class));
		tagLibrary.setAdder ("rowdata", LayoutDataAdder.getInstance ());
		// StackLayout
		tagLibrary.setCreator ("stacklayout", DefaultCreator.getCreator (StackLayout.class));
		tagLibrary.setAdder ("stacklayout", DoNothingAdder.getInstance ());

		////////////////////////////////////////////////////////////////////////////
		//
		// SWT Listeners
		//
		////////////////////////////////////////////////////////////////////////////

		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, ArmListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, BidiSegmentListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, CloseWindowListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, ControlListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, CTabFolder2Listener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, CTabFolderListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, DisposeListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, ExtendedModifyListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, FocusListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, HelpListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, ImageLoaderListener.class);	// only used by ImageLoader, but doesn't hurt to add it here
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, KeyListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, LineBackgroundListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, LineStyleListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, LocationListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, MenuListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, ModifyListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, MouseListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, MouseMoveListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, MouseTrackListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, OpenWindowListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, PaintListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, ProgressListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, SelectionListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, ShellListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, StatusTextListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, TitleListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, TraverseListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, TreeListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, VerifyKeyListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, VerifyListener.class);
		Utils.addListenerSetter (commonTagLibrary, CommonLib.LISTENERS, VisibilityWindowListener.class);
		// DragSource
		commonTagLibrary.setSetter (CommonLib.LISTENERS, "draglistener", new ObjectOrHookVariableSetter ("addDragListener", DragSourceListener.class));
		commonTagLibrary.setSetter (CommonLib.LISTENERS, "dragsourcelistener", new ObjectOrHookVariableSetter ("addDragListener", DragSourceListener.class));
		// DropSource
		commonTagLibrary.setSetter (CommonLib.LISTENERS, "droplistener", new ObjectOrHookVariableSetter ("addDropListener", DropTargetListener.class));
		commonTagLibrary.setSetter (CommonLib.LISTENERS, "droptargetlistener", new ObjectOrHookVariableSetter ("addDragListener", DragSourceListener.class));

		// Used by Accessible only
		//CommonLib.addListenerSetter (taglibrary, LISTENERS, AccessibleListener.class);
		//CommonLib.addListenerSetter (taglibrary, LISTENERS, AccessibleControlListener.class);
		//CommonLib.addListenerSetter (taglibrary, LISTENERS, AccessibleTextListener.class);

		//CommonLib.addListenerSetter (taglibrary, LISTENERS, TextChangeListener.class);	// used internally by StyledText and StyledTextContent
	}

	/**
	 * This function creates the Swing tag library in http://cookxml.sf.net/cookswt/ namespace,
	 * with CookXml CommonLib tag library as its parent.  This is necessary because "component" tag
	 * inherits "listener" tag of CookXml CommonLib.  This function also inserts some additional
	 * setters for commonTagLibrary.
	 *
	 * @param   commonTagLibrary
	 *          The CookXml CommonLib tag library.
	 * @return	the created CookSwt tag library.
	 */
	public static InheritableTagLibrary createTagLibrary (InheritableTagLibrary commonTagLibrary)
	{
		InheritableTagLibrary tagLibrary = new InheritableTagLibrary (commonTagLibrary);
		tagLibrary.setNameSpace (NAMESPACE);

		setupTags (tagLibrary, commonTagLibrary);

		return tagLibrary;
	}
}
