/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookswt.creator;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Member;
import java.util.StringTokenizer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Widget;
import org.w3c.dom.Element;

import cookxml.cookswt.exception.BadStyleException;
import cookxml.cookswt.exception.NoWidgetConstructorException;
import cookxml.core.CookXml;
import cookxml.core.DecodeEngine;
import cookxml.core.doclet.DocletActionReporter;
import cookxml.core.exception.CookXmlException;
import cookxml.core.interfaces.Creator;
import cookxml.core.interfaces.TagLibrary;

/**
 * This is a creator that creates a SWT widget object using its pretty much
 * standard (parent, style) format of construction.  The parent is looked up
 * through the XML hierarchy, and the style is looked up using the "style"
 * attribute. 
 *
 * @cxattr	style
 * 			The style of the widget.  See {@link org.eclipse.swt.SWT} and the
 * 			widget class information for the styles allowed.  Several styles
 * 			can be chained together using '|' (e.g. <tt>H_SCROLL | V_SCROLL</tt> ).
 *
 * @since CookSwt 1.0
 * @author Heng Yuan
 */
public class WidgetCreator implements Creator, DocletActionReporter
{
	public static String STYLE_ATTR = "style";

	/**
	 * This function used to generate a creator for Widget subclasses or
	 * classes that uses similar approach that take a Widget as the first
	 * argument and style as the second argument for their constructors.
	 *
	 * @param	targetClass
	 * 			the class object
	 * @return	a default creator if a default constructor can be found.
	 */
	public static Creator getCreator (Class targetClass) throws CookXmlException
	{
		try
		{
			Constructor[] ctors = targetClass.getConstructors ();
			// find the constructor that requires widget parent and a int style
			for (int i = 0; i < ctors.length; ++i)
			{
				Constructor ctor = ctors[i];
				Class[] params = ctor.getParameterTypes ();
				if (params.length == 2 &&
					Widget.class.isAssignableFrom (params[0]) &&
					"int".equals (params[1].getName ()))
				{
					try
					{
						ctor.setAccessible (true);
					}
					catch (Exception ex)
					{
					}
					return new WidgetCreator (ctor);
				}
			}
		}
		catch (Exception ex)
		{
		}
		CookXml.getDefaultExceptionHandler ().handleException (null, new NoWidgetConstructorException (targetClass));
		return null;
	}

	private final Constructor m_ctor;

	private WidgetCreator (Constructor ctor)
	{
		m_ctor = ctor;
	}

	public Object create (String parentNS, String parentTag, Element elm, Object parentObj, DecodeEngine decodeEngine) throws Exception
	{
		// search the parent tree and try to find the parent that is a widget
		Object widgetParent = parentObj;
		Class targetClass = m_ctor.getParameterTypes ()[0];
		while (widgetParent != null && !targetClass.isInstance (widgetParent))
			widgetParent = decodeEngine.getParent (widgetParent);
		if (widgetParent == null)
			return null;
		int style = getStyle (elm, decodeEngine);
		decodeEngine.setDoAdd (false);
		return m_ctor.newInstance (new Object[]{widgetParent, new Integer (style)});
	}

	public Object editFinished (String parentNS, String parentTag, Element elm, Object parentObj, Object obj, DecodeEngine decodeEngine)
	{
		decodeEngine.addChild (parentNS, parentTag, elm, parentObj, obj);
		return obj;
	}

	public static int getStyle (Element elm, DecodeEngine decodeEngine) throws CookXmlException
	{
		return getStyle (elm, SWT.NONE, decodeEngine);
	}

	public static int getStyle (Element elm, int defaultVal, DecodeEngine decodeEngine) throws CookXmlException
	{
		int returnVal = defaultVal;
		try
		{
			String value = elm.getAttribute (STYLE_ATTR);
			if (value == null || value.length () == 0)
				return returnVal;
			StringTokenizer tokenizer = new StringTokenizer (value, "| ");
			do
			{
				String tok = tokenizer.nextToken ();
				Field field = SWT.class.getField (tok);
				returnVal |= field.getInt (null);
			}
			while (tokenizer.hasMoreElements ());
		}
		catch (Exception ex)
		{
			decodeEngine.handleException (null, new BadStyleException (SWT.class, elm.getAttribute (STYLE_ATTR)));
		}
		return returnVal;
	}

	public Object[] getActions (TagLibrary tagLibrary, String tagNS, String tag, String attrNS, String attr, Class cl)
	{
		return new Member[]{ m_ctor };
	}
}
