/*
 * (c) Copyright 2006 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookxmldoc;

import java.lang.reflect.Field;
import java.lang.reflect.Member;
import java.lang.reflect.Method;
import java.util.*;

import com.sun.javadoc.*;

import cookxml.cookxmldoc.impl.*;
import cookxml.cookxmldoc.interfaces.TagDoc;
import cookxml.cookxmldoc.interfaces.TagLibraryDoc;
import cookxml.core.doclet.DocletActionReporter;
import cookxml.core.doclet.DocletSetter;
import cookxml.core.doclet.DocletTagLibrary;
import cookxml.core.interfaces.Adder;
import cookxml.core.interfaces.Converter;
import cookxml.core.interfaces.Creator;
import cookxml.core.interfaces.Setter;
import cookxml.core.taglibrary.InheritableTagLibrary;
import cookxml.core.util.ClassUtils;
import cookxml.core.util.DocletUtils;

/**
 * This class does the internal processing of a DocletTagLibrary and convert it into
 * a processed structure TagLibraryDoc.
 *
 * @author Heng Yuan
 * @version $Id: DocBuilder.java 262 2007-06-10 16:43:48Z coconut $
 * @since CookXmlDoc 0.1
 */
class DocBuilder
{
	private static HashSet guessAttributes (DocletTagLibrary tagLibrary, String ns, String tag, Class cl)
	{
		HashSet set = new HashSet ();
		addAttributes (set, tagLibrary.getSetter (ns, tag, null, null), cl);
		addAttributes (set, tagLibrary.getSetter (null, null, null, null), cl);
		return set;
	}

	private static void addAttributes (Set set, Setter setter, Class cl)
	{
		if (setter != null && setter instanceof DocletSetter)
		{
			String[] attributes = ((DocletSetter)setter).getAttributes (cl);
			for (int i = 0; i < attributes.length; ++i)
				set.add (attributes[i]);
		}
	}

	private static void buildTag (TagDocImpl tagDoc)
	{
		String ns = tagDoc.getNameSpace ().getName ();
		DocletTagLibrary tagLibrary = tagDoc.getNameSpace ().getTagLibraryDoc ().getTagLibrary ();

		// now deal with tag class
		Creator creator = tagDoc.getCreator ();
		String tag = tagDoc.getName ();

		String className = null;
		String helperClassName = null;

		if (creator != null)
		{
			// first determine the direct class this tag corresponds to
			if (creator instanceof DocletActionReporter)
			{
				// we can only deal with a single member at present
				Member member = (Member)((DocletActionReporter)creator).getActions (tagLibrary, ns, tag, null, null, null)[0];
				className = member.getDeclaringClass ().getName ();
			}
			else
			{
				ClassDoc classDoc = JavaDocUtils.getClassDoc (creator.getClass ().getName ());
				if (classDoc != null)
				{
					SeeTag[] seeTags = classDoc.seeTags ();
					if (seeTags != null && seeTags.length > 0)
					{
						ClassDoc refDoc = seeTags[0].referencedClass ();
						if (refDoc != null)
							className = refDoc.qualifiedName ();
					}
				}
			}
		}

		// now check if the class is a helper class.  We do so by checking its @cxhelper
		// tag for the javadoc for the class.
		if (className != null)
		{
			ClassDoc classDoc = JavaDocUtils.getClassDoc (className);
			if (classDoc != null)
			{
				String name = JavaDocUtils.getHelper (className);
				if (name != null)
				{
					helperClassName = className;
					className = name;
				}
			}
		}

		tagDoc.setClassName (className);
		tagDoc.setHelperClassName (helperClassName);

		String comment = null;
		// also check the creator JavaDoc.
		if (creator != null)
		{
			Doc doc = JavaDocUtils.getClassDoc (creator.getClass ().getName ());
			comment = JavaDocUtils.getTagComment (doc);
			tagDoc.setExamples (JavaDocUtils.getExampleComments (doc));
		}
		if (comment == null && helperClassName != null)
		{
			Doc doc = JavaDocUtils.getClassDoc (helperClassName);
			comment = JavaDocUtils.getTagComment (doc);
			tagDoc.setExamples (JavaDocUtils.getExampleComments (doc));
		}
		tagDoc.setTagComment (comment);

		if (className != null)
		{
			ClassDoc classDoc = JavaDocUtils.getClassDoc (className);
			if (classDoc != null)
			{
				tagDoc.setDocComment (JavaDocUtils.getDocComment (classDoc));
				tagDoc.setExamples (JavaDocUtils.getExampleComments (classDoc));
			}
		}

		// we now get the class information
		Class cl = null;
		try
		{
			String inspectClass = helperClassName == null ? className : helperClassName;
			if (inspectClass != null)
				cl = Class.forName (inspectClass, false, Main.getClassLoader ());
		}
		catch (Exception ex)
		{
		}

		// now deal with attributes
		AttrDocImpl[] ctorAttrs = null;
		if (creator != null)
		{
			ctorAttrs = getAttr (tagDoc, creator.getClass ().getName ());
		}

		// get all the attribute
		Map attrMap = tagLibrary.getAttributes (ns, tag);

		// see if additional attributes can be discovered.
		if (cl != null)
		{
			HashSet set = guessAttributes (tagLibrary, ns, tag, cl);
			String[] attrs = (String[])set.toArray (new String[set.size ()]);
			HashMap map = new HashMap ();
			map.put (ns, attrs);
			attrMap = DocletUtils.mergeMap (attrMap, map);
		}

		for (Iterator iter = attrMap.entrySet ().iterator (); iter.hasNext ();)
		{
			Map.Entry entry = (Map.Entry)iter.next ();
			String attrNS = (String)entry.getKey ();
			String[] attrs = (String[])entry.getValue ();

			if (ns != null && ns.equals (attrNS))
			{
				if (ctorAttrs != null && ctorAttrs.length > 0)
				{
					HashSet set = new HashSet ();
					for (int i = 0; i < attrs.length; ++i)
						set.add (attrs[i]);
					for (int i = 0; i < ctorAttrs.length; ++i)
						set.remove (ctorAttrs[i].getName ());
					attrs = (String[])set.toArray (new String[set.size ()]);
				}
			}

			if (attrs != null)
			{
				for (int i = 0; i < attrs.length; ++i)
				{
					String attr = attrs[i];
					// lookup the setter for an attribute
					Setter setter = tagLibrary.getSetter (ns, tag, ns, attr);
					if (setter == null)
					{
						setter = tagLibrary.getSetter (ns, tag, ns, null);
						if (setter == null)
						{
							setter = tagLibrary.getSetter (ns, null, ns, attr);
							if (setter == null)
							{
								setter = tagLibrary.getSetter (ns, null, ns, null);
							}
						}
					}
					if (setter == null)
						continue;
					computerSetterDoc (tagDoc, setter, tagLibrary, ns, tag, attrNS, attrs[i], cl);
				}
			}
		}

		// now deal with adders
		Adder adder = tagLibrary.getAdder (ns, tag);
		if (adder == null)
			adder = tagLibrary.getAdder (ns, null);

		if (adder != null)
		{
			computeAdderDoc (tagDoc, adder, tagLibrary, ns, tag);
		}
	}

	private static void buildNameSpace (TagLibraryDocImpl tagLibraryDoc, String ns)
	{
		DocletTagLibrary tagLibrary = tagLibraryDoc.getTagLibrary ();

		NameSpaceDocImpl nsDoc = new NameSpaceDocImpl (tagLibraryDoc, ns);
		tagLibraryDoc.addNameSpace (nsDoc);

		// deal with converters
		Map converterMap = tagLibrary.getConverters (ns);
		for (Iterator iter = converterMap.entrySet ().iterator (); iter.hasNext ();)
		{
			Map.Entry entry = (Map.Entry)iter.next ();
			ConverterDocImpl converterDoc = new ConverterDocImpl ((Class)entry.getKey (), (Converter)entry.getValue ());
			Doc classDoc = JavaDocUtils.getClassDoc (entry.getValue ().getClass ().getName ());
			converterDoc.setDocComment (JavaDocUtils.getDocComment (classDoc));
			converterDoc.setDocComment (JavaDocUtils.getTagComment (classDoc));
			nsDoc.addConverter (converterDoc);
		}

		// deal with tags
		String[] tags = (String[])tagLibrary.getTags ().get (ns);

		if (tags == null)
			return;

		// remove tags that's already being in the NameSpaceDoc
		TagDoc[] existingDocs = nsDoc.getTags ();
		if (existingDocs.length > 0)
		{
			HashSet set = new HashSet ();
			for (int i = 0; i < tags.length; ++i)
				set.add (tags[i]);
			for (int i = 0; i < existingDocs.length; ++i)
				set.remove (existingDocs[i].getName ());
			tags = (String[])set.toArray (new String[set.size ()]);
		}

		// create non-hidden tags
		for (int i = 0; i < tags.length; ++i)
		{
			String tag = tags[i];
			TagDocImpl tagDoc = new TagDocImpl (nsDoc, tagLibrary.getCreator (ns, tag), tag);
			nsDoc.addTag (tagDoc);
		}

		// now set tags as all existing tags
		if (existingDocs.length > 0)
		{
			HashSet set = new HashSet ();
			for (int i = 0; i < tags.length; ++i)
				set.add (tags[i]);
			for (int i = 0; i < existingDocs.length; ++i)
				set.add (existingDocs[i].getName ());
			tags = (String[])set.toArray (new String[set.size ()]);
		}

		// establish parent/child tags and create hidden tags
		while (tags.length > 0)
		{
			LinkedList newTags = new LinkedList ();
			for (int i = 0; i < tags.length; ++i)
			{
				String tag = tags[i];
				String[] parents = null;
				if (tagLibrary instanceof InheritableTagLibrary)
				{
					LinkedList list = ((InheritableTagLibrary)tagLibrary).getParentTags (ns, tag);
					if (list != null && list.size () > 0)
						parents = (String[])list.toArray (new String[list.size ()]);
				}
				if (parents == null)
					continue;
				TagDocImpl childTagDoc = (TagDocImpl)nsDoc.getTag (tag);
				for (int j = 0; j < parents.length; ++j)
				{
					String parentTag = parents[j];
					TagDocImpl parentTagDoc;
					int index = parentTag.indexOf (':');
					if (index > 0)
					{
						String parentNS = parentTag.substring (index + 1);
						parentTag = parentTag.substring (0, index);
						NameSpaceDocImpl otherNSDoc = (NameSpaceDocImpl)tagLibraryDoc.getNameSpace (parentNS);
						parentTagDoc = (TagDocImpl)otherNSDoc.getTag (parentTag);
						if (parentTagDoc == null)
						{
							parentTagDoc = new TagDocImpl (otherNSDoc, tagLibrary.getCreator (parentNS, parentTag), parentTag);
							otherNSDoc.addTag (parentTagDoc);
						}
					}
					else
						parentTagDoc = (TagDocImpl)nsDoc.getTag (parentTag);
					if (parentTagDoc == null)
					{
						parentTagDoc = new TagDocImpl (nsDoc, null, parentTag);
						nsDoc.addTag (parentTagDoc);
						newTags.add (parentTag);
					}
					childTagDoc.addParent (parentTagDoc);
				}
			}
			tags = (String[])newTags.toArray (new String[newTags.size ()]);
		}
	}

	private static void buildTags (NameSpaceDocImpl nsDoc)
	{
		// generate tag informations
		TagDocImpl[] tagDocs = (TagDocImpl[])nsDoc.getTags ();
		for (int i = 0; i < tagDocs.length; ++i)
			buildTag (tagDocs[i]);
	}

	public static TagLibraryDoc execute (DocletTagLibrary tagLibrary)
	{
		TagLibraryDocImpl tagLibraryDoc = new TagLibraryDocImpl (tagLibrary);

		String[] nsArray = tagLibrary.getNameSpaces ();

		for (int i = 0; i < nsArray.length; ++i)
			buildNameSpace (tagLibraryDoc, nsArray[i]);

		NameSpaceDocImpl[] nsDocs = (NameSpaceDocImpl[])tagLibraryDoc.getNameSpaces ();
		for (int i = 0; i < nsDocs.length; ++i)
			buildTags (nsDocs[i]);

		return tagLibraryDoc;
	}

	private static void computerSetterDoc (TagDocImpl tagDoc, Setter setter, DocletTagLibrary tagLibrary, String tagNS, String tag, String attrNS, String attr, Class cl)
	{
		ClassDoc setterClassDoc = JavaDocUtils.getClassDoc (setter.getClass ().getName ());
		String classComment = JavaDocUtils.getDocComment (setterClassDoc);
		String tagComment = JavaDocUtils.getTagComment (setterClassDoc);

		if (setter instanceof DocletActionReporter)
		{
			Object[] actions = ((DocletActionReporter)setter).getActions (tagLibrary, tagNS, tag, attrNS, attr, cl);
			for (int j = 0; j < actions.length; ++j)
			{
				AttrDocImpl attrDoc = new AttrDocImpl (tagDoc, attr, setter);
				// The following two comments maybe overridden
				attrDoc.setDocComment (classComment);
				attrDoc.setTagComment (tagComment);
				tagDoc.addAttribute (attrDoc);

				Object member = actions[j];

				// now consider the value type
				try
				{
					if (member instanceof Field)
					{
						attrDoc.setValueClass (ClassUtils.getEquivalentClass (((Field)member).getType ()));
						attrDoc.setMember ((Member)member);

						Doc memberDoc = JavaDocUtils.getMemberDoc ((Field)member, true);
						attrDoc.setDocComment (JavaDocUtils.getDocComment (memberDoc));
						attrDoc.setTagComment (JavaDocUtils.getTagComment (memberDoc));
					}
					else if (member instanceof Method)
					{
						Class[] params = ((Method)member).getParameterTypes ();
						attrDoc.setMember ((Member)member);
						if (params.length == 1)
							attrDoc.setValueClass (ClassUtils.getEquivalentClass (params[0]));

						Doc memberDoc = JavaDocUtils.getMemberDoc ((Method)member, false);
						attrDoc.setDocComment (JavaDocUtils.getDocComment (memberDoc));
						attrDoc.setTagComment (JavaDocUtils.getTagComment (memberDoc));
					}
					else if (member instanceof Setter)
					{
						Method setterMethod = Utils.getSetterMethod ((Setter)member);
						attrDoc.setMember (setterMethod);

						Doc memberDoc = JavaDocUtils.getMemberDoc (setterMethod, false);
						attrDoc.setDocComment (JavaDocUtils.getDocComment (memberDoc));
						attrDoc.setTagComment (JavaDocUtils.getTagComment (memberDoc));

						Tag[] setterTag = JavaDocUtils.getSetterTag (memberDoc);
						if (setterTag.length > 0)
						{
							String[] paramComment = JavaDocUtils.getParamComment (setterTag[0].text ());
							if (paramComment != null)
							{
								attrDoc.setTagComment (paramComment[1]);
								attrDoc.setValueClass (Class.forName (paramComment[0], false, Main.getClassLoader ()));
							}
						}
					}
					else if (member instanceof Adder)
					{
						Method adderMethod = Utils.getAdderMethod ((Adder)member);
						attrDoc.setMember (adderMethod);

						Doc memberDoc = JavaDocUtils.getMemberDoc (adderMethod, false);
						attrDoc.setDocComment (JavaDocUtils.getDocComment (memberDoc));
						attrDoc.setTagComment (JavaDocUtils.getTagComment (memberDoc));

						Tag[] setterTag = JavaDocUtils.getSetterTag (memberDoc);
						if (setterTag.length > 0)
						{
							String[] paramComment = JavaDocUtils.getParamComment (setterTag[0].text ());
							if (paramComment != null)
							{
								attrDoc.setTagComment (paramComment[1]);
								attrDoc.setValueClass (Class.forName (paramComment[0], false, Main.getClassLoader ()));
							}
						}
					}
				}
				catch (Exception ex)
				{
				}
			}
			if (tagDoc.getCreator () == null && actions.length == 0)	// virtual tags
			{
				AttrDocImpl attrDoc = new AttrDocImpl ((NameSpaceDocImpl)tagDoc.getNameSpace (), tagDoc, attr, setter, classComment, tagComment);
				attrDoc.setDocComment (classComment);
				attrDoc.setTagComment (tagComment);
				tagDoc.addAttribute (attrDoc);
			}
		}
		else
		{
			Method setterMethod = Utils.getSetterMethod (setter);

			AttrDocImpl attrDoc = new AttrDocImpl (tagDoc, attr, setter);
			tagDoc.addAttribute (attrDoc);
			attrDoc.setMember (setterMethod);

			Doc memberDoc = JavaDocUtils.getMemberDoc (setterMethod, false);
			attrDoc.setDocComment (JavaDocUtils.getDocComment (memberDoc));
			attrDoc.setTagComment (JavaDocUtils.getTagComment (memberDoc));
		}
	}

	private static void computeAdderDoc (TagDocImpl tagDoc, Adder adder, DocletTagLibrary tagLibrary, String ns, String tag)
	{
		LinkedList doneList = new LinkedList ();
		LinkedList todoList = new LinkedList ();

		Class cl = null;
		try
		{
			String className = tagDoc.getHelperClassName ();
			if (className == null)
				className = tagDoc.getClassName ();
			cl = Class.forName (className, false, Main.getClassLoader ());
		}
		catch (Throwable t)
		{
		}

		todoList.add (new Object[]{ null, adder });

		while (todoList.size () > 0)
		{
			Object[] pair = (Object[])todoList.removeFirst ();
			adder = (Adder)pair[0];
			Object key = pair[1];
			doneList.add (key);
			try
			{
				if (key instanceof Field)
				{
					AdderDocImpl adderDoc = new AdderDocImpl (tagDoc, adder);
					adderDoc.setChildClass (ClassUtils.getEquivalentClass (((Field)key).getType ()));
					MemberDoc memberDoc = JavaDocUtils.getMemberDoc ((Field)key, true);
					adderDoc.setDocComment (JavaDocUtils.getDocComment (memberDoc));
					adderDoc.setTagComment (JavaDocUtils.getTagComment (memberDoc));
					adderDoc.setMember ((Field)key);
					tagDoc.addAdder (adderDoc);
				}
				else if (key instanceof Method)
				{
					AdderDocImpl adderDoc = new AdderDocImpl (tagDoc, adder);
					Class[] params = ((Method)key).getParameterTypes ();
					if (params.length == 1)
						adderDoc.setChildClass (ClassUtils.getEquivalentClass (params[0]));
					MemberDoc memberDoc = JavaDocUtils.getMemberDoc ((Method)key, false);
					adderDoc.setDocComment (JavaDocUtils.getDocComment (memberDoc));
					adderDoc.setTagComment (JavaDocUtils.getTagComment (memberDoc));
					adderDoc.setMember ((Method)key);
					tagDoc.addAdder (adderDoc);
				}
				else if (key instanceof Adder)
				{
					if (key instanceof DocletActionReporter)
					{
						Object[] newActions = ((DocletActionReporter)key).getActions (tagLibrary, ns, tag, null, null, cl);
						for (int i = 0; i < newActions.length; ++i)
						{
							if (!doneList.contains (newActions[i]))
								todoList.add (new Object[]{ key, newActions[i] });
						}
					}
					addAdderDoc (tagDoc, (Adder)key);
				}
				else if (key instanceof Setter)	// we'd better not to have an object that is both an adder and a setter
				{
					AdderDocImpl adderDoc = new AdderDocImpl (tagDoc, adder);
					Method setterMethod = Utils.getSetterMethod ((Setter)key);
					adderDoc.setMember (setterMethod);

					MemberDoc memberDoc = JavaDocUtils.getMemberDoc (setterMethod, false);
					adderDoc.setDocComment (JavaDocUtils.getDocComment (memberDoc));
					adderDoc.setTagComment (JavaDocUtils.getTagComment (memberDoc));
				}
			}
			catch (Exception ex)
			{
			}
		}
	}

	/**
	 * This function adds AdderDocImpl to the tagDoc for adders
	 * that are not DocletActionReporters.
	 *
	 * @param	tagDoc
	 * 			the tagDoc
	 * @param	adder
	 * 			the adder
	 */
	static void addAdderDoc (TagDocImpl tagDoc, Adder adder)
	{
		boolean hasDoc = getAdderDoc (tagDoc, adder) != null;

		if (hasDoc || adder instanceof DocletActionReporter)
			return;

		Method adderMethod = Utils.getAdderMethod (adder);

		AdderDocImpl adderDoc = new AdderDocImpl (tagDoc, adder);
		tagDoc.addAdder (adderDoc);
		adderDoc.setMember (adderMethod);

		MemberDoc memberDoc = JavaDocUtils.getMemberDoc (adderMethod, false);
		adderDoc.setDocComment (JavaDocUtils.getDocComment (memberDoc));
		adderDoc.setTagComment (JavaDocUtils.getTagComment (memberDoc));
	}

	/**
	 * This function checks if a class really is a helper class.  If
	 * so, it returns the actual class.  If not, it returns null.
	 *
	 * @param	tagDoc
	 * 			the tag the adder is associated with.
	 * @param	adder
	 *			the adder.
	 * @return	the actual class name if helpClassName is really a helper class
	 */
	private static AdderDocImpl[] getAdderDoc (TagDocImpl tagDoc, Adder adder)
	{
		ClassDoc classDoc = JavaDocUtils.getClassDoc (adder.getClass ().getName ());
		if (classDoc != null)
		{
			Tag[] adderTags = JavaDocUtils.getAdderTag (classDoc);
			if (adderTags != null && adderTags.length > 0)
			{
				Collection adderDocs = new LinkedList ();
				for (int i = 0; i < adderTags.length; ++i)
				{
					String[] paramComment = JavaDocUtils.getParamComment (adderTags[i].text ());
					if (paramComment == null)
						continue;
					String className = paramComment[0];
					String comment = paramComment[1];
					AdderDocImpl adderDoc = new AdderDocImpl (tagDoc, adder);
					tagDoc.addAdder (adderDoc);
					adderDoc.setTagComment (comment);
					adderDoc.setMember (Utils.getAdderMethod (adder));
					try
					{
						adderDoc.setChildClass (Class.forName (className, false, Main.getClassLoader ()));
					}
					catch (Exception ex)
					{
					}
					adderDocs.add (adderDoc);
				}
				return (AdderDocImpl[])adderDocs.toArray (new AdderDocImpl[adderDocs.size ()]);
			}
		}
		return null;
	}

	/**
	 * This function checks if a class really is a helper class.  If
	 * so, it returns the actual class.  If not, it returns null.
	 *
	 * @param	tagDoc
	 * 			the tag to find the attributes
	 * @param	className
	 *			the Creator class.
	 * @return	the actual class name if helpClassName is really a helper class
	 */
	private static AttrDocImpl[] getAttr (TagDocImpl tagDoc, String className)
	{
		ClassDoc classDoc = JavaDocUtils.getClassDoc (className);
		if (classDoc != null)
		{
			Tag[] attrTags = JavaDocUtils.getAttrTag (classDoc);
			if (attrTags.length > 0)
			{
				Collection attrDocs = new LinkedList ();
				for (int i = 0; i < attrTags.length; ++i)
				{
					String[] paramComment = JavaDocUtils.getParamComment (attrTags[i].text ());
					if (paramComment != null)
					{
						String attr = paramComment[0];
						String comment = paramComment[1];
						AttrDocImpl attrDoc = new AttrDocImpl ((NameSpaceDocImpl)tagDoc.getNameSpace (), tagDoc, attr, null, null, comment);
						tagDoc.addAttribute (attrDoc);
						attrDocs.add (attrDoc);
					}
				}
				return (AttrDocImpl[])attrDocs.toArray (new AttrDocImpl[attrDocs.size ()]);
			}
		}
		return null;
	}
}
