/*
 * (c) Copyright 2006 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookxmldoc;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.lang.reflect.Field;
import java.lang.reflect.Member;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import com.sun.javadoc.*;
import com.sun.tools.javadoc.Main;

/**
 * This class contains utility functions used in retrieving Javadoc.
 *
 * @author Heng Yuan
 * @version $Id: JavaDocUtils.java 241 2007-06-07 04:03:39Z coconut $
 * @since CookXmlDoc 0.1
 */
public class JavaDocUtils
{
	/** the JavaDoc tag name for finding out if a class is a helper class. */
	public static String TAG_HELPER = "@cxhelper";
	/**
	 * the JavaDoc tag name the general documentation specific for the tag library.
	 * The CookXmlDoc grabs both the regular JavaDoc comment and the specific tag
	 * library comment.  It is up to the doclet to determine what to do with them.
	 */
	public static String TAG_DOC = "@cxdoc";
	/** the JavaDoc tag name for Creators to document the attributes. */
	public static String TAG_ATTR = "@cxattr";
	/** the JavaDoc tag name for Adders to document the the child class and comment. */
	public static String TAG_ADDER = "@cxadder";
	/** the JavaDoc tag name for Setters to document the attribute value class and comment. */
	public static String TAG_SETTER = "@cxsetter";
	/** the JavaDoc tag name for examples. */
	public static String TAG_EXAMPLE = "@cxexample";

	private final static Map s_classDocMap = new HashMap ();

	/** Maintains the source path used in class doc generation. */
	private static String s_sourcePath;
	/** the Java source compatibility level. */
	private static String s_source = "1.4";
	/** whether or not to show JavaDoc running messages. */
	private static boolean s_quiet;

	/** Used for hidding error/warning messages. */
	private final static PrintWriter s_dummyWriter = new PrintWriter (new OutputStream ()
	{
		public void write (int b)
		{
		}
		public void write (byte b[])
		{
		}
		public void write (byte b[], int off, int len)
		{
		}
	});

	/**
	 * Do not call this function.
	 * <p>
	 * This function is used internally to be called by the JavaDoc tool.
	 *
	 * @param	root
	 *			the root of the processed JavaDocs.
	 * @return	true
	 */
	public static boolean start (RootDoc root)
	{
		ClassDoc[] classDocs = root.classes ();
		for (int i = 0; i < classDocs.length; ++i)
			s_classDocMap.put (classDocs[i].qualifiedName (), classDocs[i]);
		return true;
	}

	/**
	 * This function is used to initate internal RootDoc which is then used for
	 * query later on by getClassDoc (className).
	 *
	 * @see #getClassDoc(String className)
	 * @param	sourcePath
	 *			the source directory
	 */
	public static void setSourcePath (String sourcePath)
	{
		s_sourcePath = sourcePath;
	}

	/**
	 * This function is used to set the Java source compatibility leve.
	 *
	 * @param	source
	 *			the Java source compatibility
	 */
	public static void setSource (String source)
	{
		s_sourcePath = source;
	}

	/**
	 * This function is used to suppress JavaDoc parsing messages.
	 *
	 * @param	b
	 *			if true, set to quiet, other wise, not.
	 */
	public static void setQuiet (boolean b)
	{
		s_quiet = b;
	}

	/**
	 * This function is used to obtain the ClassDoc (JavaDoc Java representation) of the
	 * source class file.  setSourcePath(southPath) function must have been called beforehand.
	 *
	 * @see #setSourcePath(String classPath)
	 * @param	className
	 *			the class name
	 * @return	the ClassDoc associated with the class.
	 */
	static ClassDoc getClassDoc (String className)
	{
		int index = className.indexOf ('$');
		if (index > 0)
		{
			String c = className.substring (0, index);
			ClassDoc classDoc = getClassDoc (c);
			if (classDoc == null)
				return null;
			return getInnerClassDoc (classDoc, className.substring (index + 1));
		}

		ClassDoc doc = (ClassDoc)s_classDocMap.get (className);
		if (doc != null)
			return doc;
		/*
		if (s_rootDoc != null)
		{
			doc = s_rootDoc.classNamed (className);
			if (doc != null)
				return doc;
		}
		*/
		String packageName = className.substring (0, className.lastIndexOf ('.'));
		String[] args = new String[]{ "-source", s_source, "-sourcepath", s_sourcePath, packageName};
		if (s_quiet)
			Main.execute ("HtmlDoclet", s_dummyWriter, s_dummyWriter, s_dummyWriter, JavaDocUtils.class.getName (), args);
		else
			Main.execute ("HtmlDoclet", JavaDocUtils.class.getName (), args);
		return (ClassDoc)s_classDocMap.get (className);
	}

	/**
	 * This function get the ClassDoc associated with a innerClass.
	 *
	 * @param	containingClassDoc
	 *			the class for the containing class.
	 * @param	innerClass
	 *			the inner class name
	 * @return	the document for the inner class.
	 */
	static ClassDoc getInnerClassDoc (ClassDoc containingClassDoc, String innerClass)
	{
		ClassDoc[] innerClasses = containingClassDoc.innerClasses ();
		for (int i = 0; i < innerClasses.length; ++i)
		{
			if (innerClasses[i].name ().equals (innerClass))
				return innerClasses[i];
		}
		return null;
	}

	static MemberDoc getMemberDoc (Member member, boolean accessible)
	{
		ClassDoc classDoc = getClassDoc (member.getDeclaringClass ().getName ());
		if (classDoc == null)
			return null;
		if (member instanceof Field)
		{
			FieldDoc[] fieldDoc = classDoc.fields (!accessible);
			if (fieldDoc == null)
				return null;
			String fieldName = member.getName ();
			for (int i = 0; i < fieldDoc.length; ++i)
			{
				if (fieldName.equals (fieldDoc[i].name ()))
					return fieldDoc[i];
			}
			return null;
		}
		else if (member instanceof Method)
		{
			MemberDoc[] memberDoc = classDoc.methods (true);
			if (memberDoc == null)
				return null;
			String memberString = member.toString ();
			int parenStart = memberString.indexOf ('(');
			int parenEnd = memberString.indexOf (')');
			memberString = memberString.substring (parenStart + 1, parenEnd);
			memberString = memberString.replaceAll (",", ", ");
			for (int i = 0; i < memberDoc.length; ++i)
			{
				if (!memberDoc[i].name ().equals (member.getName ()))
					continue;
				String docString = memberDoc[i].toString ();
				parenStart = docString.indexOf ('(');
				parenEnd = docString.indexOf (')');
				docString = docString.substring (parenStart + 1, parenEnd);
				if (memberString.equals (docString))
					return memberDoc[i];
			}
			return null;
		}
		else // constructor
		{
			ConstructorDoc[] ctorDoc = classDoc.constructors (true);
			if (ctorDoc == null)
				return null;
			for (int i = 0; i < ctorDoc.length; ++i)
			{
				System.out.println ("ctor: " + ctorDoc[i]);
			}
			return null;
		}
	}

	/**
	 * This function checks if a class really is a helper class.  If
	 * so, it returns the actual class.  If not, it returns null.
	 *
	 * @param	helperClassName
	 *			the class to be checked.
	 * @return	the actual class name if helpClassName is really a helper class
	 */
	static String getHelper (String helperClassName)
	{
		ClassDoc classDoc = getClassDoc (helperClassName);
		if (classDoc != null)
		{
			Tag[] helperTags = classDoc.tags (TAG_HELPER);
			if (helperTags.length > 0)
				return helperTags[0].text ();
		}
		return null;
	}

	static Tag[] getAttrTag (Doc doc)
	{
		if (doc == null)
			return null;
		return doc.tags (TAG_ATTR);
	}

	static Tag[] getSetterTag (Doc doc)
	{
		if (doc == null)
			return null;
		return doc.tags (TAG_SETTER);
	}

	static Tag[] getAdderTag (Doc doc)
	{
		if (doc == null)
			return null;
		return doc.tags (TAG_ADDER);
	}

	/**
	 * Obtain the Javadoc comment.
	 *
	 * @param	doc
	 *			the Javadoc to check.
	 * @return	the Javadoc comment.
	 */
	static String getDocComment (Doc doc)
	{
		if (doc == null)
			return null;
		return doc.commentText ();
	}

	/**
	 * Obtain the tag comment.
	 *
	 * @param	doc
	 *			the Javadoc to check.
	 * @return	the tag comment.
	 */
	static String getTagComment (Doc doc)
	{
		if (doc == null)
			return null;
		Tag[] tags = doc.tags (TAG_DOC);
		if (tags.length > 0)
			return tags[0].text ();
		return null;
	}

	/**
	 * Obtain the example tags.  There can be a number of them.
	 *
	 * @param	doc
	 *			the Javadoc to check.
	 * @return	the example tags.
	 */
	static String[] getExampleComments (Doc doc)
	{
		if (doc == null)
			return null;
		Tag[] tags = doc.tags (TAG_EXAMPLE);
		String[] examples = new String[tags.length];
		for (int i = 0; i < examples.length; ++i)
			examples[i] = tags[i].text ();
		return examples;
	}

	static String[] getParamComment (String text)
	{
		text = text.trim ();
		StringTokenizer tok = new StringTokenizer (text, " \t\r\n");
		if (!tok.hasMoreTokens ())
			return null;
		String className = tok.nextToken ();
		String comment = text.substring (className.length ()).trim ();
		return new String[]{ className, comment };
	}
}
