/*
 * (c) Copyright 2006 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookxmldoc;

import java.io.File;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Collection;
import java.util.LinkedList;
import java.util.StringTokenizer;

import cookxml.cookxmldoc.htmldoclet.HtmlDoclet;
import cookxml.cookxmldoc.interfaces.Doclet;
import cookxml.cookxmldoc.interfaces.OptionParser;
import cookxml.cookxmldoc.interfaces.TagLibraryDoc;
import cookxml.core.doclet.DocletTagLibrary;

/**
 * @author Heng Yuan
 * @version $Id: Main.java 215 2007-06-06 03:59:41Z coconut $
 * @since CookXmlDoc 0.1
 */
public class Main
{
	public static String OPTION_HELP = "-help";
	public static String OPTION_SOURCE = "-source";
	public static String OPTION_CLASS_PATH = "-classpath";
	public static String OPTION_SOURCE_PATH = "-sourcepath";
	public static String OPTION_TAG_LIBRARY = "-taglibrary";
	public static String OPTION_DOCLET = "-doclet";
	public static String OPTION_QUIET = "-quiet";

	private static boolean s_printUsage;
	private static DocletTagLibrary s_tagLibrary;
	private static ClassLoader s_classLoader;
	private static Class s_docletClass;

	private static OptionParser s_helpParser = new OptionParser ()
	{
		public int handleOption (String[] args, int index) throws Exception
		{
			if (!OPTION_HELP.equals (args[index]))
				return 0;
			return 1;
		}

		public String toString ()
		{
			return OPTION_HELP + "\t\t\t\tPrint this help message.";
		}
	};

	private static OptionParser s_sourceParser = new OptionParser ()
	{
		public int handleOption (String[] args, int index) throws Exception
		{
			if (!OPTION_SOURCE.equals (args[index]))
				return 0;
			JavaDocUtils.setSource (args[index + 1]);
			return 2;
		}

		public String toString ()
		{
			return OPTION_SOURCE + " <release>\t\tSpecify the source compatiblity release version.";
		}
	};

	private static OptionParser s_classPathParser = new OptionParser ()
	{
		public int handleOption (String[] args, int index) throws Exception
		{
			if (!OPTION_CLASS_PATH.equals (args[index]))
				return 0;
			String classPath = args[index + 1];
			StringTokenizer tokenizer = new StringTokenizer (classPath, File.pathSeparator);
			Collection list = new LinkedList ();
			while (tokenizer.hasMoreTokens ())
				list.add (new File (tokenizer.nextToken ()).toURL ());
			s_classLoader = new URLClassLoader ((URL[])list.toArray (new URL[list.size ()]));
			return 2;
		}

		public String toString ()
		{
			return OPTION_CLASS_PATH + " <pathlist>\t\tSpecify the user class paths.";
		}
	};

	public static OptionParser s_sourcePathParser = new OptionParser ()
	{
		public int handleOption (String[] args, int index) throws Exception
		{
			if (!OPTION_SOURCE_PATH.equals (args[index]))
				return 0;
			JavaDocUtils.setSourcePath (args[index + 1]);
			return 2;
		}

		public String toString ()
		{
			return OPTION_SOURCE_PATH + " <pathlist>\t\tSpecify the Java source file locations.";
		}
	};

	private static OptionParser s_tagLibraryParser = new OptionParser ()
	{
		public int handleOption (String[] args, int index) throws Exception
		{
			if (!OPTION_TAG_LIBRARY.equals (args[index]))
				return 0;
			String function = args[index + 1];
			int lastDot = function.lastIndexOf ('.');
			String className = function.substring (0, lastDot);
			function = function.substring (lastDot + 1);
			Method method = s_classLoader.loadClass (className).getMethod (function, new Class[0]);
			if (method == null || !Modifier.isStatic (method.getModifiers ()))
				throw new IllegalArgumentException (args[index + 1] + " must be a static function.");
			s_tagLibrary = (DocletTagLibrary)method.invoke (null, new Object[0]);
			return 2;
		}

		public String toString ()
		{
			return OPTION_TAG_LIBRARY + " <function>\t\tSpecify the function used to load the tag library.";
		}
	};

	private static OptionParser s_docletParser = new OptionParser ()
	{
		public int handleOption (String[] args, int index) throws Exception
		{
			if (!OPTION_DOCLET.equals (args[index]))
				return 0;
			s_docletClass = s_classLoader.loadClass (args[index + 1]);
			return 2;
		}

		public String toString ()
		{
			return OPTION_DOCLET + " <class>\t\t\tSpecify the doclet to be used to generate output.";
		}
	};

	private static OptionParser s_quietParser = new OptionParser ()
	{
		public int handleOption (String[] args, int index) throws Exception
		{
			if (!OPTION_QUIET.equals (args[index]))
				return 0;
			JavaDocUtils.setQuiet (true);
			return 1;
		}

		public String toString ()
		{
			return OPTION_QUIET + "\t\t\t\tSuppress messages.";
		}
	};

	private static OptionParser[] s_optionParsers = new OptionParser[]
	{
		s_helpParser,
		s_sourceParser,
		s_sourcePathParser,
		s_classPathParser,
		s_tagLibraryParser,
		s_docletParser,
		s_quietParser
	};

	private static Doclet parseOptions (String[] args) throws Exception
	{
		OptionParser[] optionParsers = s_optionParsers;
		int i;
		for (i = 0; i < args.length;)
		{
			int j;
			for (j = 0; j < optionParsers.length; ++j)
			{
				int count = optionParsers[j].handleOption (args, i);
				if (count > 0)
				{
					i += count;
					break;
				}
			}
			if (j == optionParsers.length)
				break;
		}

		Doclet doclet = getDoclet ();
		optionParsers = doclet.getOptionParsers ();
		for (; i < args.length;)
		{
			int j;
			for (j = 0; j < optionParsers.length; ++j)
			{
				int count = optionParsers[j].handleOption (args, i);
				if (count > 0)
				{
					i += count;
					break;
				}
			}
			if (j == optionParsers.length)
				throw new Exception ("Unable to parse option: " + args[i]);
		}

		if (s_printUsage || args.length == 0)
		{
			s_printUsage = false;

			System.out.println ("Usage: cookxmldoc [cookxmldoc options] [doclet options]");
			for (int j = 0; j < s_optionParsers.length; ++j)
				System.out.println (s_optionParsers[j]);
			System.out.println ();
			System.out.println (doclet + " options:");
			for (int j = 0; j < optionParsers.length; ++j)
				System.out.println (optionParsers[j]);

			return null;
		}

		return doclet;
	}

	private static Doclet getDoclet () throws Exception
	{
		Constructor ctor = s_docletClass.getConstructor (new Class[0]);
		if (ctor == null)
			throw new IllegalArgumentException ("default constructor not found in the doclet class.");
		return (Doclet)ctor.newInstance (new Object[0]);
	}

	public static ClassLoader getClassLoader ()
	{
		return s_classLoader;
	}

	public static void main (String[] args) throws Exception
	{
		s_printUsage = false;
		s_tagLibrary = null;
		s_classLoader = ClassLoader.getSystemClassLoader ();
		s_docletClass = HtmlDoclet.class;

		Doclet doclet = parseOptions (args);

		if (doclet == null)
			return;

		if (s_tagLibrary == null)
			throw new IllegalArgumentException ("Tag Library not found.");
		TagLibraryDoc tagLibraryDoc = DocBuilder.execute (s_tagLibrary);
		doclet.execute (tagLibraryDoc);
	}
}
