/*
 * (c) Copyright 2006 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookxmldoc;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.StringTokenizer;

import org.apache.tools.ant.AntClassLoader;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Execute;
import org.apache.tools.ant.taskdefs.LogOutputStream;
import org.apache.tools.ant.taskdefs.PumpStreamHandler;
import org.apache.tools.ant.types.Commandline;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.util.JavaEnvUtils;

/**
 * A task for CookXmlDoc within Ant.
 *
 * @author Heng Yuan
 * @version $Id: Task.java 215 2007-06-06 03:59:41Z coconut $
 * @since CookXmlDoc 0.1
 */
public class Task extends org.apache.tools.ant.Task
{
	private Collection m_jvmArgs = new LinkedList ();
	private String m_maxMemory;
	private Path m_toolsJar;
	private Collection m_classPaths = new LinkedList ();
	private Collection m_sourcePaths = new LinkedList ();
	private String m_docTitle;
	private String m_tagLibrary;
	private final Collection m_links = new LinkedList ();
	private final Collection m_prefixes = new LinkedList ();
	private File m_destDir;
	private String m_source;

	public void setJvmArgs (String arg)
	{
		StringTokenizer tok = new StringTokenizer (arg, " ");
		while (tok.hasMoreTokens ())
		{
			JvmArg ja = new JvmArg ();
			ja.setValue (tok.nextToken ());
			m_jvmArgs.add (ja);
		}
	}

	public void setMaxMemory (String memory)
	{
		m_maxMemory = memory;
	}

	public void setToolsJar (Path path)
	{
		m_toolsJar = path;
	}

	public void setClassPath (Path path)
	{
		NamePath p = new NamePath ();
		p.setName (path);
		m_classPaths.add (p);
	}

	public void setSourcePath (Path path)
	{
		NamePath p = new NamePath ();
		p.setName (path);
		m_sourcePaths.add (p);
	}

	public void setDestDir (File destDir)
	{
		m_destDir = destDir;
	}

	public void setSource (String source)
	{
		m_source = source;
	}

	public void setDocTitle (String docTitle)
	{
		m_docTitle = docTitle;
	}

	public void setTagLibrary (String tagLibrary)
	{
		m_tagLibrary = tagLibrary;
	}

	public void execute () throws BuildException
	{
		Commandline cmd = new Commandline ();
		cmd.setExecutable (JavaEnvUtils.getJdkExecutable ("java"));

		for (Iterator iter = m_jvmArgs.iterator (); iter.hasNext ();)
		{
			JvmArg arg = (JvmArg)iter.next ();
			String value = arg.getValue ();
			if (value == null)
				return;
			cmd.createArgument ().setValue (value);
		}

		if (m_maxMemory != null)
		{
			cmd.createArgument ().setValue ("-Xmx" + m_maxMemory);
		}

		String path = "";
		if (m_toolsJar != null)
			path += m_toolsJar.toString () + File.pathSeparator;
		else
			path += System.getProperty ("java.home") + "/lib/tools.jar" + File.pathSeparator;

		ClassLoader cl = getClass ().getClassLoader ();
		if (cl instanceof AntClassLoader)
			path += ((AntClassLoader)cl).getClasspath () + File.pathSeparator;

		for (Iterator iter = m_classPaths.iterator (); iter.hasNext ();)
		{
			NamePath p = (NamePath)iter.next ();
			if (p.getName () != null)
				path += p.getName ().toString () + File.pathSeparator;
		}

		cmd.createArgument ().setValue ("-cp");
		cmd.createArgument ().setValue (path);

		cmd.createArgument ().setValue ("cookxml.cookxmldoc.Main");

		if (m_tagLibrary != null)
		{
			cmd.createArgument ().setValue ("-taglibrary");
			cmd.createArgument ().setValue (m_tagLibrary);
		}

		if (m_source != null)
		{
			cmd.createArgument ().setValue ("-source");
			cmd.createArgument ().setValue (m_source);
		}

		path = "";
		for (Iterator iter = m_sourcePaths.iterator (); iter.hasNext ();)
		{
			NamePath p = (NamePath)iter.next ();
			if (p.getName () != null)
				path += p.getName ().toString () + File.pathSeparator;
		}
		if (path.length () > 0)
		{
			cmd.createArgument ().setValue ("-sourcepath");
			cmd.createArgument ().setValue (path);
		}

		for (Iterator iter = m_links.iterator (); iter.hasNext ();)
		{
			Link link = (Link)iter.next ();
			if (link.getHref () != null)
			{
				cmd.createArgument ().setValue ("-link");
				cmd.createArgument ().setValue (link.getHref ());
			}
		}

		for (Iterator iter = m_prefixes.iterator (); iter.hasNext ();)
		{
			Prefix prefix = (Prefix)iter.next ();
			if (prefix.getNs () != null && prefix.getPrefix () != null)
			{
				cmd.createArgument ().setValue ("-prefix");
				cmd.createArgument ().setValue (prefix.getNs ());
				cmd.createArgument ().setValue (prefix.getPrefix ());
			}
		}

		if (m_destDir != null)
		{
			cmd.createArgument ().setValue ("-d");
			cmd.createArgument ().setFile (m_destDir);
		}

		if (m_docTitle != null)
		{
			cmd.createArgument ().setValue ("-doctitle");
			cmd.createArgument ().setValue (m_docTitle);
		}

		log (cmd.describeCommand (), Project.MSG_VERBOSE);
		log ("CookXmlDoc execution", Project.MSG_INFO);

		LogOutputStream out = new LogOutputStream (this, Project.MSG_INFO);
		LogOutputStream err = new LogOutputStream (this, Project.MSG_WARN);

		Execute exe = new Execute (new PumpStreamHandler (out, err));
		exe.setAntRun (getProject ());

		exe.setWorkingDirectory (null);
		try
		{
			exe.setCommandline (cmd.getCommandline ());
			int ret = exe.execute ();
			if (ret != 0)
			{
				throw new BuildException ("CookXmlDoc returned " + ret, getLocation ());
			}
		}
		catch (IOException ex)
		{
			throw new BuildException ("CookXmlDoc failed: " + ex, ex, getLocation ());
		}
		finally
		{
			try
			{
				out.flush ();
			}
			catch (Exception ex)
			{
			}
			try
			{
				err.flush ();
			}
			catch (Exception ex)
			{
			}
			try
			{
				out.close ();
				err.close ();
			}
			catch (Exception ex)
			{
			}
		}
	}

	public JvmArg createJvmArg ()
	{
		JvmArg arg = new JvmArg ();
		m_jvmArgs.add (arg);
		return arg;
	}

	public NamePath createClassPath ()
	{
		NamePath path = new NamePath ();
		m_classPaths.add (path);
		return path;
	}

	public NamePath createSourcePath ()
	{
		NamePath path = new NamePath ();
		m_sourcePaths.add (path);
		return path;
	}

	public Link createLink ()
	{
		Link link = new Link ();
		m_links.add (link);
		return link;
	}

	public Prefix createPrefix ()
	{
		Prefix prefix = new Prefix ();
		m_prefixes.add (prefix);
		return prefix;
	}

	public static class JvmArg
	{
		private String m_value;
		public void setValue (String value)
		{
			m_value = value;
		}
		public String getValue ()
		{
			return m_value;
		}
	}

	public static class NamePath
	{
		private Path m_name;

		public void setName (Path name)
		{
			m_name = name;
		}

		public Path getName ()
		{
			return m_name;
		}
	}

	public static class Link
	{
		private String m_href;

		public String getHref ()
		{
			return m_href;
		}

		public void setHref (String href)
		{
			m_href = href;
		}
	}

	public static class Prefix
	{
		private String m_ns;
		private String m_prefix;

		public String getNs ()
		{
			return m_ns;
		}

		public void setNs (String ns)
		{
			m_ns = ns;
		}

		public String getPrefix ()
		{
			return m_prefix;
		}

		public void setPrefix (String prefix)
		{
			m_prefix = prefix;
		}
	}
}
