/*
 * (c) Copyright 2006 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookxmldoc.htmldoclet;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.lang.reflect.Member;
import java.net.URL;
import java.text.BreakIterator;
import java.text.ChoiceFormat;
import java.text.MessageFormat;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.TreeSet;

import cookxml.cookxmldoc.interfaces.*;

/**
 * @author Heng Yuan
 * @version $Id: Generator.java 247 2007-06-07 06:57:03Z coconut $
 * @since CookXmlDoc 0.1
 */
class Generator
{
	private final MessageFormat m_indexMsg;
	private final MessageFormat m_nsMsg;
	private final MessageFormat m_nsIterMsg;
	private final MessageFormat m_tagsMsg;
	private final MessageFormat m_tagsIterMsg;
	private final MessageFormat m_tagMsg;
	private final MessageFormat m_tagParentMsg;
	private final MessageFormat m_tagParentIterMsg;
	private final MessageFormat m_tagChildMsg;
	private final MessageFormat m_tagClassMsg;
	private final MessageFormat m_tagHelperMsg;
	private final MessageFormat m_tagConverterMsg;
	private final MessageFormat m_tagAttrsTableMsg;
	private final MessageFormat m_tagAttrsTableIterMsg;
	private final MessageFormat m_tagChildrenTableMsg;
	private final MessageFormat m_tagChildrenTableIterMsg;
	private final MessageFormat m_tagTagCommentMsg;
	private final MessageFormat m_tagDocCommentMsg;
	private final MessageFormat m_tagExampleMsg;
	private final ChoiceFormat m_tagExampleCounter = new ChoiceFormat (new double[]{ 1, 2 }, new String[]{ "", " {1}" });

	private final Map m_auxMap;
	private final LinkManager m_linkManager;

	public Generator (Map formatMap, Map auxMap, LinkManager linkManager)
	{
		m_indexMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.INDEX_HTML));
		m_nsMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.NS_HTML));
		m_nsIterMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.NS_ITER_HTML));
		m_tagsMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAGS_HTML));
		m_tagsIterMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAGS_ITER_HTML));
		m_tagMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAG_HTML));
		m_tagParentMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAG_PARENT_HTML));
		m_tagParentIterMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAG_PARENT_ITER_HTML));
		m_tagChildMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAG_CHILD_HTML));
		m_tagClassMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAG_CLASS_HTML));
		m_tagHelperMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAG_HELPER_HTML));
		m_tagConverterMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAG_CONVERTER_HTML));
		m_tagAttrsTableMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAG_ATTRS_TABLE_HTML));
		m_tagAttrsTableIterMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAG_ATTRS_TABLE_ITER_HTML));
		m_tagChildrenTableMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAG_CHILDREN_TABLE_HTML));
		m_tagChildrenTableIterMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAG_CHILDREN_TABLE_ITER_HTML));
		m_tagTagCommentMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAG_TAG_COMMENT_HTML));
		m_tagDocCommentMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAG_DOC_COMMENT_HTML));
		m_tagExampleMsg = new MessageFormat ((String)formatMap.get (HtmlDoclet.TAG_EXAMPLE_HTML));
		m_tagExampleMsg.setFormatByArgumentIndex (0, m_tagExampleCounter);

		m_auxMap = auxMap;
		m_linkManager = linkManager;
	}

	private void generateIndex (File outputDir, String docTitle) throws IOException
	{
		File file = new File (outputDir, "index.html");
		PrintStream os = new PrintStream (new FileOutputStream (file));
		os.print (m_indexMsg.format (new String[]{ docTitle }));
		os.close ();
	}

	private void generateAuxFiles (File outputDir) throws IOException
	{
		for (Iterator iter = m_auxMap.entrySet ().iterator (); iter.hasNext ();)
		{
			Map.Entry entry = (Map.Entry)iter.next ();
			File file = new File (outputDir, (String)entry.getKey ());
			PrintStream os = new PrintStream (new FileOutputStream (file));
			os.print (entry.getValue ());
			os.close ();
		}
	}

	private void generateNSList (TagLibraryDoc tagLibraryDoc, File outputDir) throws IOException
	{
		File file = new File (outputDir, "all-namespaces.html");
		PrintStream os = new PrintStream (new FileOutputStream (file));

		StringBuffer strBuffer = new StringBuffer ();
		Object[] pair = new Object[2];

		NameSpaceDoc[] namespaces = tagLibraryDoc.getNameSpaces ();
		for (int i = 0; i < namespaces.length; ++i)
		{
			String ns = namespaces[i].getName ();
			pair[0] = ns;
			pair[1] = m_linkManager.getPrefix (ns);
			strBuffer.append (m_nsIterMsg.format (pair));
		}

		os.print (m_nsMsg.format (new String[]{ strBuffer.toString () }));

		os.close ();
	}

	private void generateTagList (TagDoc[] tags, File outputDir, String prefix) throws IOException
	{
		TreeMap sortMap = new TreeMap ();
		for (int i = 0; i < tags.length; ++i)
			sortMap.put (tags[i].getName (), tags[i]);
		sortMap.values ().toArray (tags);

		// generate the tag list
		File file = new File (outputDir, prefix + ".html");
		PrintStream os = new PrintStream (new FileOutputStream (file));

		StringBuffer buffer = new StringBuffer ();
		String[] tagListAttr = new String[2];

		tagListAttr[1] = prefix;

		for (int i = 0; i < tags.length; ++i)
		{
			tagListAttr[0] = tags[i].getName ();
			buffer.append (m_tagsIterMsg.format (tagListAttr));
		}

		String[] singleArray = new String[1];
		singleArray[0] = buffer.toString ();
		String output = m_tagsMsg.format (singleArray);

		os.print (output);
		os.close ();
	}

	private void generateTagInfo (TagDoc tagDoc, File outputDir) throws IOException
	{
		String tag = tagDoc.getName ();
		String ns = tagDoc.getNameSpace ().getName ();

		File file = new File (outputDir, tag + ".html");

		PrintStream os = new PrintStream (new FileOutputStream (file));

		Object[] tagInfoArray = new Object[12];
		for (int i = 0; i < tagInfoArray.length; ++i)
			tagInfoArray[i] = "";

		tagInfoArray[0] = ns;
		tagInfoArray[1] = tag;

		StringBuffer strBuffer;
		Object[] array;

		TagDoc[] ancestors = tagDoc.getAncestors ();

		// deal with parent tags
		if (ancestors.length > 0)
		{
			strBuffer = new StringBuffer ();
			array = new Object[3];
			Integer zero = new Integer (0);
			Integer one = new Integer (1);
			for (int i = 0; i < ancestors.length; ++i)
			{
				array[0] = ancestors[i].getName ();
				array[1] = m_linkManager.getPrefix (ancestors[i].getNameSpace ().getName ());
				array[2] = i == 0 ? zero : one;
				strBuffer.append (m_tagParentIterMsg.format (array));
			}
			tagInfoArray[2] = m_tagParentMsg.format (new Object[]{ strBuffer.toString () });
		}

		// deal with child tags
		TagDoc[] children = tagDoc.getChildren ();
		if (children.length > 0)
		{
			// cut/paste of code for parent tags
			// just replace m_tagParentMsg with m_tagChildMsg
			// ancestors to children
			strBuffer = new StringBuffer ();
			array = new Object[3];
			Integer zero = new Integer (0);
			Integer one = new Integer (1);
			for (int i = 0; i < children.length; ++i)
			{
				array[0] = children[i].getName ();
				array[1] = m_linkManager.getPrefix (children[i].getNameSpace ().getName ());
				array[2] = i == 0 ? zero : one;
				strBuffer.append (m_tagParentIterMsg.format (array));
			}
			tagInfoArray[3] = m_tagChildMsg.format (new Object[]{ strBuffer.toString () });
		}

		String className = tagDoc.getClassName ();
		String helperClassName = tagDoc.getHelperClassName ();

		tagInfoArray[4] = className == null ? "" : m_tagClassMsg.format (new Object[]{ m_linkManager.getClassLink (className) });
		tagInfoArray[5] = helperClassName == null ? "" : m_tagHelperMsg.format (new Object[]{ m_linkManager.getClassLink (helperClassName) });

		// now deal with converter
		if (className != null)
		{
			ConverterDoc converter = ((NameSpaceDoc)tagDoc.getNameSpace ()).getConverter (className);
			if (converter != null)
			{
				array = new Object[3];
				URL url = m_linkManager.getClassURL (converter.getConverter ().getClass ().getName ());
				if (url == null)
				{
					array[0] = "";
					array[1] = "";
				}
				else
				{
					array[0] = "<a href=\"" + url + "\">";
					array[1] = "</a>";
				}
				array[2] = converter.getDocComment ();
				tagInfoArray[6] = m_tagConverterMsg.format (array);
			}
		}

		// now deal with tag description
		// class comment
		String comment;
		comment = tagDoc.getDocComment ();
		if (comment.length () > 0)
		{
			tagInfoArray[7] = m_tagDocCommentMsg.format (new Object[]{ parseComment (ns, comment) });
		}
		else
			tagInfoArray[7] = "";
		// tag comment
		comment = tagDoc.getTagComment ();
		if (comment.length () > 0)
		{
			tagInfoArray[8] = m_tagTagCommentMsg.format (new Object[]{ parseComment (ns, comment) });
		}
		else
			tagInfoArray[8] = "";

		// now deal with attributes
		AttrDoc[] attrDocs = tagDoc.getAttributes ();
		if (attrDocs.length > 0)
		{
			TreeSet sortSet = new TreeSet ();
			for (int i = 0; i < attrDocs.length; ++i)
				sortSet.add (attrDocs[i]);
			sortSet.toArray (attrDocs);

			strBuffer = new StringBuffer ();
			for (int i = 0; i < attrDocs.length; ++i)
				strBuffer.append (generateAttrIter (attrDocs[i]));
			String attrsTable = m_tagAttrsTableMsg.format (new Object[]{ strBuffer.toString () });
			tagInfoArray[9] = attrsTable;
		}
		else
			tagInfoArray[9] = "<!-- none -->";

		// now deal with children
		AdderDoc[] adderDocs = tagDoc.getAdders ();
		if (adderDocs.length > 0)
		{
			strBuffer = new StringBuffer ();
			for (int i = 0; i < adderDocs.length; ++i)
				strBuffer.append (generateAdderIter (adderDocs[i]));
			String childrenTable = m_tagChildrenTableMsg.format (new Object[]{ strBuffer.toString () });
			tagInfoArray[10] = childrenTable;
		}
		else
			tagInfoArray[10] = "<!-- none -->";

		// now deal with examples
		String[] examples = tagDoc.getExamples ();
		if (examples.length > 0)
		{
			strBuffer = new StringBuffer ();
			for (int i = 0; i < examples.length; ++i)
			{
				String example = parseComment (ns, examples[i]);
				strBuffer.append (m_tagExampleMsg.format (new Object[]{ new Integer (examples.length), new Integer (i + 1), example }));
			}
			tagInfoArray[11] = strBuffer.toString ();
		}
		else
			tagInfoArray[11] = "";

		os.print (m_tagMsg.format (tagInfoArray));
		os.close ();
	}

	private String generateAttrIter (AttrDoc attrDoc)
	{
		String attr = attrDoc.getName ();
		String[] array = new String[3];
		array[0] = attr;
		array[1] = "";
		array[2] = "";

		Member member = attrDoc.getMember ();
		if (member != null)
			array[0] = m_linkManager.getAttrLink (member, attr);

		Class vc = attrDoc.getValueClass ();
		if (vc != null)
		{
			String className = vc.getName ();
			String valueTag = m_linkManager.getTag (className);
			if (valueTag != null)
			{
				array[1] = m_linkManager.getTagLink (null, valueTag, false);
			}
			else
				array[1] = m_linkManager.getClassLink (vc.getName ());
		}

		String comment = attrDoc.getTagComment ();
		if (comment.length () > 0)
			array[2] = comment;
		else
		{
			comment = attrDoc.getDocComment ();
			if (comment.length () > 0)
			{
				BreakIterator iter = BreakIterator.getSentenceInstance ();
				iter.setText (comment);
				int start = iter.first ();
				int end = iter.next ();
				if (end > 0 && end > start)
					comment = comment.substring (start, end);
			}
		}
		array[2] = parseComment (attrDoc.getTag ().getNameSpace ().getName (), comment);
		return m_tagAttrsTableIterMsg.format (array);
	}

	private String generateAdderIter (AdderDoc adderDoc)
	{
		String[] array = new String[3];
		array[0] = "";
		array[1] = "";
		array[2] = "";

		Member member = adderDoc.getMember ();
		if (member != null)
			array[0] = m_linkManager.getMemberLink (member);

		Class vc = adderDoc.getChildClass ();
		if (vc != null)
		{
			String className = vc.getName ();
			String valueTag = m_linkManager.getTag (className);
			if (valueTag != null)
			{
				array[1] = m_linkManager.getTagLink (null, valueTag, false);
			}
			else
				array[1] = m_linkManager.getClassLink (vc.getName ());
		}

		String comment = adderDoc.getTagComment ();
		if (comment.length () > 0)
			array[2] = comment;
		else
		{
			comment = adderDoc.getDocComment ();
			if (comment.length () > 0)
			{
				BreakIterator iter = BreakIterator.getSentenceInstance ();
				iter.setText (comment);
				int start = iter.first ();
				int end = iter.next ();
				if (end > 0 && end > start)
					comment = comment.substring (start, end);
			}
		}
		array[2] = parseComment (adderDoc.getTag ().getNameSpace ().getName (), comment);
		return m_tagChildrenTableIterMsg.format (array);
	}

	public void execute (TagLibraryDoc tagLibraryDoc, File outputDir, String docTitle) throws IOException
	{
		outputDir.mkdirs ();

		m_linkManager.createClass2TagMap (tagLibraryDoc);

		generateIndex (outputDir, docTitle);
		generateAuxFiles (outputDir);
		generateNSList (tagLibraryDoc, outputDir);

		NameSpaceDoc[] namespaces = tagLibraryDoc.getNameSpaces ();
		for (int i = 0; i < namespaces.length; ++i)
		{
			String ns = namespaces[i].getName ();
			String prefix = m_linkManager.getPrefix (ns);
			TagDoc[] tags = namespaces[i].getTags ();
			generateTagList (tags, outputDir, prefix);

			File tagDir = new File (outputDir, prefix);
			tagDir.mkdir ();

			for (int j = 0; j < tags.length; ++j)
				generateTagInfo (tags[j], tagDir);
		}
	}

	private String parseComment (String ns, String comment)
	{
		return m_linkManager.parseLinkComment (ns, parseCodeComment (comment));
	}

	private static String parseCodeComment (String text)
	{
		if (text == null)
			return text;
		StringBuffer strBuffer = null;
		int start;
		int len = text.length ();
		for (start = 0; start < len;)
		{
			int newStart = text.indexOf ("<codeblock ", start);
			if (newStart < 0)
				break;
			if (strBuffer == null)
				strBuffer = new StringBuffer ();
			strBuffer.append (text.substring (start, newStart));
			newStart = text.indexOf ("type=\"", newStart + 9);
			if (newStart < 0)
				break;
			int end = text.indexOf ('"', newStart + 6);
			if (end < 0)
				break;
			String type = text.substring (newStart + 6, end);
			newStart = text.indexOf ('>', end);
			if (newStart < 0)
				break;
			end = text.indexOf ("</codeblock>", newStart);
			if (end < 0)
				break;
			String code = text.substring (newStart + 1, end);
			strBuffer.append (Highlighter.getHighlightText (type, code));
			start = end + 12;
		}
		if (strBuffer == null)
			return text;
		if (start < len)
			strBuffer.append (text.substring (start, len));
		return strBuffer.toString ();
	}
}
