/*
 * (c) Copyright 2007 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookxmldoc.htmldoclet;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import javax.swing.text.Segment;

import cookxml.cookxmldoc.htmldoclet.highlighter.*;

/**
 * This class utilizes jEdit highlighter to generate color highlighted HTML
 * from the given text.
 *
 * @author Heng Yuan
 * @version $Id: Highlighter.java 215 2007-06-06 03:59:41Z coconut $
 * @since CookXmlDoc 0.1
 */
public class Highlighter
{
	public static String STYLE_NULL = "<span class=\"code_text\">";
	public static String STYLE_COMMENT1 = "<span class=\"code_comment1\">";
	public static String STYLE_COMMENT2 = "<span class=\"code_comment2\">";
	public static String STYLE_LITERAL1 = "<span class=\"code_literal1\">";
	public static String STYLE_LITERAL2 = "<span class=\"code_literal2\">";
	public static String STYLE_LABEL = "<span class=\"code_label\">";
	public static String STYLE_KEYWORD1 = "<span class=\"code_keyword1\">";
	public static String STYLE_KEYWORD2 = "<span class=\"code_keyword2\">";
	public static String STYLE_KEYWORD3 = "<span class=\"code_keyword3\">";
	public static String STYLE_OPERATOR = "<span class=\"code_operator\">";
	public static String STYLE_INVALID = "<span class=\"code_invalid\">";
	public static String STYLE_END = "</span>";

	private final static HashMap s_highlighterMap = new HashMap ();

	static
	{
		Class[] param = new Class[0];
		try
		{
			s_highlighterMap.put ("plain", PlainTextTokenMarker.class.getConstructor (param));
			s_highlighterMap.put ("c", CTokenMarker.class.getConstructor (param));
			s_highlighterMap.put ("c++", CCTokenMarker.class.getConstructor (param));
			s_highlighterMap.put ("html", HTMLTokenMarker.class.getConstructor (param));
			s_highlighterMap.put ("js", JavaScriptTokenMarker.class.getConstructor (param));
			s_highlighterMap.put ("java", JavaTokenMarker.class.getConstructor (param));
			s_highlighterMap.put ("php", PHPTokenMarker.class.getConstructor (param));
			s_highlighterMap.put ("python", PythonTokenMarker.class.getConstructor (param));
			s_highlighterMap.put ("xml", XMLTokenMarker.class.getConstructor (param));
		}
		catch (Exception ex)
		{
			ex.printStackTrace ();
		}
	}

	/**
	 * Highlight a particular text with the given format type in HTML.
	 *
	 * @param	type
	 * 			the format type
	 * @param	text
	 * 			the text string to be formatted.
	 * @return	the HTML formatted text.
	 */
	public static String getHighlightText (String type, String text)
	{
		if (type == null)
			type = "plain";
		Constructor ctor = (Constructor)s_highlighterMap.get (type);
		if (ctor == null)
			return text;
		try
		{
			return getHighlightText ((TokenMarker)ctor.newInstance (new Object[0]), text);
		}
		catch (Exception ex)
		{
			return text;
		}
	}

	/**
	 * Highlight a particular text using the given formatting engine.
	 *
	 * @param	tm
	 * 			the formatting engine.
	 * @param	text
	 * 			the text string to be formatted.
	 * @return	the HTML formatted text.
	 */
	public static String getHighlightText (TokenMarker tm, String text)
	{
		try
		{
			char[] array = text.toCharArray ();
			StringBuffer buffer = new StringBuffer ();
			buffer.append ("<div class=\"codeblock\">");
			int[] position = new int[1];

			try
			{
				int len = text.length ();
				int start = 0;
				int end = 0;
				boolean eatFirst = false;
				for (int lineNum = 0; start < len && end < len; ++lineNum)
				{
					end = text.indexOf ('\n', start);
					if (end < 0)
						end = len;
					// trim the first empty line
					if (lineNum == 0 && !eatFirst)
					{
						if (text.substring (0, end).trim ().length () == 0)
						{
							if (end == len)
								break;
							eatFirst = true;
							start = end + 1;
							--lineNum;
							continue;
						}
					}
					// trim the last empty line
					if (end == len)
					{
						if (text.substring (start, end).trim ().length () == 0)
						{
							break;
						}
					}
					if (lineNum > 0)
						buffer.append ("<br/>\n");
					tm.insertLines (tm.getLineCount (), 1);
					Segment line = new Segment (array, start, end - start);
					Token first = tm.markTokens (line, lineNum);
					position[0] = start;
					parseToken (buffer, line.array, position, first);
					start = end + 1;
				}
			}
			catch (Exception ex)
			{
				ex.printStackTrace ();
			}
			buffer.append ("</div>");
			return buffer.toString ();
		}
		catch (Exception ex)
		{
			return text;
		}
	}

	private static void parseToken (StringBuffer buffer, char[] text, int[] position, Token token)
	{
		int col = 0;
end:
		for (; token != null; token = token.next)
		{
			switch (token.id)
			{
				case Token.END:
					break end;
				case Token.NULL:
					col = appendText (buffer, text, position, token.length, STYLE_NULL, col);
					break;
				case Token.COMMENT1:
					col = appendText (buffer, text, position, token.length, STYLE_COMMENT1, col);
					break;
				case Token.COMMENT2:
					col = appendText (buffer, text, position, token.length, STYLE_COMMENT2, col);
					break;
				case Token.LITERAL1:
					col = appendText (buffer, text, position, token.length, STYLE_LITERAL1, col);
					break;
				case Token.LITERAL2:
					col = appendText (buffer, text, position, token.length, STYLE_LITERAL2, col);
					break;
				case Token.LABEL:
					col = appendText (buffer, text, position, token.length, STYLE_LABEL, col);
					break;
				case Token.KEYWORD1:
					col = appendText (buffer, text, position, token.length, STYLE_KEYWORD1, col);
					break;
				case Token.KEYWORD2:
					col = appendText (buffer, text, position, token.length, STYLE_KEYWORD2, col);
					break;
				case Token.KEYWORD3:
					col = appendText (buffer, text, position, token.length, STYLE_KEYWORD3, col);
					break;
				case Token.OPERATOR:
					col = appendText (buffer, text, position, token.length, STYLE_OPERATOR, col);
					break;
				case Token.INVALID:
					col = appendText (buffer, text, position, token.length, STYLE_INVALID, col);
					break;
				default:
					col = appendText (buffer, text, position, token.length, STYLE_NULL, col);
					break;
			}
		}
	}

	private static int appendText (StringBuffer buffer, char[] text, int[] position, int length, String style, int col)
	{
		buffer.append (style);
		for (int i = 0; i < length; ++i)
		{
			char c = text[position[0] + i];
			switch (c)
			{
				case '<':
					buffer.append ("&lt;");
					++col;
					break;
				case '-':
					buffer.append ("&#45;");
					++col;
					break;
				case ' ':
					buffer.append ("&nbsp;");
					++col;
					break;
				case '\t':
				{
					int s = 4 - col % 4;
					for (int j = 0; j < s; ++j)
						buffer.append ("&nbsp;");
					col += s;
					break;
				}
				default:
					buffer.append (c);
					++col;
					break;
			}
		}
		position[0] += length;
		buffer.append (STYLE_END);
		return col;
	}

	/**
	 * Debugging function.
	 *
	 * @param	args
	 * 			no command line arguments needed.
	 */
	public static void main (String[] args)
	{
		String text = "\n<html><body bgcolor=\"aaa\">\n<p>Hello World!</p>\n</body></html>\n ";
		System.out.println (getHighlightText ("plain", text));
	}
}
