/*
 * (c) Copyright 2006 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookxmldoc.htmldoclet;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import cookxml.cookxmldoc.interfaces.Doclet;
import cookxml.cookxmldoc.interfaces.OptionParser;
import cookxml.cookxmldoc.interfaces.TagLibraryDoc;
import cookxml.core.util.TextUtils;

/**
 * This Doclet generates HTML output.
 *
 * @author Heng Yuan
 * @version $Id: HtmlDoclet.java 241 2007-06-07 04:03:39Z coconut $
 * @since CookXmlDoc 0.1
 */
public class HtmlDoclet implements Doclet
{
	public static String OPTION_DOC_TITLE = "-doctitle";
	public static String OPTION_LINK = "-link";
	public static String OPTION_OFFLINE_LINK = "-linkoffline";
	public static String OPTION_OUTPUT_DIR = "-d";
	public static String OPTION_PREFIX = "-prefix";

	public final static String INDEX_HTML = "index.html";
	public final static String NS_HTML = "all-namespaces.html";
	public final static String NS_ITER_HTML = "all-namespaces-iter.html";
	public final static String TAGS_HTML = "tags.html";
	public final static String TAGS_ITER_HTML = "tags-iter.html";
	public final static String TAG_HTML = "tag.html";
	public final static String TAG_PARENT_HTML = "tag-parent.html";
	public final static String TAG_PARENT_ITER_HTML = "tag-parent-iter.html";
	public final static String TAG_CHILD_HTML = "tag-child.html";
	public final static String TAG_CLASS_HTML = "tag-class.html";
	public final static String TAG_HELPER_HTML = "tag-helper.html";
	public final static String TAG_CONVERTER_HTML = "tag-converter.html";
	public final static String TAG_ATTRS_TABLE_HTML = "tag-attrs-table.html";
	public final static String TAG_ATTRS_TABLE_ITER_HTML = "tag-attrs-table-iter.html";
	public final static String TAG_CHILDREN_TABLE_HTML = "tag-children-table.html";
	public final static String TAG_CHILDREN_TABLE_ITER_HTML = "tag-children-table-iter.html";
	public final static String TAG_TAG_COMMENT_HTML = "tag-tag-comment.html";
	public final static String TAG_DOC_COMMENT_HTML = "tag-doc-comment.html";
	public final static String TAG_EXAMPLE_HTML = "tag-example.html";

	private File m_outputDir = new File (".");
	private String m_docTitle = "Tag Library Documentation";
	private Map m_formatMap;
	private Map m_auxMap;
	private LinkManager m_linkManager;

	private OptionParser m_docTitleParser = new OptionParser ()
	{
		public int handleOption (String[] args, int index) throws Exception
		{
			if (!OPTION_DOC_TITLE.equals (args[index]))
				return 0;
			m_docTitle = args[index + 1];
			return 2;
		}

		public String toString ()
		{
			return OPTION_LINK + " <url>\t\t\tSpecify the linked JavaDoc location URL.";
		}
	};

	private OptionParser m_linkParser = new OptionParser ()
	{
		public int handleOption (String[] args, int index) throws Exception
		{
			if (!OPTION_LINK.equals (args[index]))
				return 0;
			URL htmlURL = new URL (args[index + 1]);
			m_linkManager.addPackageLink (htmlURL);
			return 2;
		}

		public String toString ()
		{
			return OPTION_LINK + " <url>\t\t\tSpecify the linked JavaDoc location URL.";
		}
	};

	private OptionParser m_offlineLinkParser = new OptionParser ()
	{
		public int handleOption (String[] args, int index) throws Exception
		{
			if (!OPTION_OFFLINE_LINK.equals (args[index]))
				return 0;
			URL htmlURL = new URL (args[index + 1]);
			URL packageListURL = new URL (args[index + 2]);
			m_linkManager.addPackageLink (htmlURL, packageListURL);
			return 3;
		}

		public String toString ()
		{
			return OPTION_OFFLINE_LINK + " <url1> <url2>\tLink to JavaDoc at <url1> using package list at <url2>.";
		}
	};

	private OptionParser m_outputDirParser = new OptionParser ()
	{
		public int handleOption (String[] args, int index) throws Exception
		{
			if (!OPTION_OUTPUT_DIR.equals (args[index]))
				return 0;
			m_outputDir = new File (args[index + 1]);
			return 2;
		}

		public String toString ()
		{
			return OPTION_OUTPUT_DIR + " <directory>\t\t\tSpecify the HTML output directory.";
		}
	};

	private OptionParser m_prefixParser = new OptionParser ()
	{
		public int handleOption (String[] args, int index) throws Exception
		{
			if (!OPTION_PREFIX.equals (args[index]))
				return 0;
			String ns = args[index + 1];
			String prefix = args[index + 2];
			m_linkManager.addPrefix (ns, prefix);
			return 3;
		}

		public String toString ()
		{
			return OPTION_OUTPUT_DIR + " <ns> <short name>\t\tSpecify a short <name> for a namespace <ns>.";
		}
	};

	public String getLocation ()
	{
		return "cookxml/cookxmldoc/resources/";
	}

	public HtmlDoclet () throws IOException
	{
		String location = getLocation ();
		ClassLoader cl = HtmlDoclet.class.getClassLoader ();
		MessageFormat classLinkFormat = new MessageFormat (TextUtils.readText (location + "class-link.html", cl));
		MessageFormat tagLinkFormat = new MessageFormat (TextUtils.readText (location + "tag-link.html", cl));
		m_linkManager = new LinkManager (classLinkFormat, tagLinkFormat);
	}

	private Map getFormatMap (String location, ClassLoader cl) throws IOException
	{
		HashMap map = new HashMap ();

		putFormatMap (map, location, INDEX_HTML, cl);
		putFormatMap (map, location, NS_HTML, cl);
		putFormatMap (map, location, NS_ITER_HTML, cl);
		putFormatMap (map, location, TAGS_HTML, cl);
		putFormatMap (map, location, TAGS_ITER_HTML, cl);
		putFormatMap (map, location, TAG_HTML, cl);
		putFormatMap (map, location, TAG_PARENT_HTML, cl);
		putFormatMap (map, location, TAG_PARENT_ITER_HTML, cl);
		putFormatMap (map, location, TAG_CHILD_HTML, cl);
		putFormatMap (map, location, TAG_CLASS_HTML, cl);
		putFormatMap (map, location, TAG_HELPER_HTML, cl);
		putFormatMap (map, location, TAG_CONVERTER_HTML, cl);
		putFormatMap (map, location, TAG_ATTRS_TABLE_HTML, cl);
		putFormatMap (map, location, TAG_ATTRS_TABLE_ITER_HTML, cl);
		putFormatMap (map, location, TAG_CHILDREN_TABLE_HTML, cl);
		putFormatMap (map, location, TAG_CHILDREN_TABLE_ITER_HTML, cl);
		putFormatMap (map, location, TAG_TAG_COMMENT_HTML, cl);
		putFormatMap (map, location, TAG_DOC_COMMENT_HTML, cl);
		putFormatMap (map, location, TAG_EXAMPLE_HTML, cl);

		return map;
	}

	private void putFormatMap (HashMap map, String location, String name, ClassLoader cl) throws IOException
	{
		map.put (name, TextUtils.readText (location + name, cl));
	}

	public Map getFormatMap () throws IOException
	{
		if (m_formatMap != null)
			return m_formatMap;

		ClassLoader cl = HtmlDoclet.class.getClassLoader ();

		m_formatMap = getFormatMap (getLocation (), cl);
		return m_formatMap;
	}

	public Map getAuxMap () throws IOException
	{
		if (m_auxMap != null)
			return m_auxMap;

		HashMap map = new HashMap ();

		ClassLoader cl = HtmlDoclet.class.getClassLoader ();

		map.put ("stylesheet.css", TextUtils.readText ("cookxml/cookxmldoc/resources/stylesheet.css", cl));

		m_auxMap = map;
		return map;
	}

	public String getDocTitle ()
	{
		return m_docTitle;
	}

	public File getOutputDir ()
	{
		return m_outputDir;
	}

	public LinkManager getLinkManager () throws IOException
	{
		return m_linkManager;
	}

	public OptionParser[] getOptionParsers ()
	{
		return new OptionParser[]
		{
			m_docTitleParser,
			m_linkParser,
			m_offlineLinkParser,
			m_outputDirParser,
			m_prefixParser
		};
	}

	public void execute (TagLibraryDoc tagLibraryDoc) throws Exception
	{
		new Generator (getFormatMap (), getAuxMap (), getLinkManager ()).execute (tagLibraryDoc, getOutputDir (), getDocTitle ());
	}

	public String toString ()
	{
		return "The Default HTML Doclet";
	}
}
