/*
 * (c) Copyright 2006 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookxmldoc.htmldoclet;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.Field;
import java.lang.reflect.Member;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import cookxml.cookxmldoc.interfaces.NameSpaceDoc;
import cookxml.cookxmldoc.interfaces.TagDoc;
import cookxml.cookxmldoc.interfaces.TagLibraryDoc;

/**
 * This class is used to generate HTML link string.
 *
 * @author Heng Yuan
 * @version $Id: LinkManager.java 246 2007-06-07 06:12:54Z coconut $
 * @since CookXmlDoc 0.1
 */
public class LinkManager
{
	/** maintains the package -> html URL relationship. */
	private final Map m_packageMap = new HashMap ();
	/** maintains the class -> tag relationship. */
	private final Map m_class2TagMap = new HashMap ();
	/** maintains the ns -> prefix relationship. */
	private final Map m_prefixMap = new HashMap ();

	private final MessageFormat m_tagLinkFormat;
	private final MessageFormat m_classLinkFormat;

	public LinkManager (MessageFormat classLinkFormat, MessageFormat tagLinkFormat)
	{
		m_classLinkFormat = classLinkFormat;
		m_tagLinkFormat = tagLinkFormat;
	}

	/**
	 * Retrieve "package-list" from the URL given and use the URL as
	 * the base for the JavaDoc of the packages.
	 *
	 * @param	htmlURL
	 *			the URL to find the "package-list" and associated JavaDocs.
	 */
	public void addPackageLink (URL htmlURL)
	{
		try
		{
			addPackageLink (htmlURL, new URL (htmlURL, "package-list"));
		}
		catch (Exception ex)
		{
		}
	}

	/**
	 * Specify package list URL and the corresponding JavaDoc URL.
	 *
	 * @param	htmlURL
	 *			JavaDoc URL
	 * @param	packageListURL
	 *			the URL for the package list
	 */
	public void addPackageLink (URL htmlURL, URL packageListURL)
	{
		InputStream is = null;
		try
		{
			is = packageListURL.openStream ();
			String[] packageList = readPackageList (is);
			is.close ();
			addPackageLink (htmlURL, packageList);
		}
		catch (Exception ex)
		{
			if (is != null)
			{
				try
				{
					is.close ();
				}
				catch (IOException ex2)
				{
				}
			}
		}
	}

	private void addPackageLink (URL url, String[] packageList) throws MalformedURLException
	{
		for (int i = 0; i < packageList.length; ++i)
			m_packageMap.put (packageList[i], new URL (url, packageList[i].replace ('.', '/') + '/'));
	}

	private static String[] readPackageList (InputStream is) throws IOException
	{
		BufferedReader reader = new BufferedReader (new InputStreamReader (is));
		String line;
		Collection list = new LinkedList ();
		while ((line = reader.readLine ()) != null)
		{
			line = line.trim ();
			list.add (line);
		}
		return (String[])list.toArray (new String[0]);
	}

	/**
	 * Obtain the HTML string representing the URL link to the class JavaDoc.
	 *
	 * @param	className
	 * 			the class name
	 * @return	an HTML string
	 */
	public String getClassLink (String className)
	{
		if (className == null || className.length () == 0)
			return "";
		// parse [ to deal with arrays
		// [Ljava.lang.String; = String[]
		// [I = int[]
		int bracketIndex = className.lastIndexOf ('[');
		if (bracketIndex >= 0)
		{
			int index = className.lastIndexOf (';');
			if (index > 0)
				className = className.substring (bracketIndex + 2, index);
			else
				className = className.substring (bracketIndex + 1);
		}

		URL url = getClassURL (className);
		if (url == null)
			return className;

		className = className.replace ('$', '.');
		int index = className.lastIndexOf ('.');
		String shortName = className;
		if (index > 0)
			shortName = className.substring (index + 1);

		String link = m_classLinkFormat.format (new Object[]{ url, shortName, className });
		for (int i = -1; i < bracketIndex; ++i)
			link = link + "[]";
		return link;
	}

	public String getAttrLink (Member member, String attr)
	{
		return m_classLinkFormat.format (new Object[]{ getMemberURL (member), attr, member.getName () });
	}

	public String getMemberLink (String member)
	{
		int pound = member.indexOf ('#');
		if (pound < 0)
			return getClassLink (member);
		int lastDot = member.lastIndexOf ('.', pound);
		if (lastDot < 0)
			lastDot = 0;
		else
			lastDot = lastDot + 1;
		String className = member.substring (lastDot, pound);
		String m = member.substring (pound + 1);
		m = m.replaceAll (" ", "").replaceAll (",", ", ");
		return m_classLinkFormat.format (new Object[]{ getMemberURL (member), className + '.' + m, member.substring (0, pound) + '.' + m });
	}

	public String getMemberLink (Member member)
	{
		URL url = getMemberURL (member);
		return m_classLinkFormat.format (new Object[]{ url, member.getName (), member.getDeclaringClass ().getName () + '.' + member.getName ()});
	}

	public URL getMemberURL (Member member)
	{
		String className = member.getDeclaringClass ().getName ();
		URL url = getClassURL (className);
		if (url == null)
			return null;
		try
		{
			if (member instanceof Field)
				return new URL (url, "#" + member.getName ());
			String name = member.toString ();
			int parenIndex = name.indexOf ('(');
			return new URL (url, "#" + member.getName () + name.substring (parenIndex).replaceAll (",", ", "));
		}
		catch (Exception ex)
		{
			return null;
		}
	}

	public URL getMemberURL (String member)
	{
		int pound = member.indexOf ('#');
		if (pound < 0)
			return getClassURL (member);
		String className = member.substring (0, pound);
		URL url = getClassURL (className);
		if (url == null)
			return null;
		try
		{
			String m = member.substring (pound + 1);
			m = m.replaceAll (" ", "");
			int paren = m.indexOf ('(');
			if (paren < 0)
				return new URL (url, "#" + m.trim ());
			return new URL (url, "#" + m.replaceAll (",", ", "));
		}
		catch (Exception ex)
		{
			return null;
		}
	}


	public URL getClassURL (String className)
	{
		int index = className.lastIndexOf ('.');
		String packageName;
		if (index < 0)
			packageName = "";
		else
			packageName = className.substring (0, index);

		URL url = (URL)m_packageMap.get (packageName);
		if (url == null)
			return null;
		if (index > 0)
			className = className.substring (index + 1);
		className = className.replace ('$', '.');
		try
		{
			return new URL (url, className + ".html");
		}
		catch (Exception ex)
		{
			return null;
		}
	}

	public String parseLinkComment (String ns, String comment)
	{
		if (comment == null)
			return comment;
		StringBuffer strBuffer = null;
		int start;
		int len = comment.length ();
		for (start = 0; start < len;)
		{
			int newStart = comment.indexOf ("{@", start);
			if (newStart < 0)
				break;
			int linkType = -1;
			int linkStart = newStart + 2;
			if (comment.startsWith ("link", linkStart))
			{
				linkType = 0;	// link
				linkStart += 4;
			}
			else if (comment.startsWith ("tag", linkStart))
			{
				linkType = 1;
				linkStart += 3;
			}
			if (linkType < 0)
				break;
			if (strBuffer == null)
				strBuffer = new StringBuffer ();
			strBuffer.append (comment.substring (start, newStart));
			int end = comment.indexOf ('}', linkStart);
			if (end < 0)
				end = len;
			String linkText = comment.substring (linkStart, end).trim ();
			if (linkType == 0)
			{
				strBuffer.append (getMemberLink (linkText));
			}
			else if (linkType == 1)
			{
				strBuffer.append (getTagLink (getPrefix (ns), linkText, false)); 
			}
			start = end + 1;
		}
		if (strBuffer == null)
			return comment;
		if (start < len)
			strBuffer.append (comment.substring (start, len)); 
		return strBuffer.toString ();
	}

	/**
	 * Update the map that maps a class name to a tag.
	 *
	 * @param	tagLibraryDoc
	 *			the doc root
	 */
	public void createClass2TagMap (TagLibraryDoc tagLibraryDoc)
	{
		NameSpaceDoc[] nsDocs = tagLibraryDoc.getNameSpaces ();
		Map map = m_class2TagMap;
		for (int i = 0; i < nsDocs.length; ++i)
		{
			NameSpaceDoc nsDoc = nsDocs[i];
			String ns = nsDoc.getName ();
			String prefix = getPrefix (ns);
			TagDoc[] tagDocs = nsDoc.getTags ();
			for (int j = 0; j < tagDocs.length; ++j)
			{
				TagDoc tagDoc = tagDocs[j];
				String clName = tagDoc.getClassName ();
				if (clName == null)
					continue;
				String tag = tagDoc.getName ();
				map.put (clName, prefix + ":" + tag);
				clName = tagDoc.getHelperClassName ();
				if (clName == null)
					continue;
				map.put (clName, prefix + ":" + tag);
			}
		}
	}

	/**
	 * Get the tag corresponding to the class name.
	 *
	 * @param	className
	 * 			the class name
	 * @return	the tag name
	 */
	public String getTag (String className)
	{
		return (String)m_class2TagMap.get (className);
	}

	public String getTagLink (String thisPrefix, String prefixAndTag, boolean atRoot)
	{
		int index = prefixAndTag.indexOf (':');
		String vNS;
		String vTag;
		if (index >= 0)
		{
			vNS = prefixAndTag.substring (0, index);
			vTag = prefixAndTag.substring (index + 1);
		}
		else
		{
			vNS = thisPrefix;
			vTag = prefixAndTag;
		}

		return m_tagLinkFormat.format (new Object[]{ vNS, vTag, atRoot ? "" : "../" });
	}

	/**
	 * Add a prefix name for namespace.
	 *
	 * @param	ns
	 * 			the name space of the tag library.
	 * @param	prefix
	 * 			the prefix name to be used.
	 */
	public void addPrefix (String ns, String prefix)
	{
		m_prefixMap.put (ns, prefix);
	}

	/**
	 * Obtain the prefix that is used to represent a namespace.  If the namespace
	 * is not in the prefix map.  Then one would be generated for that namespace.
	 *
	 * @param	ns
	 *			the namespace being queried
	 * @return	a non-null and non-empty prefix for the namespace.
	 */
	public String getPrefix (String ns)
	{
		Map prefixMap = m_prefixMap;

		String prefix = (String)prefixMap.get (ns);

		if (prefix != null && prefix.length () > 0)
			return prefix;

		if (ns == null || ns.length () == 0)
			prefix = "_";
		else
		{
			try
			{
				URI uri = new URI (ns);

				String path = uri.getPath ();
				if (path == null || path.length () == 0 || "/".equals (path))
					path = "";
				else
				{
					if (path.startsWith ("/"))
						path = path.substring (1);
					if (path.endsWith ("/"))
						path = path.substring (0, path.length () - 1);
					path = path.replace ('/', '.');
					if (prefixMap.get (path) == null)
					{
						prefix = path;
						prefixMap.put (ns, prefix);
						return prefix;
					}
				}

				String host = uri.getHost ();
				if (host == null)
					host = "";
				else if (path.length () > 0)
					host = host + ".";
				String scheme = uri.getScheme ();
				if (scheme == null || "http".equals (scheme))
					scheme = "";
				else
					scheme = scheme + ".";
				prefix = scheme + host + path;
				if (prefixMap.get (prefix) == null)
				{
					prefixMap.put (ns, prefix);
					return prefix;
				}
			}
			catch (Exception ex)
			{
			}
			prefix = "invalid_ns";
		}
		prefixMap.put (ns, prefix);
		return prefix;
	}
}
