/*
 * (c) Copyright 2006 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookxmldoc.interfaces;

import cookxml.core.interfaces.Creator;

/**
 * This interface is used to hold information of regarding a tag.
 *
 * @author Heng Yuan
 * @since CookXmlDoc 0.1
 */
public interface TagDoc
{
	/**
	 * Get the namespace for the tag.
	 *
	 * @return	the namespace for the tag.
	 */
	public NameSpaceDoc getNameSpace ();

	/**
	 * Get all the tags inherited.
	 *
	 * @return	a non-null array.
	 */
	public TagDoc[] getAncestors ();

	/**
	 * Get all the direct child tags.
	 *
	 * @return	a non-null array.
	 */
	public TagDoc[] getChildren ();

	/**
	 * Obtain the Creator that is used to create the object corresponding
	 * to this tag.  In case of a hidden tag, the creator is null.
	 *
	 * @return	the creator for this tag.
	 */
	public Creator getCreator ();

	/**
	 * Reports whether or not this tag is an internal tag.  An internal
	 * tag does not have a creator and is mainly used for tag inheritance.
	 *
	 * @return	whether or not the tag is an internal tag.
	 */
	public boolean isInternal ();

	/**
	 * Get the tag name.
	 * @return	the tag name.
	 */
	public String getName ();

	/**
	 * Get the Javadoc comment.
	 *
	 * @return	the Javadoc comment.
	 */
	public String getDocComment ();

	/**
	 * Get the comment for the tag.
	 *
	 * @return	the tag comment.
	 */
	public String getTagComment ();

	/**
	 * Get the list of adders associated with the tag.
	 *
	 * @return	a non-null array of adders.
	 */
	public AdderDoc[] getAdders ();

	/**
	 * Get the list of attributes associated with the tag.
	 *
	 * @return	a non-null array of attributes.
	 */
	public AttrDoc[] getAttributes ();

	/**
	 * Get the associated class for this tag.
	 *
	 * @return	the associated class.
	 */
	public String getClassName ();

	/**
	 * Get the associated helper class for this tag.
	 *
	 * @return	the associated helper class.
	 */
	public String getHelperClassName ();

	/**
	 * Get the examples for this tag.
	 *
	 * @return	an array of examples.
	 */
	public String[] getExamples ();
}
