/*
 * (c) Copyright 2006 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.cookxslt.pattern;

import java.io.IOException;
import java.io.StreamTokenizer;
import java.io.StringReader;
import java.util.Collection;
import java.util.LinkedList;

/**
 * @author Heng Yuan
 */
class PatternParser
{
	private static Pattern parsePattern (StreamTokenizer tokenizer) throws IOException
	{
		Pattern pattern = parseLocationPathPattern (tokenizer);
		if (pattern == null)
			throw new IOException ("No pattern");
		if (tokenizer.ttype == StreamTokenizer.TT_EOF)
			return pattern;
		Collection patterns = new LinkedList ();
		patterns.add (pattern);
		while (tokenizer.ttype == '|')
			patterns.add (parseLocationPathPattern (tokenizer));
		if (tokenizer.ttype == StreamTokenizer.TT_EOF)
			return new MultiPattern (patterns);
		throw new IOException ("Unknown token");
	}

	private static Pattern parseLocationPathPattern (StreamTokenizer tokenizer) throws IOException
	{
		tokenizer.nextToken ();

		IdKeyPattern idPattern = null;
		if ("id".equals (tokenizer.sval) || "key".equals (tokenizer.sval))
			idPattern = parseIdKeyPattern (tokenizer);

		int skip = 0;
		if (tokenizer.ttype == '/')
		{
			++skip;
			// check if we have another
			tokenizer.nextToken ();
			if (tokenizer.ttype == '/')
			{
				++skip;
				tokenizer.nextToken ();
			}
		}

		if (idPattern != null)
		{
			idPattern.setSkip (skip);
			return parseRelativePathPattern (tokenizer, idPattern);
		}
		else if (skip == 1)
		{
			if (tokenizer.ttype == StreamTokenizer.TT_EOF)
				return RootPattern.getInstance ();
			return parseRelativePathPattern (tokenizer, RootPattern.getInstance ());
		}
		return parseRelativePathPattern (tokenizer, null);
	}

	private static IdKeyPattern parseIdKeyPattern (StreamTokenizer tokenizer) throws IOException
	{
		if ("id".equals (tokenizer.sval))
		{
			tokenizer.nextToken ();
			if (tokenizer.ttype == '(')
			{
				tokenizer.nextToken ();
				if (tokenizer.ttype == StreamTokenizer.TT_WORD)
				{
					String id = tokenizer.sval;
					tokenizer.nextToken ();
					if (tokenizer.ttype == ')')
					{
						tokenizer.nextToken ();
						return new IdPattern (id);
					}
				}
			}
		}
		else if ("key".equals (tokenizer.sval))
		{
			tokenizer.nextToken ();
			if (tokenizer.ttype == '(')
			{
				tokenizer.nextToken ();
				if (tokenizer.ttype == StreamTokenizer.TT_WORD)
				{
					String key = tokenizer.sval;
					tokenizer.nextToken ();
					if (tokenizer.ttype == ',')
					{
						tokenizer.nextToken ();
						if (tokenizer.ttype == StreamTokenizer.TT_WORD)
						{
							String value = tokenizer.sval;
							if (tokenizer.ttype == ')')
							{
								tokenizer.nextToken ();
								return new KeyPattern (key, value);
							}
						}
					}
				}
			}
		}
		throw new IOException ("Invalid pattern");
	}

	private static Pattern parseRelativePathPattern (StreamTokenizer tokenizer, Pattern prevPattern) throws IOException
	{
		TagPattern pattern = parseStepPattern (tokenizer);

		int skip;

		LinkedList list = null;
		while (tokenizer.ttype == '/')
		{
			skip = 1;
			// check if we have another
			tokenizer.nextToken ();
			if (tokenizer.ttype == '/')
			{
				++skip;
				tokenizer.nextToken ();
			}
			pattern.skip = skip;

			if (list == null)
			{
				list = new LinkedList ();
				list.add (pattern);
			}
			pattern = parseStepPattern (tokenizer);
			list.add (pattern);
		}

		if (prevPattern != null)
		{
			if (list == null)
				list = new LinkedList ();
			list.addFirst (prevPattern);
		}

		if (list == null)
			return pattern;
		else
			return new PathPattern (list);
	}

	private static TagPattern parseStepPattern (StreamTokenizer tokenizer) throws IOException
	{
		char axis = parseAxisSpecifier (tokenizer);
		TagPattern pattern = parseNodeTest (tokenizer);
		pattern.axis = axis;

		if (tokenizer.ttype == '[')
			parsePredicate (tokenizer);
		return pattern;
	}

	private static char parseAxisSpecifier (StreamTokenizer tokenizer) throws IOException
	{
		if (tokenizer.ttype == '@')
		{
			tokenizer.nextToken ();
			return '@';
		}
		if (tokenizer.ttype != StreamTokenizer.TT_WORD)
			return '\0';
		if ("child::".equals (tokenizer.sval))
		{
			tokenizer.nextToken ();
			return 'c';
		}
		if ("attribute::".equals (tokenizer.sval))
		{
			tokenizer.nextToken ();
			return 'a';
		}
		return '\0';
	}

	private static TagPattern parseNodeTest (StreamTokenizer tokenizer) throws IOException
	{
		if (tokenizer.ttype == '*')
		{
			TagPattern pattern = new TagPattern ();
			pattern.name = "*";
			tokenizer.nextToken ();
			return pattern;
		}
		else if (tokenizer.ttype == StreamTokenizer.TT_WORD)
		{
			TagPattern pattern = new TagPattern ();
			pattern.name = tokenizer.sval;
			tokenizer.nextToken ();
			return pattern;
		}
		throw new IOException ("Invalid token type: " + tokenizer.ttype);
	}

	private static Pattern parsePredicate (StreamTokenizer tokenizer) throws IOException
	{
		tokenizer.nextToken ();
		Pattern pattern = parsePredicateExpr (tokenizer);
		if (tokenizer.ttype != ']')
			throw new IOException ("Invalid token type: " + tokenizer.ttype);
		return pattern;
	}

	private static Pattern parsePredicateExpr (StreamTokenizer tokenizer) throws IOException
	{
		return null;
	}

	private static void match (StreamTokenizer tokenizer, String token) throws IOException
	{
		if (tokenizer.ttype != StreamTokenizer.TT_WORD || !token.equals (tokenizer.sval))
			throw new IOException ("Invalid token");
	}

	public static Pattern createPattern (String pattern) throws IOException
	{
		StringReader reader = new StringReader (pattern);
		StreamTokenizer tokenizer = new StreamTokenizer (reader);
		tokenizer.resetSyntax ();
		tokenizer.wordChars ('A', 'Z');
		tokenizer.wordChars ('a', 'z');
		tokenizer.wordChars ('-', '-');
		tokenizer.wordChars ('_', '_');
		tokenizer.wordChars (':', ':');
		tokenizer.slashStarComments (false);
		return parsePattern (tokenizer);
	}

	public static void main (String[] args) throws Exception
	{
		System.out.println (createPattern ("@abc:abc/def//aaa/*"));
		System.out.println (createPattern ("/@abc:abc/def//aaa/*"));
	}
}
