/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.adder;

import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.Set;

import cookxml.core.DecodeEngine;
import cookxml.core.FunctionHandler;
import cookxml.core.doclet.DocletActionReporter;
import cookxml.core.exception.AdderException;
import cookxml.core.interfaces.Adder;
import cookxml.core.interfaces.Handler;
import cookxml.core.interfaces.TagLibrary;

/**
 * This Adder redirect the call to a different function, if the parameters
 * match.
 *
 * @author Heng Yuan
 * @version $Id: CallFunctionAdder.java 265 2007-06-10 18:47:06Z coconut $
 * @since CookXml 1.0
 */
public class CallFunctionAdder implements Adder, DocletActionReporter
{
	private final Method m_method;
	private final String m_funcName;
	private final Class m_childClass;
	private final Class m_parentClass;

	public static Adder getAdder (String funcName, Class parentClass, Class childClass)
	{
		Method method;
		if (parentClass == null ||
			childClass == null ||
			(method = FunctionHandler.getMethod (parentClass, funcName, childClass)) == null)
			return new CallFunctionAdder (funcName, parentClass, childClass);
		try
		{
			method.setAccessible (true);
		}
		catch (Exception ex)
		{
		}
		return new CallFunctionAdder (funcName, method, parentClass, childClass);
	}

	public CallFunctionAdder (String funcName, Class parentClass, Class childClass)
	{
		m_funcName = funcName;
		m_method = null;
		m_childClass = childClass;
		m_parentClass = parentClass;
	}

	private CallFunctionAdder (String funcName, Method method, Class parentClass, Class childClass)
	{
		m_method = method;
		m_funcName = funcName;
		m_childClass = childClass;
		m_parentClass = parentClass;
	}

	public boolean add (String ns, String parentTag, Object parent, Object child, DecodeEngine decodeEngine) throws Exception
	{
		if (parent == null ||
			(m_parentClass != null && !m_parentClass.isInstance (parent)) ||
			(m_childClass != null && !m_childClass.isInstance (child)))
			return false;

		if (m_method != null)
		{
			m_method.invoke (parent, new Object[]{ child });
			return true;
		}
		return add (ns, parentTag, parent, child, decodeEngine, m_parentClass, m_childClass, m_funcName, this);
	}

	public static boolean add (String parentNS, String parentTag, Object parent, Object child, DecodeEngine decodeEngine, Class parentClass, Class childClass, String funcName, Adder adder) throws AdderException
	{
		if (parent == null)
			return false;

		if ((parentClass != null && !parentClass.isInstance (parent)) ||
			(childClass != null && !childClass.isInstance (child)))
			throw new AdderException (decodeEngine, null, adder, parentNS, parentTag, parent, child);

		if (childClass == null)
		{
			if (child != null)
				childClass = child.getClass ();
		}

		try
		{
			Handler handler = decodeEngine.getHandler (parentNS, parentTag, 'f', funcName, childClass);

			// get the handler first.
			// if the handler is not in cache, then we need to find it and then put
			// it into the cache.
			if (handler == null)
			{
				handler = FunctionHandler.getHandler (parent, funcName, childClass);
				if (handler != null)
					decodeEngine.setHandler (parentNS, parentTag, 'f', funcName, childClass, handler);
			}
			if (handler != null)
			{
				handler.invoke (parentNS, parent, child, decodeEngine);
				return true;
			}
		}
		catch (Exception ex)
		{
			throw new AdderException (decodeEngine, ex, adder, parentNS, parentTag, parent, child);
		}
		throw new AdderException (decodeEngine, null, adder, parentNS, parentTag, parent, child);
	}

	public String toString ()
	{
		return super.toString () + "[ function = " + m_funcName + ", parent class = " + m_parentClass + ", child class = " + m_childClass + " ]";
	}

	public Object[] getActions (TagLibrary tagLibrary, String tagNS, String tag, String attrNS, String attr, Class cl)
	{
		if (m_method != null)
			return new Object[]{ m_method };
		if (cl == null)
			return new Object[0];

		String funcName = m_funcName;
		Method[] methods = cl.getMethods ();

		Set set = new HashSet ();
		for (int i = 0; i < methods.length; ++i)
		{
			Method method = methods[i];
			if (!funcName.equals (method.getName ()))
				continue;
			Class[] params = method.getParameterTypes ();
			if (params.length != 1)
				continue;
			set.add (method);
		}

		return set.toArray ();
	}
}
