/*
 * (c) Copyright 2007 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.creator;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import org.w3c.dom.Element;

import cookxml.core.CookXml;
import cookxml.core.DecodeEngine;
import cookxml.core.doclet.DocletActionReporter;
import cookxml.core.exception.CookXmlException;
import cookxml.core.exception.CreatorException;
import cookxml.core.exception.NoFactoryFunctionException;
import cookxml.core.interfaces.Creator;
import cookxml.core.interfaces.TagLibrary;

/**
 * This creator creates a new object by calling a static function.
 *
 * @author Heng Yuan
 * @version $Id: GetInstanceCreator.java 255 2007-06-10 07:08:13Z coconut $
 * @since CookXml 3.1
 */
public class GetInstanceCreator implements Creator, DocletActionReporter
{
	private final static Class[] EMPTY_CLASS_PARAM = new Class[0];
	private final static Object[] EMPTY_OBJECT_PARAM = new Object[0];

	/**
	 * This function used to generate a creator that creates a new object
	 * using the public static function specified.  The resulting object
	 * is added before its setup.
	 *
	 * @param	targetClass the class object
	 * @param	funcName the name of the static function to be called.
	 * @return a creator if the static function can be found.
	 * @throws cookxml.core.exception.CookXmlException in case the function could not be found.
	 */
	public static Creator getCreator (Class targetClass, String funcName) throws CookXmlException
	{
		return getCreator (targetClass, funcName, false);
	}

	/**
	 * This function used to generate a creator that creates a new object
	 * using the public static function specified.
	 *
	 * @param	targetClass the class object
	 * @param	funcName the name of the static function to be called.
	 * @param	addAfterFinish a flag that indicates not to add the object after
	 * its creation, but rather wait until all the setup
	 * has been done.
	 * @return a creator if the static function can be found.
	 * @throws cookxml.core.exception.CookXmlException in case the function could not be found.
	 */
	public static Creator getCreator (Class targetClass, String funcName, boolean addAfterFinish) throws CookXmlException
	{
		Method factory;
		try
		{
			factory = targetClass.getMethod (funcName, EMPTY_CLASS_PARAM);
			int flag = factory.getModifiers ();
			if (Modifier.isStatic (flag))
			{

				boolean access;
				try
				{
					// can make invokations run faster if we have permission to do so.
					factory.setAccessible (true);
					access = true;
				}
				catch (Exception ex)
				{
					access = false;
				}
				if (access || Modifier.isPublic (flag))
					return new GetInstanceCreator (factory, addAfterFinish);
			}
		}
		catch (Throwable t)
		{
		}
		CookXml.getDefaultExceptionHandler ().handleException (null, new NoFactoryFunctionException (targetClass, funcName));
		return null;
	}

	private final Method m_factory;
	private final boolean m_addAfterFinish;

	private GetInstanceCreator (Method factory, boolean addAfterFinish)
	{
		m_factory = factory;
		m_addAfterFinish = addAfterFinish;
	}

	public Object create (String parentNS, String parentTag, Element elm, Object parentObj, DecodeEngine decodeEngine)
	{
		try
		{
			if (m_addAfterFinish)
				decodeEngine.setDoAdd (false);
			return m_factory.invoke (null, EMPTY_OBJECT_PARAM);
		}
		catch (Exception ex)
		{
			throw new CreatorException (decodeEngine, ex, this, parentNS, parentTag, elm, parentObj);
		}
	}

	public Object editFinished (String parentNS, String parentTag, Element elm, Object parentObj, Object obj, DecodeEngine decodeEngine) throws CookXmlException
	{
		if (m_addAfterFinish)
			decodeEngine.addChild (parentNS, parentTag, elm, parentObj, obj);
		return obj;
	}

	public Object[] getActions (TagLibrary tagLibrary, String tagNS, String tag, String attrNS, String attr, Class cl)
	{
		return new Object[]{ m_factory };
	}
}
