/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.creator;

import java.lang.reflect.Constructor;
import java.lang.reflect.Member;

import org.w3c.dom.Element;

import cookxml.core.CookXml;
import cookxml.core.DecodeEngine;
import cookxml.core.doclet.DocletActionReporter;
import cookxml.core.exception.CookXmlException;
import cookxml.core.exception.CreatorException;
import cookxml.core.exception.NoDefaultConstructorException;
import cookxml.core.interfaces.Creator;
import cookxml.core.interfaces.Helper;
import cookxml.core.interfaces.TagLibrary;

/**
 * This creator is for any helper classes that have a default constructor.
 *
 * @author Heng Yuan
 * @version $Id: HelperCreator.java 255 2007-06-10 07:08:13Z coconut $
 * @since CookXml 2.2.1
 */
public class HelperCreator implements Creator, DocletActionReporter
{
	public final static Class[] EMPTY_CLASS_PARAM = new Class[0];
	public final static Object[] EMPTY_OBJECT_PARAM = new Object[0];

	/**
	 * This function used to generate a creator.
	 *
	 * @param	targetClass
	 * 			the class object
	 * @return	a default creator if a default constructor can be found.
	 */
	public static Creator getCreator (Class targetClass) throws CookXmlException
	{
		if (!Helper.class.isAssignableFrom (targetClass))
		{
			CookXml.getDefaultExceptionHandler ().handleException (null, new NoDefaultConstructorException (targetClass));
			return null;
		}

		try
		{
			Constructor ctor = targetClass.getConstructor (EMPTY_CLASS_PARAM);
			return new HelperCreator (ctor);
		}
		catch (Exception ex)
		{
			CookXml.getDefaultExceptionHandler ().handleException (null, new NoDefaultConstructorException (targetClass));
			return null;
		}
	}

	private final Constructor m_ctor;

	private HelperCreator (Constructor ctor)
	{
		m_ctor = ctor;
		try
		{
			// can make invokations run faster if we have permission to do so.
			m_ctor.setAccessible (true);
		}
		catch (Exception ex)
		{
		}
	}

	public Object create (String parentNS, String parentTag, Element elm, Object parentObj, DecodeEngine decodeEngine) throws CreatorException
	{
		try
		{
			decodeEngine.setDoAdd (false);
			return m_ctor.newInstance (EMPTY_OBJECT_PARAM);
		}
		catch (Exception ex)
		{
			throw new CreatorException (decodeEngine, ex, this, parentNS, parentTag, elm, parentObj);
		}
	}

	public Object editFinished (String parentNS, String parentTag, Element elm, Object parentObj, Object obj, DecodeEngine decodeEngine) throws CookXmlException
	{
		if (obj == null)
			return null;

		try
		{
			obj = ((Helper)obj).getFinalObject ();
		}
		catch (Exception ex)
		{
			decodeEngine.handleException (null, new CookXmlException (null, decodeEngine, null, ex));
			return null;
		}
		decodeEngine.addChild (parentNS, parentTag, elm, parentObj, obj);
		return obj;
	}

	public Object[] getActions (TagLibrary tagLibrary, String tagNS, String tag, String attrNS, String attr, Class cl)
	{
		return new Member[]{ m_ctor };
	}
}
