/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.creator;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;

import cookxml.core.DecodeEngine;
import cookxml.core.exception.CreatorException;
import cookxml.core.interfaces.Creator;
import cookxml.core.util.DocumentElement;
import cookxml.core.util.IncludeUtils;

/**
 * This creator is used to implement a creator that is used to include another XML
 * document.
 *
 * @cxdoc
 * This tag is used to load another XML file.  This tag is different from {@tag idref} in that
 * {@tag idref} retrieves the same object of the element referenced, while this tag creates
 * a new object for the element.  The constructor of this tag (ctor) is in the format of
 * "resourcename" or "resourcename#id", or "#id".  Thus, this tag is able to include the
 * entire content of an XML document, or the element of having the specified id, or the
 * current document which have the specified id.  If the XML document is not the current
 * document, the document is parsed using the same DocumentBuilder used to parse the
 * current document.
 * <p>
 * It is necessary to be aware that if the included document also contains include tags,
 * include using local id may not work as expected.
 * <p>
 * By default, if the class loader is not specified, the system class loader is used to
 * lookup the resource.  So the resource is never looked up relative to the current
 * XML document.  An important reason to use ClassLoader to load the resource is to deal
 * with plugin based architecture where the default system ClassLoader is different from
 * the one that loads the plugin jar ({@link java.net.URLClassLoader}).  However, this
 * approach also means that the path is not relative to the xml currently being loaded,
 * but to the ClassLoader being used.  There could be better solutions, but that is what
 * I have so far.
 * <p>
 * Regardless, a good practice is to always specify the absolute path of the XML. If you use
 * <a href="http://proguard.sourceforge.net/">ProGuard</a> to generate an optimized jar,
 * this approach is the way to go.  Also, nice IDEs such as IntelliJ IDEA would also be
 * able to recognize this path and can refactor this path name.
 *
 * @cxexample
 * <codeblock type="xml">
 * 	<include ctor="#id"/>
 *	<include ctor="resource.xml"/>
 * 	<include ctor="resource.xml#ok"/>
 * 	<include ctor="resource.xml#ok" text="OK"/>
 * </codeblock>
 * <p>
 * Assume resource.xml looks like:
 * <p>
 * <codeblock type="xml">
 * 	<panel>
 * 		<button id="ok" text="Okay"/>
 * 	</panel>
 * </codeblock>
 * 
 * @cxattr	ctor
 * 			Specifies the resource location.  The format can be "resourcename",
 *			"resourcename#id", or "#id".
 * @author Heng Yuan
 * @version $Id: IncludeCreator.java 249 2007-06-07 07:18:23Z coconut $
 * @since CookXml 2.0
 */
public class IncludeCreator implements Creator
{
	private final static Creator s_instance = new IncludeCreator ();

	public static Creator getInstance ()
	{
		return s_instance;
	}

	public final static String ID_SEPARATOR = "#";

	public Object create (String parentNS, String parentTag, Element elm, Object parentObj, DecodeEngine decodeEngine) throws CreatorException
	{
		try
		{
			Attr attr = elm.getAttributeNode ("ctor");
			if (attr == null)
				return null;
			DocumentElement docElm = IncludeUtils.getElement (attr.getNodeValue (), decodeEngine);
			if (docElm == null)
				return null;
			decodeEngine.pushData (docElm.doc);
			Object obj = decodeEngine.decodeElement (parentNS, parentTag, docElm.element, parentObj);
			decodeEngine.addCurrentSkipList (attr);
			return obj;
		}
		catch (Exception ex)
		{
			throw new CreatorException (decodeEngine, ex, this, parentNS, parentTag, elm, parentObj);
		}
	}

	public Object editFinished (String parentNS, String parentTag, Element elm, Object parentObj, Object obj, DecodeEngine decodeEngine) throws CreatorException
	{
		decodeEngine.popData ();
		return obj;
	}
}
