/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.creator;

import org.w3c.dom.Element;

import cookxml.core.DecodeEngine;
import cookxml.core.exception.CookXmlException;
import cookxml.core.exception.CreatorException;
import cookxml.core.interfaces.Creator;

/**
 * This creator creates a null object and forces it to be added/set to the parent object.  It is
 * the way (other than hacking the converter) to create a null value.
 *
 * @cxdoc
 * This tag is used to add a null value to the parent.  Usually CookXml ignores null objects
 * since a null object has no properties and cannot have children.
 *
 * @cxexample
 * Intentionally add some null values to a vector:
 * <p>
 * <codeblock type="xml">
 * 	<vector>
 * 		<text>some text</text>
 * 		<null/>
 * 		<null/>
 * 		<text>some text</text>
 * 	<vector>
 * </codeblock>
 *
 * @cxexample
 * A more common use is to set a particular attribute to null in the parent tag.  In the following
 * example, assuming that the foo tag has an attribute named bar.  The following xml sets the
 * value of the bar attribute to null.
 * <p>
 * <codeblock type="xml">
 * 	<foo>
 * 		<null setas="bar"/>
 * 	</foo>
 * </codeblock>
 *
 * @author Heng Yuan
 * @version $Id: NullCreator.java 249 2007-06-07 07:18:23Z coconut $
 * @since CookXml 1.0
 */
public class NullCreator implements Creator
{
	private final static Creator s_instance = new NullCreator ();

	public static Creator getInstance ()
	{
		return s_instance;
	}

	private NullCreator ()
	{
	}

	public Object create (String parentNS, String parentTag, Element elm, Object parentObj, DecodeEngine decodeEngine) throws CreatorException
	{
		try
		{
			decodeEngine.addChild (parentNS, parentTag, elm, parentObj, null);
		}
		catch (CookXmlException ex)
		{
			throw new CreatorException (decodeEngine, ex, this, parentNS, parentTag, elm, parentObj);
		}
		return null;
	}

	public Object editFinished (String parentNS, String parentTag, Element elm, Object parentObj, Object obj, DecodeEngine decodeEngine) throws CookXmlException
	{
		return obj;
	}
}
