/*
 * (c) Copyright 2005 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.exception;

import java.util.HashMap;

import cookxml.core.CookXml;
import cookxml.core.DecodeEngine;

/**
 * This is the base class for all CookXml related exceptions.
 * <p>
 * In order to stay compatible with older versions of CookXml, the base class
 * is set to RuntimeException.  However, for better error checking and correction,
 * it is recommanded that you change it to Exception s.t. you are aware of the
 * exceptions that are generated.
 * <p>
 * Occationally, certain exceptions may be passed to exception handler twice.  You
 * may wish to modify the exception object a bit through set/get property values
 * to make notes.
 *
 * @see RuntimeException
 * @see Exception
 * @since CookXml 2.0
 * @author Heng Yuan
 */
public class CookXmlException extends RuntimeException
{
    public final CookXml cookXml;
	public final DecodeEngine decodeEngine;

	private HashMap m_properties;

	public CookXmlException ()
	{
		this (null, null, null, null);
	}

	public CookXmlException (String msg)
	{
		this (null, null, msg, null);
	}

	public CookXmlException (Throwable cause)
	{
		this (null, null, null, cause);
	}

	public CookXmlException (String msg, Throwable cause)
	{
		this (null, null, msg, cause);
	}

	public CookXmlException (CookXml cookXml)
	{
		this (cookXml, null, null, null);
	}

	public CookXmlException (DecodeEngine decodeEngine)
	{
		this (null, decodeEngine, null, null);
	}

	public CookXmlException (CookXml cookXml, DecodeEngine decodeEngine)
	{
		this (cookXml, decodeEngine, null, null);
	}

	public CookXmlException (CookXml cookXml, DecodeEngine decodeEngine, String msg, Throwable cause)
	{
		super (msg, cause);
		if (cookXml == null)
		{
			if (decodeEngine != null)
				cookXml = decodeEngine.getCookXml ();
		}
		this.cookXml = cookXml;
		this.decodeEngine = decodeEngine;
	}

    public Object setProperty (String key, Object value)
	{
		HashMap properties = m_properties;
		if (properties == null)
		{
			properties = new HashMap ();
			m_properties = properties;
		}
		Object oldValue = properties.get (key);
		properties.put (key, value);
		return oldValue;
	}

	public Object getProperty (String key)
	{
		HashMap properties = m_properties;
		if (properties == null)
			return null;
		return properties.get (key);
	}
}
