/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.interfaces;

import org.w3c.dom.Element;

import cookxml.core.DecodeEngine;

/**
 * This interface handles object creation.
 *
 * @author Heng Yuan
 * @version $Id: Creator.java 218 2007-06-06 06:10:10Z coconut $
 * @since CookXml 1.0
 */
public interface Creator
{
	/**
	 * This function is called whenever decodeEngine starts processing an element.  It should return
	 * an object that corresponds to the tag, or null to indicate no object could be created.
	 * If the object returned is of NoAdd type, then the object is not added to the parent object
	 * by decodeEngine.
	 * <p>
	 * There can be creative use of this function.  For example, one could create another instance
	 * of CookXml that process the element differently, then call this instanceof decodeEngine to
	 * add the processed object to the parent, and then return null to tell this instance of
	 * decodeEngine/CookXml to stop processing the element.
	 *
	 * @param   parentNS
	 *          the parent tag namespace.
	 * @param	parentTag
	 * 			the parent tag that contains this element
	 * @param	elm
	 * 			the DOM element that contains all the information regarding this node.
	 * @param	parentObj
	 * 			the parent object in the node
	 * @param	decodeEngine
	 * 			the decode engine that is being used @throws	CreatorException
	 * @return	the object created, or null to tell decodeEngine to stop processing this node
	 * @throws	Exception
	 * 			If the exception is not a {@link cookxml.core.exception.CreatorException},
	 * 			then it is wrapped inside one by the DecodeEngine and handled by the
	 *			{@link cookxml.core.interfaces.ExceptionHandler}.
	 */
	public Object create (String parentNS, String parentTag, Element elm, Object parentObj, DecodeEngine decodeEngine) throws Exception;

	/**
	 * This function is called when the element and its subnodes have all been processed. and about
	 * to be returned.  For many helper objects that implement NoAdd, this is a great time to generate
	 * the real object and call the decodeEngine.addChild to add the real object to the parent.
	 *
	 * @param   parentNS
	 *          the parent tag namespace.
	 * @param	parentTag
	 * 			the parent element tag
	 * @param	elm
	 * 			the DOM element that contains all the information regarding this node.
	 * @param	parentObj
	 * 			the parent object of the parent element
	 * @param	obj
	 * 			the object that was created by this create function
	 * @param	decodeEngine
	 *			the decode engine that is being used.
	 * @return	the object.  The purpose is to allow a helper object to return the real object.
	 * @throws	Exception
	 * 			If the exception is not a {@link cookxml.core.exception.CookXmlException},
	 * 			then it is wrapped inside one by the DecodeEngine and handled by the
	 *			{@link cookxml.core.interfaces.ExceptionHandler}.
	 */
	public Object editFinished (String parentNS, String parentTag, Element elm, Object parentObj, Object obj, DecodeEngine decodeEngine) throws Exception;
}
