/*
 * (c) Copyright 2005 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.setter;

import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.Set;

import cookxml.core.DecodeEngine;
import cookxml.core.FunctionHandler;
import cookxml.core.doclet.DocletActionReporter;
import cookxml.core.exception.CookXmlException;
import cookxml.core.exception.NoHandlerException;
import cookxml.core.exception.SetterException;
import cookxml.core.interfaces.Handler;
import cookxml.core.interfaces.Setter;
import cookxml.core.interfaces.TagLibrary;

/**
 * This Setter redirect the call to a function, if the parameters match.
 *
 * @author Heng Yuan
 * @version $Id: CallFunctionSetter.java 265 2007-06-10 18:47:06Z coconut $
 * @since CookXml 1.0
 */
public class CallFunctionSetter implements Setter, DocletActionReporter
{
	public static Setter getSetter (String funcName, Class parentClass, Class childClass)
	{
		Method method;
		if (parentClass == null || (method = FunctionHandler.getMethod (parentClass, funcName, childClass)) == null)
			return new CallFunctionSetter (funcName, parentClass, childClass);
		try
		{
			method.setAccessible (true);
		}
		catch (Exception ex)
		{
		}
		return new CallFunctionSetter (funcName, method, parentClass, childClass);
	}

	private final String m_funcName;
	private final Method m_method;
	private final Class m_valueClass;
	private final Class m_objectClass;

	public CallFunctionSetter (String funcName, Class parentClass, Class childClass)
	{
		m_funcName = funcName;
		m_method = null;
		m_valueClass = childClass;
		m_objectClass = parentClass;
	}

	public CallFunctionSetter (String funcName, Method method, Class parentClass, Class childClass)
	{
		m_funcName = funcName;
		m_method = method;
		m_valueClass = childClass;
		m_objectClass = parentClass;
	}

	public void setAttribute (String ns, String tag, String attrNS, String attr, Object obj, Object value, DecodeEngine decodeEngine) throws Exception
	{
		if ((m_objectClass != null && !m_objectClass.isInstance (obj)) ||
			(m_valueClass != null && !m_valueClass.isInstance (value)))
			throw new SetterException (decodeEngine, null, this, ns, tag, attrNS, attr, obj, value);

		if (m_method != null)
			m_method.invoke (obj, new Object[]{ value });
		else
			set (ns, tag, obj, value, decodeEngine, m_objectClass, m_valueClass, m_funcName);
	}

	public static void set (String ns, String tag, Object obj, Object value, DecodeEngine decodeEngine, Class objectClass, Class valueClass, String funcName) throws CookXmlException
	{
		if (obj == null ||
			(objectClass != null && !objectClass.isInstance (obj)) ||
			(valueClass != null && !valueClass.isInstance (value)))
			return;

		if (valueClass == null)
		{
			if (value != null)
				valueClass = value.getClass ();
		}
		Handler handler = decodeEngine.getHandler (ns, tag, 'f', funcName, valueClass);

		// get the handler first.
		// if the handler is not in cache, then we need to find it and then put
		// it into the cache.
		if (handler == null)
		{
			handler = FunctionHandler.getHandler (obj, funcName, valueClass);
			if (handler == null)
				throw new NoHandlerException (decodeEngine, ns, tag, funcName, obj, value);
			decodeEngine.setHandler (ns, tag, 'f', funcName, valueClass, handler);
		}

		handler.invoke (ns, obj, value, decodeEngine);
	}

	public Object[] getActions (TagLibrary tagLibrary, String tagNS, String tag, String attrNS, String attr, Class cl)
	{
		if (m_method != null)
			return new Object[]{ m_method };
		if (cl == null || attr == null)
			return new Object[0];
		Method[] methods = cl.getMethods ();

		String funcName = m_funcName;
		Class valueClass = m_valueClass;
		Set set = new HashSet ();
		for (int i = 0; i < methods.length; ++i)
		{
			Method method = methods[i];
			if (!funcName.equals (method.getName ()))
				continue;
			Class[] params = method.getParameterTypes ();
			if (params.length != 1)
				continue;
			if (params[0].isAssignableFrom (valueClass))
				set.add (method);
		}

		return set.toArray ();
	}
}
