/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.setter;

import java.lang.reflect.Field;

import cookxml.core.DecodeEngine;
import cookxml.core.doclet.DocletActionReporter;
import cookxml.core.interfaces.Setter;
import cookxml.core.interfaces.TagLibrary;

/**
 * This setter is intended to locate a constant inside a class and returns the correct result.
 * If the constant cannot be located, then the DefaultSetter is called on the original input.
 * Thus, this setter can handle situtations where both constant values and arbitrary values
 * can be assigned.
 *
 * @author Heng Yuan
 * @version $Id: ConstantSetter.java 224 2007-06-06 06:57:33Z coconut $
 * @since CookXml 1.0
 */
public class ConstantSetter implements Setter, DocletActionReporter
{
	private Class m_constantClass;
	private Setter m_setter;

	public ConstantSetter (Class constantClass)
	{
		this (constantClass, DefaultSetter.getInstance ());
	}

	public ConstantSetter (Class constantClass, Setter setter)
	{
		m_constantClass = constantClass;
		m_setter = setter;
	}

	public void setAttribute (String ns, String tag, String attrNS, String attr, Object obj, Object value, DecodeEngine decodeEngine) throws Exception
	{
		if (value == null || !(value instanceof String))
			return;
		String str = (String)value;
		if (str.length () == 0)
			return;

		boolean hasField = false;
		Object constVal = null;
		try
		{
			Field field = m_constantClass.getField (str);
			constVal = field.get (null);
			hasField = true;
		}
		catch (Exception ex)
		{
		}
		if (hasField)
			m_setter.setAttribute (ns, tag, attrNS, attr, obj, constVal, decodeEngine);
		else
			DefaultSetter.getInstance ().setAttribute (ns, tag, attrNS, attr, obj, value, decodeEngine);
	}

	public Object[] getActions (TagLibrary tagLibrary, String tagNS, String tag, String attrNS, String attr, Class cl)
	{
		if (m_setter instanceof DocletActionReporter)
			return ((DocletActionReporter)m_setter).getActions (tagLibrary, tagNS, tag, attrNS, attr, cl);
		return new Object[]{ m_setter };
	}
}
