/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.setter;

import java.lang.reflect.Field;
import java.lang.reflect.Member;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.HashSet;
import java.util.Set;

import cookxml.core.DecodeEngine;
import cookxml.core.FunctionHandler;
import cookxml.core.VariableHandler;
import cookxml.core.doclet.DocletActionReporter;
import cookxml.core.doclet.DocletSetter;
import cookxml.core.exception.NoHandlerException;
import cookxml.core.exception.SetterException;
import cookxml.core.interfaces.Handler;
import cookxml.core.interfaces.TagLibrary;

/**
 * This setter attempts to call Java setter functions with the converted attribute values.
 *
 * @author Heng Yuan
 * @version $Id: DefaultSetter.java 265 2007-06-10 18:47:06Z coconut $
 * @since CookXml 1.0
 */
public class DefaultSetter implements DocletSetter, DocletActionReporter
{
	public static String PREFIX = "set";

	private final static DefaultSetter s_instance = new DefaultSetter ();

	public static DefaultSetter getInstance ()
	{
		return s_instance;
	}

	private DefaultSetter ()
	{
	}

	public void setAttribute (String ns, String tag, String attrNS, String attr, Object obj, Object value, DecodeEngine decodeEngine) throws SetterException
	{
		try
		{
			Class valueClass = null;
			if (value != null && !(value instanceof String))	// ignore string class since we may need to do conversions
				valueClass = value.getClass ();

			Handler handler = decodeEngine.getHandler (ns, tag, ' ', attr, valueClass);

			// get the handler first.
			// if the handler is not in cache, then we need to find it and then put
			// it into the cache.
			if (handler == null)
			{
				String funcName = PREFIX + attr;

				handler = FunctionHandler.getHandler (obj, funcName, valueClass);
				if (handler == null)
					handler = VariableHandler.getHandler (obj, attr, valueClass);

				if (handler != null)
					decodeEngine.setHandler (ns, tag, ' ', attr, valueClass, handler);
			}
			//System.out.println ("obj = " + obj.getClass () + ", value = " + value);
			if (handler != null)
			{
				handler.invoke (ns, obj, value, decodeEngine);
				return;
			}
		}
		catch (Exception ex)
		{
			throw new SetterException (decodeEngine, ex, this, ns, tag, attrNS, attr, obj, value);
		}
		throw new SetterException (decodeEngine, new NoHandlerException (decodeEngine, ns, tag, attr, obj, value), this, ns, tag, attrNS, attr, obj, value);
	}

	public String[] getAttributes (Class cl)
	{
		Method[] methods = cl.getMethods ();
		Field[] fields = cl.getFields ();

		Set set = new HashSet ();
		for (int i = 0; i < methods.length; ++i)
		{
			Method method = methods[i];
			if (!method.getName ().startsWith (PREFIX))
				continue;
			Class[] params = method.getParameterTypes ();
			if (params.length != 1)
				continue;
			String attr = method.getName ().substring (PREFIX.length ()).toLowerCase ();
			if (attr.length () > 0)
				set.add (attr);
		}

		for (int i = 0; i < fields.length; ++i)
		{
			Field field = fields[i];
			if (Modifier.isFinal (field.getModifiers ()))
				continue;
			set.add (field.getName ().toLowerCase ());
		}

		return (String[])set.toArray (new String[0]);
	}

	public Object[] getActions (TagLibrary tagLibrary, String tagNS, String tag, String attrNS, String attr, Class cl)
	{
		if (cl == null || attr == null)
			return new Member[0];
		Method[] methods = cl.getMethods ();
		Field[] fields = cl.getFields ();

		Set set = new HashSet ();
		for (int i = 0; i < methods.length; ++i)
		{
			Method method = methods[i];
			if (!method.getName ().startsWith (PREFIX))
				continue;
			Class[] params = method.getParameterTypes ();
			if (params.length != 1)
				continue;
			if (attr.equals (method.getName ().substring (PREFIX.length ()).toLowerCase ()))
				set.add (method);
		}

		for (int i = 0; i < fields.length; ++i)
		{
			Field field = fields[i];
			if (Modifier.isFinal (field.getModifiers ()))
				continue;
			if (attr.equals (field.getName ().toLowerCase ()))
				set.add (field);
		}

		return set.toArray ();
	}
}
