/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.taglibrary;

import java.lang.reflect.Member;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;

import cookxml.core.DecodeEngine;
import cookxml.core.adder.CompoundAdder;
import cookxml.core.doclet.DocletActionReporter;
import cookxml.core.interfaces.Adder;
import cookxml.core.interfaces.Setter;
import cookxml.core.interfaces.TagLibrary;
import cookxml.core.util.DocletUtils;

/**
 * This tag library class avoids the hassele of setting up all the things (adders and setters)
 * for a tag.  It suited perfectly for Swing in which most components inherit JComponent.
 *
 * @author Heng Yuan
 * @version $Id: InheritableTagLibrary.java 254 2007-06-10 06:51:35Z coconut $
 * @since CookXml 1.0
 */
public class InheritableTagLibrary extends SingleNSTagLibrary
{
	private final Map m_inheritMap;

	private class ParentAdder implements Adder, DocletActionReporter
	{
		public boolean add (String ns, String parentTag, Object parent, Object child, DecodeEngine decodeEngine) throws Exception
		{
			LinkedList parentList = getParentTags (ns, parentTag);
			if (parentList == null)
				return false;

			for (Iterator iter = parentList.iterator (); iter.hasNext ();)
			{
				String tag = (String)iter.next ();
				String tagNS;
				int nsIndex = tag.indexOf (':');
				if (nsIndex >= 0)
				{
					tagNS = tag.substring (nsIndex + 1);
					tag = tag.substring (0, nsIndex);
				}
				else
					tagNS = ns;
				Adder adder = getAdder (tagNS, tag);
				if (adder == null)
					continue;
				if (adder.add (ns, tag, parent, child, decodeEngine))
					return true;
			}
			return false;
		}

		public Object[] getActions (TagLibrary tagLibrary, String tagNS, String tag, String attrNS, String attr, Class cl)
		{
			LinkedList parentList = getParentTags (tagNS, tag);
			if (parentList == null)
				return new Member[0];
			LinkedList list = new LinkedList ();
			for (Iterator iter = parentList.iterator (); iter.hasNext ();)
			{
				String thisTag = (String)iter.next ();
				String thisNS;
				int nsIndex = thisTag.indexOf (':');
				if (nsIndex >= 0)
				{
					thisNS = thisTag.substring (nsIndex + 1);
					thisTag = thisTag.substring (0, nsIndex);
				}
				else
					thisNS = tagNS;
				Adder adder = getAdder (thisNS, thisTag);
				if (adder == null)
					continue;
				if (adder instanceof DocletActionReporter)
				{
					Object[] actions = ((DocletActionReporter)adder).getActions (tagLibrary, thisNS, thisTag, null, null, cl);
					for (int i = 0; i < actions.length; ++i)
					{
						if (!list.contains (actions[i]))
							list.add (actions[i]);
					}
				}
				else if (!list.contains (adder))
					list.add (adder);
			}
			return list.toArray ();
		}
	}

	private final Adder m_parentAdder = new ParentAdder ();

	/**
	 * Construct a tag library that doesn't have a parent.
	 */
	public InheritableTagLibrary ()
	{
		this (null);
	}

	/**
	 * Construct a tag library with a parent.
	 *
	 * @param	parentTagLibrary the parent tag library.
	 */
	public InheritableTagLibrary (TagLibrary parentTagLibrary)
	{
		super (parentTagLibrary);
		m_inheritMap = new HashMap ();
	}

	/**
	 * Create a tag inheritance.  Adder and setters are inherited.
	 * <p/>
	 * <em>Cycles must not exist!</em>
	 *
	 * @param	parentNs
	 * 			the optional namespace of the parent tag.  If null, then use the same
	 * 			name space as the childTag.
	 * @param	parentTag
	 * 			the parent tag name
	 * @param	thisTag
	 * 			the child tag name
	 */
	public void inheritTag (String parentNs, String parentTag, String thisTag)
	{
		LinkedList list = (LinkedList)m_inheritMap.get (thisTag);
		if (list == null)
		{
			list = new LinkedList ();
			m_inheritMap.put (thisTag, list);
		}

		String insertStr = (parentNs == null || parentNs.equals (getNameSpace ())) ? parentTag : parentTag + ":" + parentNs;

		if (!list.contains (insertStr))	// simple dup check, but doesn't check cycles
			list.add (insertStr);
	}

	/**
	 * create a tag inheritance.  Adder and setters are inherited.
	 * <em>Cycles must not exist!</em>
	 *
	 * @param	parentTag
	 * 			the parent tag name
	 * @param	thisTag
	 * 			the child tag name
	 */
	public void inheritTag (String parentTag, String thisTag)
	{
		inheritTag (null, parentTag, thisTag);
	}

	public Setter getSetter (String ns, String tag, String attrNS, String attr)
	{
		Setter setter = super.getSetter (ns, tag, attrNS, attr);
		if (setter != null)
			return setter;

		if (tag == null)
			tag = "";
		if (attr == null)
			attr = "";
		LinkedList parentList = getParentTags (tag);
		if (parentList == null)
			return null;

		for (Iterator iter = parentList.iterator (); iter.hasNext ();)
		{
			String parentTag = (String)iter.next ();
			String tagNS;
			int nsIndex = parentTag.indexOf (':');
			if (nsIndex >= 0)
			{
				tagNS = parentTag.substring (nsIndex + 1);
				parentTag = parentTag.substring (0, nsIndex);
			}
			else
				tagNS = ns;

			setter = getSetter (tagNS, parentTag, attrNS, attr);
			if (setter != null)
				return setter;
		}
		return null;
	}

	public Adder getAdder (String ns, String tag)
	{
		Adder adder = super.getAdder (ns, tag);
		if (adder != null)
			return adder;

		// we can't just return m_parentAdder here since no adder
		// means the DecodeEngine can try to use the DefaultAdder if possible.

		// in this case, we need to deal with parent adders.
		// we dynamically generate one in case of multiple inheritance.
		if (tag == null)
			tag = "";
		LinkedList parentList = getParentTags (tag);
		if (parentList == null || parentList.size () == 0)
			return null;

		for (Iterator iter = parentList.iterator (); iter.hasNext ();)
		{
			String parentTag = (String)iter.next ();
			String tagNS;
			int nsIndex = parentTag.indexOf (':');
			if (nsIndex >= 0)
			{
				tagNS = parentTag.substring (nsIndex + 1);
				parentTag = parentTag.substring (0, nsIndex);
			}
			else
				tagNS = ns;
			Adder newAdder = getAdder (tagNS, parentTag);
			if (newAdder == null)
				continue;
			if (adder == null)
				adder = newAdder;
			else
			{
				CompoundAdder tmpAdder = new CompoundAdder ();
				tmpAdder.add (adder);
				tmpAdder.add (newAdder);
				adder = tmpAdder;
			}
		}
		return adder;
	}

	/**
	 * Instead of creating an adder that does it all, this function internally
	 * creates a compound adder that combine the functionalities of individual
	 * specialized adders.  The adder added later has precedence over adders
	 * added earlier.
	 *
	 * @param	tag
	 * 			the tag name.
	 * @param	adder
	 * 			the individual Adder.
	 */
	public void addAdder (String tag, Adder adder)
	{
		if (tag == null)
			tag = "";
		Adder oldAdder = super.getAdder (getNameSpace (), tag);
		if (oldAdder == null)
			oldAdder = m_parentAdder;

		if (oldAdder instanceof CompoundAdder)
		{
			((CompoundAdder)oldAdder).add (adder);
		}
		else
		{
			super.setAdder (tag, CompoundAdder.getAdder (oldAdder, adder));
		}
	}

	/**
	 * Retrieve the direct parent tags.  If the parent tag is in a different namespace,
	 * the entry will be in tag:ns format.
	 *
	 * @param	tag
	 * 			the tag name
	 * @return	direct
	 * 			parent tags
	 */
	public LinkedList getParentTags (String tag)
	{
		return (LinkedList)m_inheritMap.get (tag);
	}

	/**
	 * This for documentation purpose to retrieve the direct parent tags.
	 * <p/>
	 * If the parent tag is in a different namespace, the entry will be in
	 * tag:ns format.
	 *
	 * @param	ns
	 * 			the namespace of the tag.
	 * @param	tag
	 * 			the tag name
	 * @return	direct parent tags
	 */
	public LinkedList getParentTags (String ns, String tag)
	{
		String thisNS = getNameSpace ();
		if (thisNS == null || ns == null || thisNS.equals (ns))
		{
			LinkedList list = (LinkedList)m_inheritMap.get (tag);
			if (list != null)
				return list;
		}
		TagLibrary tagLibrary = getParent ();
		if (tagLibrary == null || !(tagLibrary instanceof InheritableTagLibrary))
			return null;
		return ((InheritableTagLibrary)tagLibrary).getParentTags (ns, tag);
	}

	public Map getAttributes (String ns, String tag)
	{
		Map map = super.getAttributes (ns, tag);

		// try to see if a tag has parent tags
		LinkedList parentTags = getParentTags (ns, tag);
		if (parentTags == null)
			return map;

		// hmm, in those cases, we need to merge possible tags.
		for (Iterator iter = parentTags.iterator (); iter.hasNext ();)
		{
			String parentTag = (String)iter.next ();
			String parentNS = ns;
			int index = parentTag.indexOf (':');
			if (index >= 0)
			{
				parentNS = parentTag.substring (index + 1);
				parentTag = parentTag.substring (0, index);
			}
			map = DocletUtils.mergeMap (map, getAttributes (parentNS, parentTag));
		}

		return map;
	}
}
