/*
 * (c) Copyright 2006 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.taglibrary;

import java.util.*;

import cookxml.core.MultiSpecialCreator;
import cookxml.core.exception.UnknownNameSpaceException;
import cookxml.core.interfaces.*;

/**
 * This tag library pools several SingleNSTagLibrary together to serve as a single
 * TagLibrary.
 *
 * @author Heng Yuan
 * @version $Id: MultiNSTagLibrary.java 226 2007-06-06 07:01:13Z coconut $
 * @since CookXml 3.0
 */
public class MultiNSTagLibrary implements TagLibrary
{
	private final TagLibrary m_parent;

	private final Map m_tagLibraryMap = new HashMap ();

	public MultiNSTagLibrary ()
	{
		this (null);
	}

	public MultiNSTagLibrary (TagLibrary parentTagLibrary)
	{
		m_parent = parentTagLibrary;
	}

	public void addTagLibrary (String ns, SingleNSTagLibrary tagLibrary)
	{
		m_tagLibraryMap.put (ns, tagLibrary);
	}

	public SingleNSTagLibrary getTagLibrary (String ns)
	{
		return (SingleNSTagLibrary)m_tagLibraryMap.get (ns);
	}

	public SpecialCreator getSpecialCreator ()
	{
		List list = new LinkedList ();
		SpecialCreator creator;
		for (Iterator iter = m_tagLibraryMap.values ().iterator (); iter.hasNext ();)
		{
			TagLibrary tagLibrary = (TagLibrary)iter.next ();
			creator = tagLibrary.getSpecialCreator ();
			if (creator != null && !list.contains (creator))
				list.add (creator);
		}
		if (m_parent != null)
		{
			creator = m_parent.getSpecialCreator ();
			if (creator != null && !list.contains (creator))
				list.add (creator);
		}
		int size = list.size ();
		if (size == 0)
			return null;
		if (size == 1)
			return (SpecialCreator)list.get (0);
		SpecialCreator[] specialCreators = new SpecialCreator[size];
		list.toArray (specialCreators);
		return new MultiSpecialCreator (specialCreators);
	}

	public Creator getCreator (String ns, String tag)
	{
		TagLibrary tagLib = (TagLibrary)m_tagLibraryMap.get (ns);
		if (tagLib == null)
		{
			tagLib = m_parent;
			if (tagLib == null)
				throw new UnknownNameSpaceException (ns);
		}
		return tagLib.getCreator (ns, tag);
	}

	public Setter getSetter (String ns, String tag, String attrNS, String attr)
	{
		TagLibrary tagLib = (TagLibrary)m_tagLibraryMap.get (ns);
		if (tagLib == null)
		{
			tagLib = m_parent;
			if (tagLib == null)
				throw new UnknownNameSpaceException (ns);
		}
		return tagLib.getSetter (ns, tag, attrNS, attr);
	}

	public Adder getAdder (String ns, String tag)
	{
		TagLibrary tagLib = (TagLibrary)m_tagLibraryMap.get (ns);
		if (tagLib == null)
		{
			tagLib = m_parent;
			if (tagLib == null)
				throw new UnknownNameSpaceException (ns);
		}
		return tagLib.getAdder (ns, tag);
	}

	public Converter getConverter (String ns, Class targetClass)
	{
		TagLibrary tagLib = (TagLibrary)m_tagLibraryMap.get (ns);
		if (tagLib == null)
		{
			tagLib = m_parent;
			if (tagLib == null)
				throw new UnknownNameSpaceException (ns);
		}
		return tagLib.getConverter (ns, targetClass);
	}
}
