/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.taglibrary;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import cookxml.core.doclet.DocletTagLibrary;
import cookxml.core.interfaces.*;
import cookxml.core.util.DocletUtils;

/**
 * This class by itself can only add creators, adders, setters, and converters for
 * a single namespace.  Hence the name SingleNSTagLibrary.
 * <p>
 * However, this library can inherit tag libraries of different namespaces.  If the
 * namespace of the element is null or the same as the this tag library's namespace,
 * then this tag library is used.  Otherwise, the parent tag library, if exists, is
 * used for the lookup.  Using parent-hiearchy is one way to create a tag library
 * that can handle multiple namespaces.
 * <p>
 * This class was the TagLibrary class in CookXml 2.0.
 *
 * @author Heng Yuan
 * @version $Id: SingleNSTagLibrary.java 255 2007-06-10 07:08:13Z coconut $
 * @since CookXml 1.0
 */
public class SingleNSTagLibrary implements DocletTagLibrary
{
	private final TagLibrary m_parent;

	private final Map m_creatorMap = new HashMap ();
	private final Map m_converterMap = new HashMap ();
	private final Map m_adderMap = new HashMap ();
	private final Map m_setterMap = new HashMap ();

	/**
	 * The namespace of the tag library.
	 *
	 * @since CookXml 3.0
	 */
	private String m_ns;

	/**
	 * The default constructor that creates an instance of SingleNSTagLibrary
	 * without a parent.
	 */
	public SingleNSTagLibrary ()
	{
		this (null);
	}

	/**
	 * The constructor that creates an instance of SingleNSTagLibrary
	 * with a parent tag library.
	 * <p/>
	 * Note that the namespace for the parent and child tag library can
	 * be different.
	 *
	 * @param parentTagLibrary the parent tag library.
	 */
	public SingleNSTagLibrary (TagLibrary parentTagLibrary)
	{
		m_parent = parentTagLibrary;
	}

	/**
	 * Set the namespace of the tag library.
	 *
	 * @param	ns
	 * 			the namespace of the tag library.
	 * @since CookXml 3.0
	 */
	public void setNameSpace (String ns)
	{
		m_ns = ns;
	}

	/**
	 * Get the namespace of the tag library.
	 *
	 * @return	the namespace of the tag library.
	 * @since CookXml 3.0
	 */
	public String getNameSpace ()
	{
		return m_ns;
	}

	public SpecialCreator getSpecialCreator ()
	{
		if (m_parent != null)
			return m_parent.getSpecialCreator ();
		return null;
	}

	public Creator getCreator (String ns, String tag)
	{
		if (ns != null && !ns.equals (m_ns))
		{
			if (m_parent == null)
				return null;
			else
				return m_parent.getCreator (ns, tag);
		}

		Creator creator = (Creator)m_creatorMap.get (tag);
		if (creator == null && m_parent != null)
			return m_parent.getCreator (ns, tag);
		return creator;
	}

	public Setter getSetter (String ns, String tag, String attrNS, String attr)
	{
		if (ns != null && !ns.equals (m_ns))
		{
			if (m_parent == null)
				return null;
			else
				return m_parent.getSetter (ns, tag, attrNS, attr);
		}

		if (tag == null)
			tag = "";
		if (attr == null)
			attr = "";
		Setter setter = (Setter)m_setterMap.get (tag + "/" + attr);
		if (setter == null && m_parent != null)
			return m_parent.getSetter (ns, tag, attrNS, attr);
		return setter;
	}

	public Adder getAdder (String ns, String tag)
	{
		if (ns != null && !ns.equals (m_ns))
		{
			if (m_parent == null)
				return null;
			else
				return m_parent.getAdder (ns, tag);
		}

		if (tag == null)
			tag = "";
		Adder adder = (Adder)m_adderMap.get (tag);
		if (adder == null && m_parent != null)
			return m_parent.getAdder (ns, tag);
		return adder;
	}

	public Converter getConverter (String ns, Class targetClass)
	{
		if (ns != null && !ns.equals (m_ns))
		{
			if (m_parent == null)
				return null;
			else
				return m_parent.getConverter (ns, targetClass);
		}

		Converter converter = (Converter)m_converterMap.get (targetClass);
		if (converter == null && m_parent != null)
		{
			// use null here to indicate that the namespace is the same as this
			// library or not specified to help find a suitable converter in
			// the parent tag library.
			return m_parent.getConverter (null, targetClass);
		}
		return converter;
	}

	/**
	 * Insert an object factory (Creator) for a tag into the tag library.
	 *
	 * @param	tag
	 * 			the tag name.
	 * @param	creator
	 * 			the Creator object.
	 */
	public void setCreator (String tag, Creator creator)
	{
		m_creatorMap.put (tag, creator);
	}

	/**
	 * Insert a setter delegate for a tag and the attribute into the tag library.
	 *
	 * @param	tag
	 * 			the tag name.
	 * @param	attr
	 * 			the tag attribute.
	 * @param	setter
	 * 			the Setter object.
	 */
	public void setSetter (String tag, String attr, Setter setter)
	{
		if (tag == null)
			tag = "";
		if (attr == null)
			attr = "";
		m_setterMap.put (tag + "/" + attr, setter);
	}

	/**
	 * Insert the adder for a tag into the tag library.  Note that, this function
	 * will associate only one and only one adder with the tag.
	 *
	 * @param	tag
	 * 			the tag name.
	 * @param	adder
	 * 			the Adder object.
	 */
	public void setAdder (String tag, Adder adder)
	{
		if (tag == null)
			tag = "";
		m_adderMap.put (tag, adder);
	}

	/**
	 * Insert a class converter into the tag library.
	 *
	 * @param targetClass the tag name.
	 * @param converter   the Converter object.
	 */
	public void setConverter (Class targetClass, Converter converter)
	{
		m_converterMap.put (targetClass, converter);
	}

	public TagLibrary getParent ()
	{
		return m_parent;
	}

	public String[] getNameSpaces ()
	{
		if (m_ns == null)
			return new String[1];

		Collection list = new LinkedList ();

		if (m_parent != null && m_parent instanceof DocletTagLibrary)
		{
			String[] ns = ((DocletTagLibrary)m_parent).getNameSpaces ();
			for (int i = 0; i < ns.length; ++i)
				list.add (ns[i]);
		}

		if (!list.contains (m_ns))
			list.add (m_ns);

		return (String[])list.toArray (new String[list.size ()]);
	}

	public Map getTags ()
	{
		HashMap map = new HashMap ();
		map.put (m_ns, m_creatorMap.keySet ().toArray (new String[m_creatorMap.size ()]));

		if (m_parent != null && m_parent instanceof DocletTagLibrary)
		{
			Map parentMap = ((DocletTagLibrary)m_parent).getTags ();
			return DocletUtils.mergeMap (map, parentMap);
		}
		return map;
	}

	public Map getAttributes (String ns, String tag)
	{
		if (m_ns != null && !m_ns.equals (ns))
		{
			if (m_parent != null && m_parent instanceof DocletTagLibrary)
				return ((DocletTagLibrary)m_parent).getAttributes (ns, tag);
			return new HashMap ();
		}

		HashMap map = new HashMap ();
		if (tag == null)
			tag = "";
		Collection attrList = new LinkedList ();
		String[] allKeys = (String[])m_setterMap.keySet ().toArray (new String[m_setterMap.size ()]);
		String prefix = tag + "/";
		for (int i = 0; i < allKeys.length; ++i)
		{
			String key = allKeys[i];
			if (!key.startsWith (prefix))
				continue;
			String attr = key.substring (prefix.length ());
			attrList.add (attr);
		}
		map.put (m_ns, attrList.toArray (new String[attrList.size ()]));

		if (m_parent != null && m_parent instanceof DocletTagLibrary)
		{
			Map parentMap = ((DocletTagLibrary)m_parent).getAttributes (ns, tag);
			return DocletUtils.mergeMap (map, parentMap);
		}
		return map;
	}

	public Map getConverters (String ns)
	{
		if (m_ns != null && !m_ns.equals (ns))
		{
			if (m_parent != null && m_parent instanceof DocletTagLibrary)
				return ((DocletTagLibrary)m_parent).getConverters (ns);
			return new HashMap ();
		}

		if (m_parent != null && m_parent instanceof DocletTagLibrary)
		{
			Map parentMap = new HashMap (((DocletTagLibrary)m_parent).getConverters (ns));
			parentMap.putAll (m_converterMap);
			return parentMap;
		}
		return m_converterMap;
	}

}
