/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.util;

import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import cookxml.core.exception.ArrayException;

/**
 * A collection of reflection/class utilities.
 *
 * @author Heng Yuan
 * @version $Id: ClassUtils.java 227 2007-06-06 07:04:07Z coconut $
 * @since CookXml 1.0
 */
public class ClassUtils
{
	/**
	 * This map is to map primitive value class to its corresponding object class type.
	 * it is very necessary since otherwise our converters for the object class
	 * type would not be able to work for the primitve value class type.
	 */
	private final static Map PRIMITIVE_CLASS_MAP = new HashMap ();

	/**
	 * This map is used to construct arrays of primitive type.
	 */
	private final static Map PRIMITIVE_ARRAY_CTOR_MAP = new HashMap ();

	public final static String THIS_VAR = "this";
	public final static String NULL_VAR = "null";

	static
	{
		// initiate the primitive class map
		Map classMap = PRIMITIVE_CLASS_MAP;
		classMap.put ("boolean", Boolean.class);
		classMap.put ("byte", Byte.class);
		classMap.put ("char", Character.class);
		classMap.put ("short", Short.class);
		classMap.put ("int", Integer.class);
		classMap.put ("long", Long.class);
		classMap.put ("float", Float.class);
		classMap.put ("double", Double.class);

		Map ctorMap = PRIMITIVE_ARRAY_CTOR_MAP;
		ctorMap.put ("boolean", new ArrayConstructor ()
		{
			public Object create (int length)
			{
				return new boolean[length];
			}
		});
		ctorMap.put ("byte", new ArrayConstructor ()
		{
			public Object create (int length)
			{
				return new byte[length];
			}
		});
		ctorMap.put ("char", new ArrayConstructor ()
		{
			public Object create (int length)
			{
				return new char[length];
			}
		});
		ctorMap.put ("short", new ArrayConstructor ()
		{
			public Object create (int length)
			{
				return new short[length];
			}
		});
		ctorMap.put ("int", new ArrayConstructor ()
		{
			public Object create (int length)
			{
				return new int[length];
			}
		});
		ctorMap.put ("long", new ArrayConstructor ()
		{
			public Object create (int length)
			{
				return new long[length];
			}
		});
		ctorMap.put ("float", new ArrayConstructor ()
		{
			public Object create (int length)
			{
				return new float[length];
			}
		});
		ctorMap.put ("double", new ArrayConstructor ()
		{
			public Object create (int length)
			{
				return new double[length];
			}
		});
	}

	public static Class getEquivalentClass (Class srcClass)
	{
		Class newClass = (Class) PRIMITIVE_CLASS_MAP.get (srcClass.getName ());
		if (newClass == null)
			return srcClass;
		return newClass;
	}

	public static Class getEquivalentClass (String srcClassName)
	{
		return (Class) PRIMITIVE_CLASS_MAP.get (srcClassName);
	}

	public static Object getArray (String name, int length) throws ArrayException
	{
		ArrayConstructor ctor = (ArrayConstructor) PRIMITIVE_ARRAY_CTOR_MAP.get (name);
		if (ctor != null)
			return ctor.create (length);
		try
		{
			return Array.newInstance (Class.forName (name), length);
		}
		catch (Exception ex)
		{
			throw new ArrayException (null, ex, name, length);
		}
	}

	public static Object getArray (Class cl, int length) throws ArrayException
	{
		try
		{
			return Array.newInstance (cl, length);
		}
		catch (Exception ex)
		{
			throw new ArrayException (null, ex, "" + cl, length);
		}
	}

	public static void setArray (Object array, int size, Collection collection)
	{
		Iterator iter = collection.iterator ();
		for (int i = 0; i < size; ++i)
		{
			Array.set (array, i, iter.next ());
		}
	}

	public static Object getField (Object obj, String fieldName) throws NoSuchFieldException, IllegalAccessException
	{
		return getField (obj, fieldName, false);
	}

	public static Object getField (Object obj, String fieldName, boolean accessible) throws NoSuchFieldException, IllegalAccessException, SecurityException
	{
		if (fieldName == null || fieldName.length () == 0)
			return null;
		if (THIS_VAR.equals (fieldName))
			return obj;
		Field field = obj.getClass ().getDeclaredField (fieldName);
		if (field == null)
			return null;
		if (accessible)
			field.setAccessible (true);
		return field.get (obj);
	}

	public static void setField (Object obj, String fieldName, Object value) throws NoSuchFieldException, IllegalAccessException
	{
		setField (obj, fieldName, value, false);
	}

	public static void setField (Object obj, String fieldName, Object value, boolean accessible) throws NoSuchFieldException, IllegalAccessException, SecurityException
	{
		if (fieldName == null || fieldName.length () == 0)
			return;
		Field field = obj.getClass ().getDeclaredField (fieldName);
		if (field == null)
			return;
		if (accessible)
			field.setAccessible (true);
		field.set (obj, value);
	}

	public static Object getConstantValue (Class targetClass, String fieldName)
	{
		try
		{
			Field field = targetClass.getField (fieldName);
			return field.get (null);
		}
		catch (Exception ex)
		{
			return null;
		}
	}
}
