/*
 * (c) Copyright 2004 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.util;

import java.io.File;
import java.io.InputStream;
import javax.xml.parsers.DocumentBuilder;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import cookxml.core.CookXml;
import cookxml.core.DecodeEngine;
import cookxml.core.creator.IncludeCreator;

/**
 * @author Heng Yuan
 * @version $Id: IncludeUtils.java 255 2007-06-10 07:08:13Z coconut $
 * @since CookXml 2.0
 */
public class IncludeUtils
{
	/**
	 * This function is used to find the element to be included.
	 *
	 * @param	includeStr
	 * 			the "pointer" to the element to be added.
	 * @param	decodeEngine
	 * 			the DecodeEngine that is being used.
	 * @return	the element retrieved.
	 */
	public static DocumentElement getElement (String includeStr, DecodeEngine decodeEngine) throws Exception
	{
		return getElement (includeStr, decodeEngine.getCookXml ().getClassLoader (), decodeEngine.getDocument (), decodeEngine.getCookXml ().getDocumentBuilder ());
	}

	/**
	 * This function is used to find the element to be included.
	 *
	 * @param	includeStr
	 * 			the "pointer" to the element to be added.
	 * @param	builder
	 * 			the XML parser.
	 * @param	docRoot
	 * 			the root of the current document.
	 * @return	the element retrieved.
	 */
	public static DocumentElement getElement (String includeStr, Document docRoot, DocumentBuilder builder) throws Exception
	{
		return getElement (includeStr, CookXml.getDefaultClassLoader (), docRoot, builder);
	}

	/**
	 * This function is used to find the element to be included.
	 *
	 * @param	includeStr
	 * 			the "pointer" to the element to be added.
	 * @param	builder
	 * 			the XML parser.
	 * @param	docRoot
	 * 			the root of the current document.
	 * @return	the element retrieved.
	 */
	public static DocumentElement getElement (String includeStr, ClassLoader classLoader, Document docRoot, DocumentBuilder builder) throws Exception
	{
		if (includeStr == null || includeStr.length () == 0)
			return null;
		String id = null;
		int idPos = includeStr.lastIndexOf (IncludeCreator.ID_SEPARATOR);
		if (idPos >= 0)
		{
			id = includeStr.substring (idPos + 1);
			includeStr = includeStr.substring (0, idPos);
		}
		Document doc;
		if (includeStr.length () == 0)
			doc = docRoot;
		else
		{
			InputStream inputStream = classLoader.getResourceAsStream (includeStr);
			if (inputStream == null)
			{
				try
				{
					doc = builder.parse (new File (includeStr));
				}
				catch (Exception ex)
				{
					doc = builder.parse (includeStr);
				}
			}
			else
				doc = builder.parse (inputStream);
		}
		if (doc == null)
			return null;
		if (id == null || id.length () == 0)
			return new DocumentElement (doc, doc.getDocumentElement ());
		return new DocumentElement (doc, getElementById (doc.getDocumentElement (), id));
	}

	/**
	 * This function retrieves the element in the document that has the given id.
	 * It seems to be a known problem for parsing XML documents without DTD.  So, this
	 * is a workaround.
	 *
	 * @param	rootElm
	 * 			sthe current root
	 * @param	id
	 * 			the id
	 * @return	the first element found.
	 */
	public static Element getElementById (Element rootElm, String id)
	{
		if (id.equals (rootElm.getAttribute ("id")))
			return rootElm;
		NodeList childNodes = rootElm.getChildNodes ();
		int size = childNodes.getLength ();
		for (int i = 0; i < size; ++i)
		{
			Node node = childNodes.item (i);
			if (node instanceof Element)
			{
				Element elm = getElementById ((Element)node, id);
				if (elm != null)
					return elm;
			}
		}
		return null;
	}
}
