/*
 * (c) Copyright 2007 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package cookxml.core.util;

import java.io.*;

import org.w3c.dom.CharacterData;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import cookxml.core.CookXml;

/**
 * Useful functions dealing with texts.
 *
 * @author Heng Yuan
 * @revision $Id: TextUtils.java 228 2007-06-06 07:05:09Z coconut $
 * @since CookXml 3.1
 */
public class TextUtils
{
	/**
	 * This function reads from the file/resource using the ClassLoader provided.
	 *
	 * @see		ClassLoader
	 * @param	input
	 *			the input file/resource location
	 * @param   classLoader
	 *          the ClassLoader that is used to locate the resource
	 * @return	the text read.
	 * @throws	java.io.IOException
	 * 			in case of error.
	 */
	public static String readText (String input, ClassLoader classLoader) throws IOException
	{
		InputStream is = classLoader.getResourceAsStream (input);
		if (is == null)
			is = new FileInputStream (input);
		BufferedReader reader = new BufferedReader (new InputStreamReader (is));
		StringBuffer strBuffer = new StringBuffer ();
		String str;
		while ((str = reader.readLine ()) != null)
			strBuffer.append (str).append ("\n");
		reader.close ();
		return strBuffer.toString ();
	}

	/**
	 * This function reads from the file/resource using the default class loader for
	 * CookXml.
	 *
	 * @see		cookxml.core.CookXml#getDefaultClassLoader()
	 *
	 * @param	input
	 *			the input file/resource location
	 * @return	the text read.
	 * @throws	java.io.IOException
	 * 			in case of error.
	 */
	public static String readText (String input) throws IOException
	{
		return readText (input, CookXml.getDefaultClassLoader ());
	}

	/**
	 * Get the text of an element.
	 *
	 * @param   elm
	 *          the element that contains the text.
	 * @return  the text in the element.
	 */
	public static String getText (Element elm)
	{
		StringBuffer buffer = null;
		String text = null;

		for (Node node = elm.getFirstChild (); node != null; node = node.getNextSibling ())
		{
			if (node instanceof CharacterData)
			{
				if (text == null)
					text = ((CharacterData)node).getData ();
				else if (buffer == null)
				{
					buffer = new StringBuffer (text);
					buffer.append (((CharacterData)node).getData ());
				}
				else
					buffer.append (((CharacterData)node).getData ());
			}
		}

		if (text == null)
			return null;
		if (buffer == null)
			return text;
		return buffer.toString ();
	}
}
