/*
 * (c) Copyright 2005 by Heng Yuan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package examples.util;

import java.io.IOException;

import cookxml.core.util.TextUtils;

/**
 * This is my simple implementation of converting an XML source code to HTML.
 * @author Heng Yuan
 */
public class XmlToHtmlConverter
{
	public final static String STYLE_SHEET = "<style type=\"text/css\"><!--" +
											 ".tagsymbol { color: #7f0000 } " +
											 ".tagname { color: #00007f } " +
											 ".tagattr { color: blue } " +
											 ".tagvalue { color: green } " +
											 ".comment { color: gray } " +
											 "--></style>";

	public final static String BODY_START = "<body><pre>";
	public final static String BODY_END = "</pre></body>";

    public final static String TAG_SYMBOL_START = "<span class=\"tagsymbol\">";
	public final static String TAG_SYMBOL_END = "</span>";

	public final static String TAG_NAME_START = "<span class=\"tagname\">";
	public final static String TAG_NAME_END = "</span>";

	public final static String TAG_ATTR_START = "<span class=\"tagattr\">";
	public final static String TAG_ATTR_END = "</span>";

	public final static String TAG_VALUE_START = "<span class=\"tagvalue\">";
	public final static String TAG_VALUE_END = "</span>";

	//public final static String TEXT_START = "<span class=\"text\">";
	//public final static String TEXT_END = "</span>";
	public final static String TEXT_START = "";
	public final static String TEXT_END = "";

	public final static String COMMENT_START = "<span class=\"comment\">";
	public final static String COMMENT_END = "</span>";

    private static String getTitleString (String title)
	{
		return "<title>" + title + "</title>";
	}

	private static String getHeader (String title)
	{
		return "<head>" + getTitleString (title) + STYLE_SHEET + "</head>";
	}

	private static int parseWhiteSpace (String xmlCode, int startPos, int size, StringBuffer htmlBuffer) throws Exception
	{
		if (startPos == size)
			return size;

        int i;
		for (i = startPos; i < size; ++i)
		{
			if (!Character.isWhitespace (xmlCode.charAt (i)))
				break;
		}

		if (i > startPos)
		{
			htmlBuffer.append (TEXT_START);
			htmlBuffer.append (xmlCode.substring (startPos, i));
			htmlBuffer.append (TEXT_END);
		}
		return i;
	}

    private static int parseTagName (String xmlCode, int startPos, int size, StringBuffer htmlBuffer) throws Exception
	{
		if (!Character.isLetter (xmlCode.charAt (startPos)))
			throw new Exception ();

        int i;
		for (i = startPos; i < size; ++i)
		{
			char ch = xmlCode.charAt (i);
            if (Character.isWhitespace (ch) ||
				"/?<&>=\\\"".indexOf (ch) >= 0)
				break;
		}
		if (i > startPos)
		{
			htmlBuffer.append (TAG_NAME_START);
			htmlBuffer.append (xmlCode.substring (startPos, i));
			htmlBuffer.append (TAG_NAME_END);
		}
		return i;
	}

	private static int parseAttrName (String xmlCode, int startPos, int size, StringBuffer htmlBuffer) throws Exception
	{
		if (!Character.isLetter (xmlCode.charAt (startPos)))
			throw new Exception ();

		int i;
		for (i = startPos; i < size; ++i)
		{
			char ch = xmlCode.charAt (i);
			if (Character.isWhitespace (ch) ||
				"/?<&>=\\\"".indexOf (ch) >= 0)
				break;
		}
		if (i > startPos)
		{
			htmlBuffer.append (TAG_ATTR_START);
			htmlBuffer.append (xmlCode.substring (startPos, i));
			htmlBuffer.append (TAG_ATTR_END);
		}
		return i;
	}

	private static int parseValue (String xmlCode, int startPos, int size, StringBuffer htmlBuffer) throws Exception
	{
		if (xmlCode.charAt (startPos) != '"')
			throw new Exception ();

		for (int i = startPos + 1; i < size; ++i)
		{
			char ch = xmlCode.charAt (i);
			if (ch == '\\')
			{
				++i;
				continue;
			}
			if (ch != '"')
				continue;

			int endPos = i + 1;
			htmlBuffer.append (TAG_VALUE_START);
			printText (xmlCode.substring (startPos, endPos), htmlBuffer);
			htmlBuffer.append (TAG_VALUE_END);

			return endPos;
		}
		throw new Exception ();
	}

	private static int parseAttribute (String xmlCode, int startPos, int size, StringBuffer htmlBuffer) throws Exception
	{
		if (startPos == size)
			return startPos;
		if (!Character.isLetter (xmlCode.charAt (startPos)))
			return startPos;

		startPos = parseAttrName (xmlCode, startPos, size, htmlBuffer);
		startPos = parseWhiteSpace (xmlCode, startPos, size, htmlBuffer);

		if (xmlCode.charAt (startPos) != '=')
			throw new Exception ();
		htmlBuffer.append (TAG_SYMBOL_START);
		htmlBuffer.append ('=');
		htmlBuffer.append (TAG_SYMBOL_END);
        ++startPos;
		startPos = parseWhiteSpace (xmlCode, startPos, size, htmlBuffer);
		startPos = parseValue (xmlCode, startPos, size, htmlBuffer);
		return startPos;
	}

	private static int parseAttributes (String xmlCode, int startPos, int size, StringBuffer htmlBuffer) throws Exception
	{
		for (;;)
		{
			int oldPos = startPos;
			startPos = parseAttribute (xmlCode, startPos, size, htmlBuffer);
			startPos = parseWhiteSpace (xmlCode, startPos, size, htmlBuffer);
			if (oldPos == startPos)
				break;
		}
		return startPos;
	}

	private static int parseXmlHeader (String xmlCode, int startPos, int size, StringBuffer htmlBuffer) throws Exception
	{
		if ((size - startPos) < 2)
			return startPos;
		if (!"<?".equals (xmlCode.substring (startPos, startPos + 2)))
			return startPos;

		htmlBuffer.append (TAG_SYMBOL_START);
		htmlBuffer.append ("&lt;?");
		htmlBuffer.append (TAG_SYMBOL_END);

		startPos += 2;

		startPos = parseWhiteSpace (xmlCode, startPos, size, htmlBuffer);
		startPos = parseTagName (xmlCode, startPos, size, htmlBuffer);

		startPos = parseAttributes (xmlCode, startPos, size, htmlBuffer);

		int endPos = startPos + 2;

		if (!"?>".equals (xmlCode.substring (startPos, endPos)))
			throw new Exception ();

		htmlBuffer.append (TAG_SYMBOL_START);
		htmlBuffer.append ("?>");
		htmlBuffer.append (TAG_SYMBOL_END);

		return endPos;
	}

	private static void printComment (String comment, StringBuffer htmlBuffer) throws Exception
	{
		int pos = 0;
		int size = comment.length ();

		for (;;)
		{
			if (pos == size)
				return;
			int index1 = comment.indexOf ('<', pos);
			int index2 = comment.indexOf ('&', pos);

			if (index1 >= 0 && ((index2 < 0) || ((index2 >= 0) && (index1 < index2))))
			{
				htmlBuffer.append (comment.substring (pos, index1));
				htmlBuffer.append ("&lt;");
				pos = index1 + 1;
			}
			else if (index2 >= 0 && ((index1 < 0) || ((index1 >= 0) && (index2 < index1))))
			{
				htmlBuffer.append (comment.substring (pos, index2));
				htmlBuffer.append ("&amp;");
				pos = index2 + 1;
			}
			else
			{
				htmlBuffer.append (comment.substring (pos));
				return;
			}
		}
	}

	private static void printText (String comment, StringBuffer htmlBuffer) throws Exception
	{
		int pos = 0;
		int size = comment.length ();

		for (;;)
		{
			if (pos == size)
				return;
			int index = comment.indexOf ('&', pos);

			if (index >= 0)
			{
				htmlBuffer.append (comment.substring (pos, index));
				htmlBuffer.append ("&amp;");
				pos = index + 1;
			}
			else
			{
				htmlBuffer.append (comment.substring (pos));
				return;
			}
		}
	}

	private static int parseComment (String xmlCode, int startPos, int size, StringBuffer htmlBuffer) throws Exception
	{
		if ((size - startPos) < 4)
			return startPos;
		if (!"<!--".equals (xmlCode.substring (startPos, startPos + 4)))
			return startPos;
		htmlBuffer.append (COMMENT_START);
		htmlBuffer.append ("&lt;");

		int index = xmlCode.indexOf ("-->", startPos);
		if (index < 0)
			throw new Exception ();
		int endPos = index + 3;
		printComment (xmlCode.substring (startPos + 1, endPos), htmlBuffer);
		htmlBuffer.append (COMMENT_END);
		return endPos;
	}

	private static int parseText (String xmlCode, int startPos, int size, StringBuffer htmlBuffer) throws Exception
	{
		int i;
		for (i = startPos; i < size; ++i)
		{
			if (xmlCode.charAt (i) == '<')
				break;
		}
		if (i > startPos)
		{
			htmlBuffer.append (TEXT_START);
			printText (xmlCode.substring (startPos, i), htmlBuffer);
			htmlBuffer.append (TEXT_END);
		}
		return i;
	}

	private static int parseElement (String xmlCode, int startPos, int size, StringBuffer htmlBuffer) throws Exception
	{
		if (startPos == size)
			return startPos;
		if (xmlCode.charAt (startPos) != '<')
			return startPos;
		if (xmlCode.charAt (startPos + 1) == '/')
			return startPos;
		htmlBuffer.append (TAG_SYMBOL_START);
		htmlBuffer.append ("&lt;");
		htmlBuffer.append (TAG_SYMBOL_END);
		++startPos;

		startPos = parseWhiteSpace (xmlCode, startPos, size, htmlBuffer);
		startPos = parseTagName (xmlCode, startPos, size, htmlBuffer);
		startPos = parseAttributes (xmlCode, startPos, size, htmlBuffer);

		if (xmlCode.charAt (startPos) == '/')
		{
			if (xmlCode.charAt (startPos + 1) == '>')
			{
				htmlBuffer.append (TAG_SYMBOL_START);
				htmlBuffer.append ("/>");
				htmlBuffer.append (TAG_SYMBOL_END);
				return startPos + 2;
			}
			throw new Exception ();
		}
		if (xmlCode.charAt (startPos) != '>')
			throw new Exception ();
		htmlBuffer.append (TAG_SYMBOL_START);
		htmlBuffer.append ('>');
		htmlBuffer.append (TAG_SYMBOL_END);
		++startPos;

        for (;;)
		{
			int oldPos = startPos;
			startPos = parseText (xmlCode, startPos, size, htmlBuffer);
			startPos = parseComment (xmlCode, startPos, size, htmlBuffer);
			startPos = parseElement (xmlCode, startPos, size, htmlBuffer);
			if (oldPos == startPos)
				break;
		}
		try
		{
			if (!"</".equals (xmlCode.substring (startPos, startPos + 2)))
				throw new Exception ();
		}
		catch (Exception ex)
		{
			throw new Exception ();
		}
		htmlBuffer.append (TAG_SYMBOL_START);
		htmlBuffer.append ("&lt;/");
		htmlBuffer.append (TAG_SYMBOL_END);
		startPos += 2;

		startPos = parseWhiteSpace (xmlCode, startPos, size, htmlBuffer);
		startPos = parseTagName (xmlCode, startPos, size, htmlBuffer);
		startPos = parseWhiteSpace (xmlCode, startPos, size, htmlBuffer);
		if (xmlCode.charAt (startPos) != '>')
			throw new Exception ();
		htmlBuffer.append (TAG_SYMBOL_START);
		htmlBuffer.append (">");
		htmlBuffer.append (TAG_SYMBOL_END);
		return startPos + 1;
	}

	private static void parseXml (String xmlCode, int startPos, int size, StringBuffer htmlBuffer) throws Exception
	{
		startPos = parseWhiteSpace (xmlCode, startPos, size, htmlBuffer);
		startPos = parseXmlHeader (xmlCode, startPos, size, htmlBuffer);

		for (;;)
		{
			int oldPos = startPos;
			startPos = parseWhiteSpace (xmlCode, startPos, size, htmlBuffer);
			startPos = parseComment (xmlCode, startPos, size, htmlBuffer);
			if (oldPos == startPos)
				break;
		}

		startPos = parseElement (xmlCode, startPos, size, htmlBuffer);

		for (;;)
		{
			int oldPos = startPos;
			startPos = parseWhiteSpace (xmlCode, startPos, size, htmlBuffer);
			startPos = parseComment (xmlCode, startPos, size, htmlBuffer);
			if (oldPos == startPos)
				break;
		}
		if (startPos != size)
			throw new Exception ();
	}

	public static String convert (String title, String xmlCode) throws IOException
	{
		try
		{
			if (xmlCode == null)
				return null;

			StringBuffer htmlBuffer = new StringBuffer ();
			htmlBuffer.append ("<html>");
			htmlBuffer.append (getHeader (title));
			htmlBuffer.append (BODY_START);

			parseXml (xmlCode, 0, xmlCode.length (), htmlBuffer);

			htmlBuffer.append (BODY_END);
			htmlBuffer.append ("</html>");
			return htmlBuffer.toString ();
		}
		catch (Exception ex)
		{
			throw new IOException ("" + ex);
		}
	}

	public static String convert (String xmlCode) throws IOException
	{
		try
		{
			if (xmlCode == null)
				return null;

			StringBuffer htmlBuffer = new StringBuffer ();
			htmlBuffer.append ("<pre>");
			parseXml (xmlCode, 0, xmlCode.length (), htmlBuffer);
			htmlBuffer.append ("</pre>");
			return htmlBuffer.toString ();
		}
		catch (Exception ex)
		{
			throw new IOException ("" + ex);
		}
	}

    public static void main (String[] args) throws Exception
	{
		if (args.length != 1)
			return;

		String xmlCode = TextUtils.readText (args[0]);
		System.out.println (convert (args[0], xmlCode));
	}
}
